#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 安装包名称，安装目录定义
PACKAGE=beszel-0.10.2.tgz

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 获取服务端安装信息
listIniKeysValues monitor environment
if [ "$MONITOR" ]; then
    # beszel 安装路径
    BESZEL_BASE=/usr/local

    # 创建服务用户
    SERVICE_USER=beszel
    if ! id "$SERVICE_USER" &>/dev/null; then
        useradd -M -s /bin/false "$SERVICE_USER" || _err "创建用户失败 ."
    fi

    # 安装服务
    if [ "$MONITOR" == "$LAN_IP_ADDRESS" ] && [ ! -d "$BESZEL_BASE/beszel" ] ; then
        _info "安装 beszel-hub ."
        tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C $BESZEL_BASE beszel/beszel
        mv -f $BESZEL_BASE/beszel/beszel-hub.service /etc/systemd/system/
        chown -R "$SERVICE_USER:$SERVICE_USER" $BESZEL_BASE/beszel

        systemctl daemon-reload
        systemctl enable beszel-hub.service &>/dev/null
        systemctl restart beszel-hub.service
        _log "启动服务 beszel-hub.service ."

        # 配置监控端
        until $BESZEL_BASE/beszel/beszel health --url http://localhost:8090 &>/dev/null; do
            _info "等待 beszel-hub 启动就绪 ."
            sleep 2
        done
        sleep 3
        
        # 初始化账号
        curl -s 'http://127.0.0.1:8090/api/beszel/create-user' \
             -H 'Content-Type: application/json' \
             --data-raw '{"email":"it@haocang.com","password":"zaq1@WSX"}' &>/dev/null

        # 获取登录 token
        TOKEN=$(curl -s 'http://127.0.0.1:8090/api/collections/_superusers/auth-with-password' \
                     -H 'Content-Type: application/json' \
                     --data-raw '{"identity":"it@haocang.com","password":"zaq1@WSX"}' | grep -oP '"token":\s*"\K[^"]*')

        # 配置邮件服务器信息
        PUB_IP=$(awk '/IP/{print $NF}' < <(curl -s www.cip.cc))
        curl -s 'http://127.0.0.1:8090/api/settings' \
             -X 'PATCH' \
             -H "Authorization: $TOKEN" \
             -H 'Content-Type: application/json' \
             --data-raw "{\"meta\":{\"appName\":\"Beszel-HC-${PUB_IP}\",\"appURL\":\"http://${LAN_IP_ADDRESS:-localhost}:8090\",\"senderName\":\"xiongjunfeng@haocang.com\",\"senderAddress\":\"xiongjunfeng@haocang.com\",\"hideControls\":true},\"smtp\":{\"enabled\":true,\"port\":465,\"host\":\"smtphz.qiye.163.com\",\"username\":\"xiongjunfeng@haocang.com\",\"authMethod\":\"PLAIN\",\"tls\":true,\"localName\":\"\",\"password\":\"15YPw2KGNQULGGQM\"}}" &>/dev/null

        # 获取 接口
        collectionId=$(curl -s 'http://127.0.0.1:8090/api/collections/_superusers/auth-refresh' -X 'POST'  -H 'Content-Type: application/json' -H "Authorization: $TOKEN" | grep -oP '"collectionId":\s*"\K[^"]*')

        # 测试邮件
        curl -s 'http://127.0.0.1:8090/api/settings/test/email' \
             -H "Authorization: $TOKEN" \
             -H 'Content-Type: application/json' \
             --data-raw "{\"email\":\"xiongjunfeng@haocang.com\",\"template\":\"verification\",\"collection\":\"$collectionId\"}" &>/dev/null

        # 通过配置文件添加资源
        echo 'systems:' > $BESZEL_BASE/beszel/beszel_data/config.yml
        let SER_LEN=${#SERVERS[@]}-1
        for ((i=0;i<=$SER_LEN;i++)); do
            cat <<-EOF >> $BESZEL_BASE/beszel/beszel_data/config.yml
			    - name: ${HOSTS[i]}   # vm-node${ip##*.}
			      host: ${SERVERS[i]}
			      port: 45876
			      users:
			        - it@haocang.com
			EOF
        done

        systemctl restart beszel-hub

        until $BESZEL_BASE/beszel/beszel health --url http://localhost:8090 &>/dev/null; do
            _info "等待 beszel-hub 重启就绪 ."
            sleep 2
        done
        sleep 3

        # 认证信息
        LOGIN_INFO='{"identity":"it@haocang.com","password":"zaq1@WSX"}'

        # 获取登录 token
        TOKEN=$(curl -s 'http://127.0.0.1:8090/api/collections/users/auth-with-password' -H 'Content-Type: application/json' --data-raw "$LOGIN_INFO" | grep -oP '"token":\s*"\K[^"]*')

        # 批量获取主机ID
        IDS=$(curl -s 'http://127.0.0.1:8090/api/collections/systems/records?page=1&perPage=500&skipTotal=1&sort=+name&fields=id' -H "Authorization: $TOKEN" -H 'Content-Type: application/json' | grep -oP '"id":\s*"\K[^"]*')

        # 获取用户 id
        USER=$(curl -s 'http://127.0.0.1:8090/api/collections/users/auth-with-password' -H 'Content-Type: application/json' --data-raw "$LOGIN_INFO" | grep -oP '"id":\s*"\K[^"]*')

        # 监控接口地址
        URL='http://127.0.0.1:8090/api/collections/alerts/records'

        # 批量启用监控
        _info "配置基础资源告警策略 ."
        for id in $IDS; do
            curl -s "$URL" -H 'Content-Type: application/json' -H "Authorization: $TOKEN" --data-raw "{\"system\":\"$id\",\"user\":\"$USER\",\"name\":\"Status\",\"value\":0,\"min\":10}" &>/dev/null
            curl -s "$URL" -H 'Content-Type: application/json' -H "Authorization: $TOKEN" --data-raw "{\"system\":\"$id\",\"user\":\"$USER\",\"name\":\"CPU\",\"value\":80,\"min\":10}" &>/dev/null
            curl -s "$URL" -H 'Content-Type: application/json' -H "Authorization: $TOKEN" --data-raw "{\"system\":\"$id\",\"user\":\"$USER\",\"name\":\"Memory\",\"value\":80,\"min\":10}" &>/dev/null
            curl -s "$URL" -H 'Content-Type: application/json' -H "Authorization: $TOKEN" --data-raw "{\"system\":\"$id\",\"user\":\"$USER\",\"name\":\"Disk\",\"value\":80,\"min\":10}" &>/dev/null
            curl -s "$URL" -H 'Content-Type: application/json' -H "Authorization: $TOKEN" --data-raw "{\"system\":\"$id\",\"user\":\"$USER\",\"name\":\"Bandwidth\",\"value\":80,\"min\":10}" &>/dev/null
            curl -s "$URL" -H 'Content-Type: application/json' -H "Authorization: $TOKEN" --data-raw "{\"system\":\"$id\",\"user\":\"$USER\",\"name\":\"Temperature\",\"value\":80,\"min\":10}" &>/dev/null
        done

        # 模拟登录激活告警配置
        curl -s 'http://127.0.0.1:8090/api/collections/user_settings/records' -H "Authorization: $TOKEN" -H 'Content-Type: application/json' --data-raw "{\"user\":\"$USER\"}" &>/dev/null

        # 获取接口名称
        id=$(curl -s 'http://127.0.0.1:8090/api/collections/user_settings/records?page=1&perPage=1&filter=&skipTotal=1&fields=id' -H "Authorization: $TOKEN" -H 'Content-Type: application/json' | grep -oP '"id":\s*"\K[^"]*')

        # 配置告警接收邮箱
        curl -s "http://127.0.0.1:8090/api/collections/user_settings/records/$id" -X 'PATCH' -H "Authorization: $TOKEN" -H 'Content-Type: application/json' --data-raw '{"settings":{"chartTime":"1h","emails":["it@haocang.com","xiongjunfeng@haocang.com"],"webhooks":[]}}' &>/dev/null
        _log "配置邮件告警接收地址: ["it@haocang.com","xiongjunfeng@haocang.com"] ."
    fi

    if [ ! -d "$BESZEL_BASE/beszel-agent" ]; then
        _info "安装 beszel-agent ."
        tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C $BESZEL_BASE beszel/beszel-agent
        mv -f $BESZEL_BASE/beszel-agent/beszel-agent.service /etc/systemd/system/
        chown -R "$SERVICE_USER:$SERVICE_USER" $BESZEL_BASE/beszel-agent

        systemctl daemon-reload
        systemctl enable beszel-agent.service &>/dev/null
        systemctl restart beszel-agent.service
        _log "启动服务 beszel-agent.service ."
    fi
fi



## nginx 代理
# server {
# 	listen 80;
# 	server_name beszel.example.com;
# 	client_max_body_size 10M;

# 	location /base-path {
# 		# 查看 http://nginx.org/en/docs/http/ngx_http_upstream_module.html#keepalive
# 		proxy_set_header Connection '';
# 		proxy_http_version 1.1;
# 		proxy_read_timeout 360s;

# 		proxy_set_header Host $host;
# 		proxy_set_header X-Real-IP $remote_addr;
# 		proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
# 		proxy_set_header X-Forwarded-Proto $scheme;

# 		rewrite /base-path/(.*) /$1  break;

# 		proxy_pass http://127.0.0.1:8090;
# 	}
# }