#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
 [ -f ]
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 如果当前主机在 SERVERS 数组，配置 hosts 解析
if [ "$(grep -Fx "$LAN_IP_ADDRESS" < <(sed 's/ /\n/g' <<<${SERVERS[@]}))" ]; then
    _log "配置 hosts 解析 ."
    sed -i '3,$d' /etc/hosts
    echo -e "\n# mango  $(date +'%F %T')" >> /etc/hosts
    let SER_LEN=${#SERVERS[@]}-1
    for ((i=0;i<=$SER_LEN;i++)); do
        echo "${SERVERS[i]}  ${HOSTS[i]}" >> /etc/hosts
    done
else
    _err "当前主机 IP 不存在于安装列表中 ."
    exit 1
fi

# 设置主机名
let SER_LEN=${#SERVERS[@]}-1
for ((i=0;i<=$SER_LEN;i++)); do
    if [ "${SERVERS[i]}" == "$LAN_IP_ADDRESS" ]; then
        hostnamectl set-hostname ${HOSTS[i]}
        _log "设置本机主机名为: ${HOSTS[i]} ."
    fi
done

# 设置时区
timedatectl set-timezone Asia/Shanghai
_log "设置系统时区为 Asia/Shanghai ."

# 优化 ssh 连接速度, 重启 sshd 服务
sed -i "s/#UseDNS yes/UseDNS no/" /etc/ssh/sshd_config
sed -i "s/GSSAPIAuthentication .*/GSSAPIAuthentication no/" /etc/ssh/sshd_config
systemctl restart sshd
_log "优化 ssh 连接速度, 重启 sshd 服务 ."

# 关闭/禁用 selinux.
setenforce 0 &> /dev/null
sed -i 's/SELINUX=enforcing/SELINUX=disabled/g' /etc/selinux/config
_log "关闭/禁用 selinux ."

# 启用防火墙
# systemctl start firewalld
# systemctl enable firewalld &> /dev/null
# _log "启用防火墙 ."

# # 配置允许 ${SERVERS[@]} 访问本机所有 ipv4 端口
# for ipadd in ${SERVERS[@]}; do
#     firewall-cmd --add-rich-rule "rule family='ipv4' source address=$ipadd accept" &>/dev/null
#     firewall-cmd --add-rich-rule "rule family='ipv4' source address=$ipadd accept" --permanent &>/dev/null
# done
# _log "配置防火墙策略 ."

systemctl start firewalld
systemctl enable firewalld &> /dev/null
_log "启用防火墙 ."

# 为每个服务器IP添加防火墙规则
for SERVER in ${SERVERS[@]} $WIN_SERVER; do
    [ "$SERVER" == "$LAN_IP_ADDRESS" ] && continue
    firewall-cmd --add-rich-rule="rule family='ipv4' source address='${SERVER/ /}' accept" --permanent &>/dev/null
    [ $? -eq 0 ] && _log "配置防火墙规则允许 ${SERVER/ /} 访问当前节点所有 ." || _err "为 $SERVER 配置防火墙规则失败 ."
done

# 重新加载防火墙使规则生效
firewall-cmd --reload &>/dev/null

chmod +x /etc/rc.d/rc.local
sed -i '15,$d' /etc/rc.d/rc.local
_log "重置 /etc/rc.d/rc.local ."

# 配置 文件/进程 限制
cat > /etc/security/limits.d/hc-limits.conf <<EOF
# mango  $(date '+%F %T')
* soft  nofile  1000000
* hard  nofile  1000000
* soft  nproc   1000000
* hard  nproc   1000000
* soft  memlock unlimited
* hard  memlock unlimited
EOF
_log "配置 文件/进程/内存 limits ."

# 配置内核参数
cat > /etc/sysctl.d/hc-sysctl.conf <<EOF
# mango  $(date '+%F %T')
fs.file-max                            = 9223372036854775807
fs.inotify.max_user_watches            = 2000000000
kernel.msgmnb                          = 65536
kernel.msgmax                          = 65536
net.core.somaxconn                     = 32768
net.core.rmem_default                  = 12697600
net.core.wmem_default                  = 12697600
net.core.rmem_max                      = 873800000
net.core.wmem_max                      = 655360000
net.ipv4.conf.all.rp_filter            = 0
net.ipv4.conf.default.arp_announce     = 2
net.ipv4.conf.lo.arp_announce          = 2
net.ipv4.conf.all.arp_announce         = 2
net.ipv4.ip_forward                    = 1
net.ipv4.ip_local_port_range           = 32768 60999
net.ipv4.neigh.default.gc_stale_time   = 120
net.ipv4.icmp_echo_ignore_all          = 0
net.ipv4.tcp_fin_timeout               = 30
net.ipv4.tcp_max_orphans               = 3276800
net.ipv4.tcp_max_syn_backlog           = 100000
net.ipv4.tcp_max_tw_buckets            = 65535
net.ipv4.tcp_rmem                      = 8192 262144 4096000
net.ipv4.tcp_syncookies                = 1
net.ipv4.tcp_synack_retries            = 2
net.ipv4.tcp_tw_reuse                  = 1
net.ipv4.tcp_wmem                      = 4096 262144 4096000
net.ipv6.conf.all.disable_ipv6         = 1
net.ipv6.conf.default.disable_ipv6     = 1
net.ipv6.conf.lo.disable_ipv6          = 1
vm.swappiness                          = 70
vm.max_map_count                       = 2000000
EOF

# 立即生效
sysctl --system > /dev/null 2>&1
_log "优化内核参数 ."

cat > /etc/profile.d/histsize.sh <<'EOF'
HISTSIZE="100000"
HISTTIMEFORMAT="[%F %T] ~~~ "

PS1='[\[\e[01;32m\]\u\[\e[00m\]@\[\e[01;33m\]\h\[\e[00m\] \[\e[01;34m\]\W\[\e[00m\]]\$ '
export SYSTEMD_PAGER=""
export HISTSIZE HISTTIMEFORMAT PS1
EOF
_log "优化命令历史记录，终端样式 ."


# 配置软件源加速
sed -e 's|^mirrorlist=|#mirrorlist=|g' \
    -e "s|^#baseurl=http://dl.rockylinux.org/\$contentdir|baseurl=$BASE_REPO_URL|g" \
    -i $(find /etc/yum.repos.d -name *.repo)
_log "配置默认软件源加速 ."

# 安装 epel 源
dnf install -y epel-release &>/dev/null
_log "安装 epel 软件源 ."

# 配置 epel 源加速
sed -e 's|^metalink=|#metalink=|g' \
    -e 's|^#baseurl=|baseurl=|g' \
    -e "s|https://download.fedoraproject.org/pub|$EPEL_REPO_URL|g" \
    -e "s|https://download.example/pub|$EPEL_REPO_URL|g" \
    -e 's|http://mirrors|https://mirrors|g' \
    -i /etc/yum.repos.d/epel{,-testing}.repo
_log "配置 epel 软件源 ."

# 安装软件
green_echo ">>> 安装常用软件包 ."
for pkg in tar wget nmap-ncat net-tools logrotate unzip lrzsz bash-completion ncdu chrony; do
    if rpm -q $pkg --quiet; then
        _info "已安装 $pkg ."
    else
        true && _log "安装 $pkg ."
        dnf install -y $pkg &>/dev/null
    fi
done


## 配置 chrony
# 备份原配置文件
green_echo ">>> 配置主机时间同步 ."
if [ -f /etc/chrony.conf.default ]; then
    cat /etc/chrony.conf.default > /etc/chrony.conf
else
    cat /etc/chrony.conf > /etc/chrony.conf.default
fi

# 微服务第一个节点配置为服务端，其他节点配置为客户端
if [ "$LAN_IP_ADDRESS" == "${WFW[0]}" ]; then
    ## 服务端
    # 配置 NTP 服务器
    sed -i '/^#\?pool /d; /^#\?server /d' /etc/chrony.conf
    i=2
    NTP_SERVER="ntp.aliyun.com ntp.tencent.com cn.pool.ntp.org time.ustc.edu.cn ntp.ntsc.ac.cn"
    for server in $NTP_SERVER; do
        sed -i "${i}a server $server iburst" /etc/chrony.conf
        let i++
    done

    # 修改服务端配置
    sed -i "s@^#\?allow .*@allow 0.0.0.0/0@"  /etc/chrony.conf
    sed -i "s@^#\?local stratum .*@local stratum 10@" /etc/chrony.conf
    _log "chronyd 服务已配置. 上游地址为 [${NTP_SERVER}] ."
else
    ## 客户端
    # 配置 NTP 服务器
    sed -i '/^#\?pool /d; /^#\?server /d' /etc/chrony.conf
    sed -i "2a server ${WFW[0]} iburst" /etc/chrony.conf
    _log "chronyd 服务已配置. 上游地址为 [${WFW[0]}] ."
fi

# 启动 chrony 服务
systemctl enable chronyd &>/dev/null
systemctl restart chronyd
_log "启动 chrony 时间同步服务."

# 立即强制同步
chronyc makestep &> /dev/null

# 手动触发同步
chronyc tracking &> /dev/null

# 查看同步状态
#chronyc sources -v
_log "chrony 时间同步服务已配置."

# 创建软件安装路径/软件安装记录
mkdir -p $SOFT_INSTALL_DIR $INSTALL_STATUS_DIR

# 环境变量配置
[ -f /etc/profile.d/hc-env.sh ] || echo "# mango  $(date '+%F %T')" > /etc/profile.d/hc-env.sh