#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称，安装目录定义
PACKAGE=zookeeper-${ZOOKEEPER_VER}.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
ZK_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${ZK_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 配置 ZK 环境变量
update_env_marked ZOOKEEPER_ENV "$(cat <<EOF
ZK_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$ZK_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 创建数据目录, 解压安装包
mkdir -p ${ZK_HOME}/{data,logs}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${ZK_HOME}
_log "解压 >>> ${PACKAGE}"

cat > ${ZK_HOME}/conf/zoo.cfg <<EOF
syncLimit                   = 5
initLimit                   = 10
tickTime                    = 2000
clientPort                  = 2181
autopurge.purgeInterval     = 5
autopurge.snapRetainCount   = 20
dataDir                     = ${ZK_HOME}/data
clientPortAddress           = $LAN_IP_ADDRESS
EOF
_log "创建配置文件 zoo.cfg ."

# IP 个数大于等于 2 集群，否则单机
if [[ ${#ZOOKEEPER[@]} -ge 2 ]]; then
    for IP in ${ZOOKEEPER[@]}; do
        let i+=1
        NODE=$(lookup_ip_hostname $IP)
        sed -i "\$a server.$i                    = ${NODE}:2888:3888" ${ZK_HOME}/conf/zoo.cfg
        [ "$IP" == "$LAN_IP_ADDRESS" ] && echo "$i" > ${ZK_HOME}/data/myid
    done
fi

# 设置 日志/PID/时区
cat > ${ZK_HOME}/conf/java.env <<EOF
# zookeeper 日志
ZOO_LOG_DIR=./logs

# zookeeper pid
ZOOPIDFILE=./data/zookeeper_server.pid

# zookeeper 时区
JVMFLAGS="-Duser.timezone=Asia/Shanghai"
EOF

# 配置 systemctl 启动 zookeeper
. /etc/profile &>/dev/null
cat > /usr/lib/systemd/system/zookeeper.service <<EOF
[Unit]
Description=zookeeper
After=network.target

[Service]
Type=simple
LimitCORE=infinity
LimitNPROC=1000000
LimitNOFILE=1000000
Restart=always
RestartSec=3
TimeoutSec=10
Environment="JAVA_HOME=${JAVA_HOME}"
WorkingDirectory=${ZK_HOME}
ExecStart=/bin/bash bin/zkServer.sh start-foreground
SuccessExitStatus=143
StandardError=append:${ZK_HOME}/logs/zookeeper.err
StandardOutput=append:${ZK_HOME}/logs/zookeeper.out

[Install]
WantedBy=multi-user.target
EOF
systemctl daemon-reload
_log "配置系统服务 ."

add_service_to_rc_local zookeeper
_log "添加服务启动命令到 /etc/rc.d/rc.local ."