#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称，安装目录定义
PACKAGE=hadoop-${HADOOP_VER}.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
HADOOP_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${HADOOP_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 配置 HADOOP 环境变量
update_env_marked HADOOP_ENV "$(cat <<EOF
HADOOP_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
HADOOP_MAPRED_HOME=\$HADOOP_HOME
pathmunge \$HADOOP_HOME/bin
pathmunge \$HADOOP_HOME/sbin
export HADOOP_HOME HADOOP_MAPRED_HOME
export HADOOP_CLASSPATH=\`hadoop classpath\`
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${HADOOP_HOME}/logs
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${HADOOP_HOME}
_log "解压 >>> ${PACKAGE}"

# hadoop 非高可用集群配置
cat > /tmp/hadoop.env <<EOF
CORE_CONF_fs_defaultFS=hdfs://$(lookup_hostname_with_port 9000 ${HADOOP[0]})
CORE_CONF_fs_trash_interval=1440
CORE_CONF_io_file_buffer_size=13107200
CORE_CONF_hadoop_tmp_dir=${HADOOP_HOME}/namesecondary

HDFS_CONF_dfs_replication=2
HDFS_CONF_dfs_namenode_name_dir=${HADOOP_HOME}/namenode
HDFS_CONF_dfs_datanode_data_dir=${HADOOP_HOME}/datanode
HDFS_CONF_dfs_permissions_enalbed=false
HDFS_CONF_dfs_namenode_http___address=$(lookup_hostname_with_port 50070 ${HADOOP[0]})
HDFS_CONF_dfs_namenode_secondary_http___address=$(lookup_hostname_with_port 50090 ${HADOOP[0]})

YARN_CONF_yarn_nodemanager_aux___services=mapreduce_shuffle
YARN_CONF_yarn_yarn_resourcemanager_hostname=${MASTER_NODE}
YARN_CONF_yarn_resourcemanager_scheduler_address=${MASTER_NODE}:8030
YARN_CONF_yarn_resourcemanager_resource___tracker_address=${MASTER_NODE}:8031
YARN_CONF_yarn_resourcemanager_address=$(lookup_hostname_with_port 8032 ${HADOOP[0]})
YARN_CONF_yarn_resourcemanager_admin_address=$(lookup_hostname_with_port 8033 ${HADOOP[0]})
YARN_CONF_yarn_resourcemanager_webapp_address=$(lookup_hostname_with_port 8088 ${HADOOP[0]})
YARN_CONF_yarn_nodemanager_vmem___check___enabled=false

MAPRED_CONF_mapreduce_framework_name=yarn
MAPRED_CONF_mapreduce_jobhistory_address=$(lookup_hostname_with_port 10020 ${HADOOP[0]})
MAPRED_CONF_mapreduce_jobhistory_webapp_address=$(lookup_hostname_with_port 19888 ${HADOOP[0]})
MAPRED_CONF_mapred_child_java_opts=-Xmx2048m
MAPRED_CONF_mapreduce_map_memory_mb=2048
MAPRED_CONF_mapreduce_reduce_memory_mb=4096
MAPRED_CONF_mapreduce_map_java_opts=-Xmx1500m
MAPRED_CONF_mapreduce_reduce_java_opts=-Xmx3100m
EOF

# hadoop 高可用集群配置
cat > /tmp/hadoop-ha.env <<EOF
CORE_CONF_fs_defaultFS=hdfs://mghdpha
CORE_CONF_hadoop_tmp_dir=file://${HADOOP_HOME}/tmpdir
CORE_CONF_ha_zookeeper_quorum=$(lookup_hostname_with_port 2181 ${ZOOKEEPER[@]})
CORE_CONF_ha_zookeeper_session___timeout_ms=15000

HDFS_CONF_dfs_replication=3
HDFS_CONF_dfs_namenode_name_dir=file://${HADOOP_HOME}/namenode
HDFS_CONF_dfs_datanode_data_dir=file://${HADOOP_HOME}/datanode
HDFS_CONF_dfs_nameservices=mghdpha
HDFS_CONF_dfs_ha_namenodes_mghdpha=nn1,nn2
HDFS_CONF_dfs_namenode_rpc___address_mghdpha_nn1=$(lookup_hostname_with_port 9000 ${HADOOP[0]})
HDFS_CONF_dfs_namenode_http___address_mghdpha_nn1=$(lookup_hostname_with_port 50070 ${HADOOP[0]})
HDFS_CONF_dfs_namenode_rpc___address_mghdpha_nn2=$(lookup_hostname_with_port 9000 ${HADOOP[1]})
HDFS_CONF_dfs_namenode_http___address_mghdpha_nn2=$(lookup_hostname_with_port 50070 ${HADOOP[1]})
HDFS_CONF_dfs_ha_automatic___failover_enabled=true
HDFS_CONF_dfs_namenode_shared_edits_dir='qjournal://$(lookup_hostname_with_port 8485 ${HADOOP[@]:2} | tr ',' ';')/mghdpha'
HDFS_CONF_dfs_client_failover_proxy_provider_mghdpha=org.apache.hadoop.hdfs.server.namenode.ha.ConfiguredFailoverProxyProvider
HDFS_CONF_dfs_ha_fencing_methods='shell(/bin/true)'
HDFS_CONF_dfs_ha_fencing_ssh_private___key___files=/root/.ssh/id_rsa
HDFS_CONF_dfs_ha_fencing_ssh_connect___timeout=30000
HDFS_CONF_dfs_journalnode_edits_dir=${HADOOP_HOME}/journalnode

YARN_CONF_yarn_resourcemanager_ha_enabled=true
YARN_CONF_yarn_resourcemanager_cluster___id=yrc
YARN_CONF_yarn_resourcemanager_ha_rm___ids=rm1,rm2
YARN_CONF_yarn_resourcemanager_hostname_rm1=$(lookup_ip_hostname ${HADOOP[0]})
YARN_CONF_yarn_resourcemanager_hostname_rm2=$(lookup_ip_hostname ${HADOOP[1]})
YARN_CONF_yarn_resourcemanager_zk___address=$(lookup_hostname_with_port 2181 ${ZOOKEEPER[@]})
YARN_CONF_yarn_nodemanager_aux___services=mapreduce_shuffle
YARN_CONF_yarn_nodemanager_resource_memory___mb=10240
YARN_CONF_yarn_log__aggregation__enable=true
YARN_CONF_yarn_log__aggregation_retain__seconds=604800
YARN_CONF_yarn_log_server_url=http://$(lookup_hostname_with_port 19888 ${HADOOP[0]})/jobhistory/logs

MAPRED_CONF_mapreduce_framework_name=yarn
MAPRED_CONF_mapreduce_map_memory_mb=4096
MAPRED_CONF_mapreduce_reduce_memory_mb=4096
MAPRED_CONF_mapreduce_map_java_opts=-Xmx3000m
MAPRED_CONF_mapreduce_reduce_java_opts=-Xmx3000m
MAPRED_CONF_mapreduce_job_reduces=2
MAPRED_CONF_mapreduce_jobhistory_address=$(lookup_hostname_with_port 10020 ${HADOOP[0]})
MAPRED_CONF_mapreduce_jobhistory_webapp_address=$(lookup_hostname_with_port 19888 ${HADOOP[0]})
EOF
_log "创建 hadoop 配置文件"

# yarn资源不足造
# https://www.cnblogs.com/jizhong/p/16828300.html


# 角色分配
# 节点数量大于等于5个, 则前2个为 namenode, 其他全部为 datanode
if (( ${#HADOOP[@]} >= 5 )); then
    namenode=${HADOOP[@]:0:2}
    zkfc=${namenode}
    resourcemanager=${namenode}

    datanode=${HADOOP[@]:2}
    nodemanager=${datanode}
    journalnode=${datanode}
    deploy_type=cluster
	env_file=/tmp/hadoop-ha.env
else
    namenode=${HADOOP[0]}
    resourcemanager=${namenode}

    datanode=${HADOOP[@]:1}
    nodemanager=${datanode}
    secondarynamenode=${HADOOP[-1]}
    deploy_type=standalone
	env_file=/tmp/hadoop.env
fi

green_echo ">>> 集群角色信息 ."
if [ "${deploy_type}" == "cluster" ]; then
    printf "%-20s %s\n" " namenode:" "${namenode}"
    printf "%-20s %s\n" " zkfc:" "${zkfc}"
    printf "%-20s %s\n" " resourcemanager:" "${resourcemanager}"
    printf "%-20s %s\n" " datanode:" "${datanode}"
    printf "%-20s %s\n" " nodemanager:" "${nodemanager}"
    printf "%-20s %s\n" " journalnode:" "${journalnode}"
else
    printf "%-20s %s\n" " namenode:" "${namenode}"
    printf "%-20s %s\n" " resourcemanager:" "${resourcemanager}"
    printf "%-20s %s\n" " datanode:" "${datanode}"
    printf "%-20s %s\n" " nodemanager:" "${nodemanager}"
    printf "%-20s %s\n" " secondarynamenode:" "${secondarynamenode}"
fi

# 设置模板内容为变量
while read line; do
    eval export $line
done < <(egrep -v '^$|^#' ${env_file})
_log "读取模板变量设置为全局变量 ."

# 拼接 hadoop 配置
function addProperty() {
    local path=$1
    local name=$2
    local value=$3
    local entry="  <property>\n    <name>${name}</name>\n    <value>${value}</value>\n  </property>"
    local escapedEntry=$(printf '%s' "$entry" | sed 's/\//\\\//g')
    sed -i "/<\/configuration>/ s/.*/${escapedEntry}\n&/" $path
}

# 读取环境变量解析参数&& 参数值
function configure() {
    local path=$1
    local envPrefix=$2
    local var value
	green_echo ">>> Configuring $(basename ${path})"

    while IFS='=' read -r line; do
        if [[ $line =~ ^${envPrefix}_([^=]+) ]]; then
            c=${BASH_REMATCH[1]}
            name=${c//___/-}
            name=${name//__/@}
            name=${name//_/.}
            name=${name//@/_}
            value=${!line}
            echo " - Setting $name=$value"
            addProperty $path $name $value
            eval unset "$line"
        fi
    done < <(sort -r < <(grep "^$envPrefix" < <(compgen -v)))
}

# 备份默认配置
/bin/cp -r ${HADOOP_HOME}/etc ${HADOOP_HOME}/bak-etc
[ -f "${HADOOP_HOME}/etc/hadoop/mapred-site.xml" ] || cp ${HADOOP_HOME}/etc/hadoop/mapred-site.xml.template ${HADOOP_HOME}/etc/hadoop/mapred-site.xml

# 配置 hadoop 环境变量
cat >> ${HADOOP_HOME}/etc/hadoop/hadoop-env.sh <<EOF
# mango  $(date '+%F %T')
export HADOOP_IDENT_STRING=root
export HADOOP_SECURE_DN_USER=root
export HADOOP_PID_DIR=./pid
export HADOOP_SECURE_DN_PID_DIR=./pid
EOF

# 配置 yarn 环境变量
cat >> ${HADOOP_HOME}/etc/hadoop/yarn-env.sh <<EOF
# mango  $(date '+%F %T')
export HADOOP_YARN_USER=root
export YARN_IDENT_STRING=root
export YARN_PID_DIR=./pid
EOF

# 配置 mapred 环境变量
cat >> ${HADOOP_HOME}/etc/hadoop/mapred-env.sh <<EOF
# mango  $(date '+%F %T')
export HADOOP_MAPRED_IDENT_STRING=root
export HADOOP_MAPRED_PID_DIR=./pid
EOF

# 生成 hadoop 配置
if [ ! -f ${HADOOP_HOME}/etc/.init ]; then
    configure ${HADOOP_HOME}/etc/hadoop/core-site.xml CORE_CONF
    configure ${HADOOP_HOME}/etc/hadoop/hdfs-site.xml HDFS_CONF
    configure ${HADOOP_HOME}/etc/hadoop/yarn-site.xml YARN_CONF
    configure ${HADOOP_HOME}/etc/hadoop/mapred-site.xml MAPRED_CONF
    _log "生成配置文件 core-site.xml hdfs-site.xml yarn-site.xml mapred-site.xml ."
    echo "# $(date +'%F %T') core-site.xml hdfs-site.xml yarn-site.xml mapred-site.xml" > ${HADOOP_HOME}/etc/.init
fi

# 配置 ssh 端口
echo "export HADOOP_SSH_OPTS='-p $SSH_PORT'" >> ${HADOOP_HOME}/etc/hadoop/hadoop-env.sh

# 配置 hadoop
rm -f ${HADOOP_HOME}/etc/hadoop/slaves
for node in $datanode; do echo "$node"  >> ${HADOOP_HOME}/etc/hadoop/slaves; done

# 配置服务
. /etc/profile &>/dev/null
function CFG_SYSTEMCTL() {
    local NAME="$1" START_CMD="$2" STOP_CMD="$3" WANTS="$4"
    cat > /usr/lib/systemd/system/${NAME}.service <<-EOF
	[Unit]
	Description=hadoop ${NAME}
	After=syslog.target network.target

	[Service]
	LimitCORE=infinity
	LimitNPROC=1000000
	LimitNOFILE=1000000
	Restart=always
	RestartSec=3
	TimeoutSec=10
	Type=forking
	User=root
	Group=root
	Environment="JAVA_HOME=${JAVA_HOME}"
	WorkingDirectory=${HADOOP_HOME}
	ExecStart=$START_CMD
	ExecStop=$STOP_CMD
	SuccessExitStatus=143

	[Install]
	WantedBy=multi-user.target
	EOF

    # 添加依赖服务
	[ -n "${WANTS}" ] && sed -i "/After=/a\Wants=${WANTS}/" /usr/lib/systemd/system/${NAME}.service

    true &&_log "配置系统服务 ${NAME} ."
    systemctl daemon-reload

    add_service_to_rc_local ${NAME}
    _log "添加服务启动命令到 /etc/rc.d/rc.local ."
}

# 配置服务
for role in namenode zkfc resourcemanager datanode journalnode nodemanager secondarynamenode; do
    # 将变量转换为数组
    role_array=(${!role})

    #echo "$role  --->  ${role_array[@]}"

    # 判断当前IP是否存在于数组中
    if contains "$LAN_IP_ADDRESS" role_array[@]; then
        case $role in
            namenode)
                CFG_SYSTEMCTL "hdp-namenode" "/bin/bash sbin/hadoop-daemon.sh start namenode" "/bin/bash sbin/hadoop-daemon.sh stop namenode"
                ;;
            zkfc)
                CFG_SYSTEMCTL "hdp-zkfc" "/bin/bash sbin/hadoop-daemon.sh start zkfc" "/bin/bash sbin/hadoop-daemon.sh stop zkfc" \
				              "hdp-namenode.service zookeeper.service"
                ;;
            resourcemanager)
                CFG_SYSTEMCTL "hdp-resourcemanager" "/bin/bash sbin/yarn-daemon.sh start resourcemanager" "/bin/bash sbin/yarn-daemon.sh stop resourcemanager" \
				              "hdp-namenode.service"
                ;;
            datanode)
                CFG_SYSTEMCTL "hdp-datanode" "/bin/bash sbin/hadoop-daemon.sh start datanode" "/bin/bash sbin/hadoop-daemon.sh stop datanode"
                ;;
            journalnode)
                CFG_SYSTEMCTL "hdp-journalnode" "/bin/bash sbin/hadoop-daemon.sh start journalnode" "/bin/bash sbin/hadoop-daemon.sh stop journalnode" \
				              "zookeeper.service"
                ;;
            nodemanager)
                CFG_SYSTEMCTL "hdp-nodemanager" "/bin/bash sbin/yarn-daemon.sh start nodemanager" "/bin/bash sbin/yarn-daemon.sh stop nodemanager" \
				              "hdp-datanode.service"
                ;;
            secondarynamenode)
                CFG_SYSTEMCTL "hdp-secondarynamenode" "/bin/bash sbin/hadoop-daemon.sh start secondarynamenode" "/bin/bash sbin/hadoop-daemon.sh stop secondarynamenode" \
				              "hdp-datanode.service"
                ;;
        esac
    fi
done

### hadoop 初始化
# if [ "$LAN_IP_ADDRESS" == "${HADOOP[0]}" ]; then
#     if [ ! -d "${HADOOP_HOME}/namenode" ]; then
#         # 启动 Zookeeper 集群
# 		_log "启动 zookeeper ."
#         for node in ${ZOOKEEPER[@]}; do
# 			ssh -p ${SSH_PORT:-22} -T $node "until ls ${ZK_HOME} &>/dev/null; do sleep 5; done; systemctl start zookeeper"
#             _log "[$node] 节点启动 zookeeper ."
#         done
#         sleep 5

#         # 初始化 zkfc
#         hdfs zkfc -formatZK -force &>/tmp/zkfc-format.out
#         _log "[$LAN_IP_ADDRESS] 节点格式化 zkfc ."
#         sleep 5

#         # 启动 JournalNode
#         for node in ${HADOOP[@]:2}; do
#             ssh -p ${SSH_PORT:-22} -T $node "systemctl start hdp-journalnode"
#             _log "[$node] 节点启动 hdp-journalnode ."
#         done
#         sleep 5

#         # 初始化 NameNode1
#         hdfs namenode -format -force &>/tmp/namenode-format.out
#         _log "[$LAN_IP_ADDRESS] 节点格式化 namenode ."

#         # 启动 NameNode1
#         systemctl start hdp-namenode
#         _log "[$LAN_IP_ADDRESS] 节点启动 namenode ."
#         sleep 5

#         # 同步/启动 NameNode2
#         ssh -p ${SSH_PORT:-22} -T ${HADOOP[1]} <<-'EOF'
#             hdfs namenode -bootstrapStandby -force &>/tmp/namenode-bootstrapStandby.out
#             systemctl start hdp-namenode
# 		EOF
#         _log "[${HADOOP[1]}] 节点同步/启动 namenode ."
#         sleep 5

#         # 启动 Zkfc、ResourceManager
#         for node in ${HADOOP[@]:0:2}; do
#             ssh -p ${SSH_PORT:-22} -T $node "systemctl start hdp-zkfc hdp-resourcemanager"
#             _log "[$node] 节点启动 hdp-zkfc、hdp-resourcemanager ."
#         done

#         # 启动 DataNode、hdp-nodemanager
#         for node in ${HADOOP[@]:2}; do
#             ssh -p ${SSH_PORT:-22} -T $node "systemctl start hdp-datanode hdp-nodemanager"
#             _log "[$node] 节点启动 hdp-datanode、hdp-nodemanager."
#         done
#     fi
# fi