#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称
PACKAGE=apache-seatunnel-${SEATUNNEL_VER}-bin.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
SEATUNNEL_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${SEATUNNEL_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 配置 SEATUNNEL 环境变量
update_env_marked SEATUNNEL_ENV "$(cat <<EOF
SEATUNNEL_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$SEATUNNEL_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 >>> ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${SEATUNNEL_HOME}/data/checkpoint_snapshot
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${SEATUNNEL_HOME}
_log "解压 >>> ${PACKAGE} ."

# 连接器
if [ ! -f "${PACKAGE_DIR}/apache-seatunnel-${SEATUNNEL_VER}-connectors.tgz" ]; then
    retry_until_success curl -sL -o ${PACKAGE_DIR}/apache-seatunnel-${SEATUNNEL_VER}-connectors.tgz ${SERVER_URL}/apache-seatunnel-${SEATUNNEL_VER}-connectors.tgz
    _log "连接器下载 >>> apache-seatunnel-${SEATUNNEL_VER}-connectors.tgz ."
fi

# 数据库驱动/ http-base
if [ ! -f "${PACKAGE_DIR}/apache-seatunnel-${SEATUNNEL_VER}-customized.tgz" ]; then
    retry_until_success curl -sL -o ${PACKAGE_DIR}/apache-seatunnel-${SEATUNNEL_VER}-customized.tgz ${SERVER_URL}/apache-seatunnel-${SEATUNNEL_VER}-customized.tgz
    _log "驱动下载 >>> apache-seatunnel-${SEATUNNEL_VER}-customized.tgz ."
fi

# web 管理
if [ ! -f "${PACKAGE_DIR}/apache-seatunnel-web-${SEATUNNEL_WEB_VER}-bin.tar.gz" ]; then
    retry_until_success curl -sL -o ${PACKAGE_DIR}/apache-seatunnel-web-${SEATUNNEL_WEB_VER}-bin.tar.gz ${SERVER_URL}/apache-seatunnel-web-${SEATUNNEL_WEB_VER}-bin.tar.gz
    _log "下载 >>> apache-seatunnel-web-${SEATUNNEL_WEB_VER}-bin.tar.gz ."
fi

# web 管理数据驱动
if [ ! -f "${PACKAGE_DIR}/apache-seatunnel-web-${SEATUNNEL_WEB_VER}-datasource.tgz" ]; then
    retry_until_success curl -sL -o ${PACKAGE_DIR}/apache-seatunnel-web-${SEATUNNEL_WEB_VER}-datasource.tgz ${SERVER_URL}/apache-seatunnel-web-${SEATUNNEL_WEB_VER}-datasource.tgz
    _log "下载 >>> apache-seatunnel-web-${SEATUNNEL_WEB_VER}-datasource.tgz ."
fi

tar xf ${PACKAGE_DIR}/apache-seatunnel-${SEATUNNEL_VER}-connectors.tgz -C ${SEATUNNEL_HOME}
tar xf ${PACKAGE_DIR}/apache-seatunnel-${SEATUNNEL_VER}-customized.tgz -C ${SEATUNNEL_HOME}
_log "解压 连接器/数据库驱动 ."

# 删除默认 http-base
rm -f ${SEATUNNEL_HOME}/connectors/connector-http-base-${SEATUNNEL_VER}.jar

# 服务端配置
sed -i "/- localhost/d" ${SEATUNNEL_HOME}/config/hazelcast.yaml
for ((i=${#SEATUNNEL[@]}-1; i>=0; i--)); do
    sed -i "/member-list:/a\          - ${SEATUNNEL[i]}" ${SEATUNNEL_HOME}/config/hazelcast.yaml
done
_log "创建 seatunnel hazelcast 配置 ."

# 客户端配置
sed -i "/- localhost:5801/d" ${SEATUNNEL_HOME}/config/hazelcast-client.yaml
for ((i=${#SEATUNNEL[@]}-1; i>=0; i--)); do
    sed -i "/cluster-members:/a\      - ${SEATUNNEL[i]}:5801" ${SEATUNNEL_HOME}/config/hazelcast-client.yaml
done
_log "创建 seatunnel hazelcast-client 配置 ."

sed -i "s|namespace:.*|namespace: ${SEATUNNEL_HOME}/data/checkpoint_snapshot|" ${SEATUNNEL_HOME}/config/seatunnel.yaml
sed -i "s|fs.defaultFS:.*|fs.defaultFS: file://${SEATUNNEL_HOME}/data|" ${SEATUNNEL_HOME}/config/seatunnel.yaml

# 配置服务
. /etc/profile &>/dev/null
function CFG_SYSTEMCTL() {
    local NAME="$1" START_CMD="$2" STOP_CMD="$3" RUN_HOME="$4" TYPE="$5"
    cat > /usr/lib/systemd/system/${NAME}.service <<-EOF
	[Unit]
	Description=Seatunnel ${NAME}
	After=syslog.target network.target

	[Service]
	LimitCORE=infinity
	LimitNPROC=1000000
	LimitNOFILE=1000000
	Restart=always
	RestartSec=10
	TimeoutSec=10
	Type=${TYPE}

	Environment="JAVA_HOME=${JAVA_HOME}"
	Environment="SEATUNNEL_HOME=${SEATUNNEL_HOME}"
	Environment="PATH=${JAVA_HOME}/bin:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin"
	WorkingDirectory=${RUN_HOME}
	ExecStart=$START_CMD
	ExecStop=$STOP_CMD
	SuccessExitStatus=143

	[Install]
	WantedBy=multi-user.target
	EOF

    # seatunnel-web 初始化数据库
    if [ "${LAN_IP_ADDRESS}" == "${SEATUNNEL[0]}" ] && [ "${NAME}" == "seatunnel-web" ]; then
        sed -i "/^ExecStart=/i \ExecStartPre=/bin/sh -c '[ -f \"script/init_sql.sh\" ] && { sh script/init_sql.sh && mv script/init_sql.sh script/init_sql.sh.bak; }; exit 0'" /usr/lib/systemd/system/${NAME}.service
    fi

    _log "配置系统服务 ${NAME} ."
    systemctl daemon-reload

    add_service_to_rc_local ${NAME}
    _log "添加服务启动命令到 /etc/rc.d/rc.local ."

    # 屏蔽错误(seatunnel-web 数据库未准备就绪)
    systemctl start ${NAME} &>/dev/null || true
    _log "启动 ${NAME} 服务 ."
}

# 配置 seatunnel 服务
CFG_SYSTEMCTL "seatunnel" "bash bin/seatunnel-cluster.sh" "bash bin/stop-seatunnel-cluster.sh" "${SEATUNNEL_HOME}" "simple"

# 安装配置 seatunnel-web
if [ "${LAN_IP_ADDRESS}" == "${SEATUNNEL[0]}" ]; then
    SEATUNNEL_WEB_HOME=${SOFT_INSTALL_DIR}/apache-seatunnel-web-${SEATUNNEL_WEB_VER}-bin
    mkdir -p ${SEATUNNEL_WEB_HOME}
    tar xf ${PACKAGE_DIR}/apache-seatunnel-web-${SEATUNNEL_WEB_VER}-bin.tar.gz --strip-components=1 -C ${SEATUNNEL_WEB_HOME}
    tar xf ${PACKAGE_DIR}/apache-seatunnel-web-${SEATUNNEL_WEB_VER}-datasource.tgz -C ${SEATUNNEL_WEB_HOME}
    /usr/bin/cp ${SEATUNNEL_HOME}/lib/mysql-connector-java-*.jar ${SEATUNNEL_WEB_HOME}/libs/
    /usr/bin/cp ${SEATUNNEL_HOME}/config/hazelcast-client.yaml ${SEATUNNEL_WEB_HOME}/conf/

    #cp ${SEATUNNEL_HOME}/connectors/plugin-mapping.properties ${SEATUNNEL_WEB_HOME}/conf/
    rm -f ${SEATUNNEL_WEB_HOME}/conf/plugin-mapping.properties
    for plugin in $(sed -n '/--connectors-v2--/ {n; :a; /--end--/!{p; n; ba}}' $SEATUNNEL_HOME/config/plugin_config); do
        grep -r $plugin $SEATUNNEL_HOME/connectors/plugin-mapping.properties >> ${SEATUNNEL_WEB_HOME}/conf/plugin-mapping.properties
    done

    # 配置 seatunnel-web 环境变量
    cat <<-EOF   > ${SEATUNNEL_WEB_HOME}/script/seatunnel_server_env.sh
	export HOSTNAME="${NACOS[1]}"
	export PORT="3306"
	export USERNAME="${MYSQL_USER}"
	export PASSWORD="${MYSQL_PASS}"
	EOF

    # 配置 seatunnel-web
    sed -i "s~username:.*~username: ${MYSQL_USER}~" ${SEATUNNEL_WEB_HOME}/conf/application.yml
    sed -i "s~password:.*~password: ${MYSQL_PASS}~" ${SEATUNNEL_WEB_HOME}/conf/application.yml
    sed -i "s~jdbc:mysql://localhost:3306~jdbc:mysql://${NACOS[1]}:3306~" ${SEATUNNEL_WEB_HOME}/conf/application.yml

    #head -c 30 < <( tr -dc 'A-Z0-9' < <(cat /dev/urandom))
    #cat /dev/urandom | tr -dc 'A-Z0-9' | head -c 30
    #cat /dev/urandom | tr -dc 'A-Z0-9' | head -c 30 | sha256sum
    #cat /dev/urandom | tr -dc 'A-Z0-9' | head -c 30 | openssl dgst -sha256

    SecretKey=$(cat /dev/urandom | tr -dc 'A-Z0-9' | head -c 65)
    sed -i "s~secretKey:.*~secretKey: ${SecretKey}~" ${SEATUNNEL_WEB_HOME}/conf/application.yml

    # # 初始化前检查依赖(由systemctl服务管理)
    chmod +x ${SEATUNNEL_WEB_HOME}/script/init_sql.sh

    # 配置 seatunnel-web 服务
    CFG_SYSTEMCTL "seatunnel-web" "bash bin/seatunnel-backend-daemon.sh start" "bash bin/seatunnel-backend-daemon.sh stop" "${SEATUNNEL_WEB_HOME}" "forking"
fi


### 初始化
# 验证 数据库已经启动
# listIniKeysValues nacos environment
# until nc -vw 1 ${NACOS[1]} -z 3306 &>/dev/null; do
#     sleep 2
#     _info "等待数据库 ${NACOS[1]} 准备就绪 ."
# done

# systemctl restart ${NAME}
# _log "启动服务 ${NAME} ."