#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称
PACKAGE=nacos-server-${NACOS_VER}.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
NACOS_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${NACOS_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 配置 NACOS 环境变量
update_env_marked NACOS_ENV "$(cat <<EOF
NACOS_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${NACOS_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${NACOS_HOME}
_log "解压 >>> ${PACKAGE}"

# 禁用 tomcat 日志
sed -i 's@server.tomcat.accesslog.enabled=true@server.tomcat.accesslog.enabled=false@g' ${NACOS_HOME}/conf/application.properties

# 配置监听地址
echo -e "\n### Specify local server's IP:\nnacos.inetutils.ip-address=$LAN_IP_ADDRESS" >> ${NACOS_HOME}/conf/application.properties

# 配置加密秘钥
IdentityValue='security'
IdentityKey='serverIdentity'
SecretKey='M0E5NTVJNFo4WklMMkpOTVhNVU9RM1FFN05SRldQNUtTMTc4OTdNTUlCRUU2V1RTTTlCVUtSR0RUSlRGNzNLQjQ2WkwwUQ=='
sed -e "s|nacos.core.auth.enabled=.*|nacos.core.auth.enabled=false|" \
    -e "s|nacos.core.auth.server.identity.key=.*|nacos.core.auth.server.identity.key=${IdentityKey}|" \
    -e "s|nacos.core.auth.server.identity.value=.*|nacos.core.auth.server.identity.value=${IdentityValue}|" \
    -e "s|nacos.core.auth.plugin.nacos.token.secret.key=.*|nacos.core.auth.plugin.nacos.token.secret.key=${SecretKey}|" \
    -i ${NACOS_HOME}/conf/application.properties

# 配置用户(mysql)  hcadmin 9JOB2THIR6F86AE6I742
cat >> ${NACOS_HOME}/conf/mysql-schema.sql <<'EOF'

INSERT INTO users (username, password, enabled) VALUES ('hcadmin', '$2a$10$pRQZ1fx3TiWXDlWoAFUY8OKKD7kXOnvBJnyYdx4ZSGg/psY186mcG', TRUE);
INSERT INTO roles (username, role) VALUES ('hcadmin', 'ROLE_ADMIN');
EOF


# 内置数据  hcadmin 9JOB2THIR6F86AE6I742
cat >> ${NACOS_HOME}/conf/derby-schema.sql <<'EOF'

INSERT INTO users (username, password, enabled) VALUES ('hcadmin', '$2a$10$pRQZ1fx3TiWXDlWoAFUY8OKKD7kXOnvBJnyYdx4ZSGg/psY186mcG', TRUE);
INSERT INTO roles (username, role) VALUES ('hcadmin', 'ROLE_ADMIN');
EOF

# 配置服务
. /etc/profile &>/dev/null
cat > /usr/lib/systemd/system/nacos.service <<EOF
[Unit]
Description=nacos
After=network.target

[Service]
Type=forking
ExecStart=${NACOS_HOME}/bin/startup.sh -m standalone
Environment="JAVA_HOME=$JAVA_HOME"
ExecStop=${NACOS_HOME}/bin/shutdown.sh
MemoryMax=2048M
Restart=always
RestartSec=10

[Install]
WantedBy=multi-user.target
EOF

_log "配置系统服务 ."
systemctl daemon-reload

add_service_to_rc_local nacos
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl restart nacos
_log "启动 Nacos 服务 ."