#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 检测安装状态
if rpm -q fastdfs-server &>/dev/null; then
    _info "本机已经安装 $(rpm -qa fastdfs-server) ."
    exit 0
fi

# 安装包名称
PACKAGE=fastdfs-${FDFS_VER}.tgz
INS_DIR=${PACKAGE%.tgz}

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 解压
tar xf ${PACKAGE_DIR}/${PACKAGE} -C ${PACKAGE_DIR}
_log "解压 >>> ${PACKAGE}"

# 安装
## rpm -e fastdfs-tool libfdfsclient fastdfs-server fastdfs-config libserverframe libserverframe
fetch_and_install_rpm ${PACKAGE_DIR}/fastdfs-${FDFS_VER}/libfastcommon-*.rpm
fetch_and_install_rpm ${PACKAGE_DIR}/fastdfs-${FDFS_VER}/libserverframe-*.rpm
fetch_and_install_rpm ${PACKAGE_DIR}/fastdfs-${FDFS_VER}/fastdfs-config-*.rpm
fetch_and_install_rpm ${PACKAGE_DIR}/fastdfs-${FDFS_VER}/fastdfs-server-*.rpm
fetch_and_install_rpm ${PACKAGE_DIR}/fastdfs-${FDFS_VER}/libfdfsclient-*.rpm
fetch_and_install_rpm ${PACKAGE_DIR}/fastdfs-${FDFS_VER}/fastdfs-tool-*.rpm
fetch_and_install_rpm ${PACKAGE_DIR}/fastdfs-${FDFS_VER}/nginx-*.rpm

# 配置
TRACKER_SERVER=${FASTDFS[@]}
STORAGE_DATA=${SOFT_INSTALL_DIR}/fastdfs
mkdir -p $STORAGE_DATA

# trackerd
sed -i "s@^base_path.*@base_path = $STORAGE_DATA@" /etc/fdfs/tracker.conf
sed -i "s@^reserved_storage_space.*@reserved_storage_space = 10%@" /etc/fdfs/tracker.conf
_log "Fastdfs 配置 Trackerd ."

# storaged
sed -i "s@^base_path.*@base_path = $STORAGE_DATA@" /etc/fdfs/storage.conf
sed -i "s@^store_path0.*@store_path0 = $STORAGE_DATA@" /etc/fdfs/storage.conf
sed -i "/^tracker_server/d" /etc/fdfs/storage.conf
for node in $TRACKER_SERVER; do
    node_ip=$(awk "/$node/{print \$1}" /etc/hosts | tail -n 1)
    node=${node_ip:-$node}
    sed -i "178a tracker_server = ${node}:22122" /etc/fdfs/storage.conf
done
_log "Fastdfs 配置 Storaged ."

# client
sed -i "s@^base_path.*@base_path = $STORAGE_DATA@" /etc/fdfs/client.conf
sed -i "/^tracker_server/d" /etc/fdfs/client.conf
for node in $TRACKER_SERVER; do
    node_ip=$(awk "/$node/{print \$1}" /etc/hosts | tail -n 1)
    node=${node_ip:-$node}
    sed -i "25a tracker_server = ${node}:22122" /etc/fdfs/client.conf
done
_log "Fastdfs 配置 Client ."

# nginx
sed -i "s@^base_path.*@base_path = $STORAGE_DATA@" /etc/fdfs/mod_fastdfs.conf
sed -i "s@^store_path0.*@store_path0 = $STORAGE_DATA@" /etc/fdfs/mod_fastdfs.conf
sed -i "s@^url_have_group_name.*@url_have_group_name = true@" /etc/fdfs/mod_fastdfs.conf
sed -i "/^tracker_server/d" /etc/fdfs/mod_fastdfs.conf
for node in $TRACKER_SERVER; do
    node_ip=$(awk "/$node/{print \$1}" /etc/hosts | tail -n 1)
    node=${node_ip:-$node}
    sed -i "39a tracker_server = ${node}:22122" /etc/fdfs/mod_fastdfs.conf
done
_log "Fastdfs 配置 Fastdfs-Nginx ."

# systemctl start fdfs_trackerd fdfs_storaged nginx
sed -i "s@/opt/fastdfs@$STORAGE_DATA@g" /usr/lib/systemd/system/fdfs_trackerd.service
sed -i "s@/opt/fastdfs@$STORAGE_DATA@g" /usr/lib/systemd/system/fdfs_storaged.service
systemctl daemon-reload
_log "Fastdfs 配置服务 ."

add_service_to_rc_local fdfs_trackerd
add_service_to_rc_local fdfs_storaged
add_service_to_rc_local nginx
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl start fdfs_trackerd fdfs_storaged nginx
_log "启动服务 ."