#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称
PACKAGE=minio-${MINIO_VER}.tgz
INS_DIR=${PACKAGE%%-*}

# 检测安装状态
MINIO_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${MINIO_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 配置 MINIO 环境变量
update_env_marked MINIO_ENV "$(cat <<EOF
MINIO_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$MINIO_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 解压
mkdir -p ${MINIO_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${MINIO_HOME}
_log "解压 >>> ${PACKAGE}"

# 创建数据目录 && 配置服务
mkdir -p ${MINIO_HOME}/data
sed -i "s|MINIO_HOME|$MINIO_HOME|" $MINIO_HOME/conf/minio.ini
sed "s|MINIO_HOME|$MINIO_HOME|" $MINIO_HOME/minio.service > /usr/lib/systemd/system/minio.service
rm -f ${MINIO_HOME}/visual3d.conf $MINIO_HOME/minio.service
systemctl daemon-reload
_log "配置系统服务 ."

add_service_to_rc_local minio
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl restart minio
_log "启动 minio 服务 ."

# 配置 mc
sleep 5
. ${MINIO_HOME}/conf/minio.ini
$MINIO_HOME/bin/mc alias set minio http://${LAN_IP_ADDRESS}:9000 ${MINIO_ROOT_USER} ${MINIO_ROOT_PASSWORD}
_log "配置 mc ."

# 创建存储桶
for bucket in applog bigfile bim-offline ht-editor tools visual3d; do
    MSG=$($MINIO_HOME/bin/mc mb minio/$bucket)
    _log "创建存储桶 $bucket >>> $MSG ."
done

# 配置存储桶公共权限(允许web浏览下载)
for bucket in visual3d tools bim-offline; do
    MSG=$($MINIO_HOME/bin/mc policy set download minio/$bucket)
    _log "配置存储桶 [$bucket] 权限为 <public> >>> $MSG ."
done

### ht 监视画面
# 安装包名称
PACKAGE=ht-3d-editor.tgz

# 检测安装状态
HT_HOME=${MINIO_HOME}/data/ht-editor/ht-3d-editor-0.3.1.4
if [ -d "${HT_HOME}" ]; then
    _info "本机已经存在 ${HT_HOME} ."
    exit 0
fi

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 解压服务端
mkdir -p ${HT_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${HT_HOME}
_log "解压 >>> ${PACKAGE} ."

# 配置白名单
SIDServiceWhiteList=$(lookup_ip_addresses ${WFW[@]} $VIP)
if [ -f ${HT_HOME}/server/config.ini ]; then
    sed -i "s#SIDServiceWhiteList =.*#SIDServiceWhiteList = ${SIDServiceWhiteList}#" ${HT_HOME}/server/config.ini
    _log "配置白名单 >>> [${SIDServiceWhiteList}] ."
fi

### nodejs
# 安装包名称
PACKAGE=node-v${NODEJS_VER}-linux-x64.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
NODEJS_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${NODEJS_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 配置 MINIO 环境变量
update_env_marked NODEJS_ENV "$(cat <<EOF
NODEJS_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$NODEJS_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 解压服务端
mkdir -p ${NODEJS_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${NODEJS_HOME}
_log "解压 >>> ${PACKAGE} ."

# 配置服务
cat > /usr/lib/systemd/system/ht.service <<EOF
[Unit]
Description=Ht-d3-editor

[Service]
LimitCORE=infinity
LimitNPROC=1000000
LimitNOFILE=1000000
WorkingDirectory=${SOFT_INSTALL_DIR}/minio/data/ht-editor
ExecStart=${NODEJS_HOME}/bin/node ht-3d-editor-0.3.1.4/server/server.js
Restart=always
RestartSec=10
User=root
Group=root

[Install]
WantedBy=multi-user.target
EOF

systemctl daemon-reload
_log "配置系统服务 ."

add_service_to_rc_local ht
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl restart ht
_log "启动 NodeJs 服务 ."