#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称
PACKAGE=tomcat-linux.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
TOMCAT_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${TOMCAT_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${TOMCAT_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${TOMCAT_HOME}
_log "解压 >>> ${PACKAGE}"

# 改默认端口号
sed -i 's#port="8080"#port="7070"#g' ${SOFT_INSTALL_DIR}/tomcat-linux/conf/server.xml
_log "更改监听端口为 7070 ."

## 安裝字體
# 安装 fc-list
if ! which fc-list &>/dev/null; then
    dnf install -y fontconfig &>/dev/null
    _log "安装 fontconfig ."
fi

# 安装 ttmkfdir
if ! which ttmkfdir &>/dev/null; then
    dnf install -y ttmkfdir &>/dev/null
    _log "安装 ttmkfdir ."
fi

# 安装包名称
PACKAGE=chinese.tar.gz

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

tar xf ${PACKAGE_DIR}/${PACKAGE} -C /usr/share/fonts
_log "解压字体包 >>> ${PACKAGE}"

# 生成字体索引
ttmkfdir -d /usr/share/fonts/chinese -o /usr/share/fonts/chinese/fonts.scale
sed -i '/<dir>\/usr\/share\/fonts<\/dir>/a\        <dir>/usr/share/fonts/chinese</dir>' /etc/fonts/fonts.conf
green_echo ">>> 生成字体索引 ."
fc-cache && fc-list | grep sim

# 配置服务
cat > /usr/lib/systemd/system/tomcat-fanruan.service <<EOF
[Unit]
Description=Tomcat
After=network.target

[Service]
Type=forking
ExecStart=${SOFT_INSTALL_DIR}/tomcat-linux/bin/startup.sh
Environment="JAVA_HOME=${SOFT_INSTALL_DIR}/tomcat-linux/jdk"
ExecStop=${SOFT_INSTALL_DIR}/tomcat-linux/bin/shutdown.sh
MemoryLimit=2048M
Restart=always
RestartSec=50s

[Install]
WantedBy=multi-user.target
EOF

systemctl daemon-reload
_log "配置系统服务 ."

add_service_to_rc_local tomcat-fanruan
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl restart tomcat-fanruan
_log "启动 tomcat 服务 ."

_info "初始化地址: http://${LAN_IP_ADDRESS}:7070/webroot/decision"



cat <<EOF  > /dev/null
# nginx 代理配置
# https://help.fanruan.com/finereport/doc-view-2815

#user  root;
worker_processes  auto;
worker_cpu_affinity  auto;

error_log  logs/error.log;
#error_log  logs/error.log  notice;
#error_log  logs/error.log  info;
#pid        logs/nginx.pid;

events {
    worker_connections  1024;
}

http {
    include       mime.types;
    default_type  application/octet-stream;
    log_format  main  '$remote_addr - $remote_user [$time_local] "$request" '
                      '$status $body_bytes_sent "$http_referer" '
                      '"$http_user_agent" "$http_x_forwarded_for" $upstream_addr';

    access_log  logs/access.log  main;
    #sendfile 指令指定 nginx 是否调用 sendfile 函数（zero copy 方式）来输出文件，

    sendfile        on;
    #tcp_nopush     on;
    keepalive_timeout  65s;
    types_hash_max_size 2048;
    #gzip  on;
    #gzip_disable "MSIE [1-6].";
    client_header_buffer_size    512k;
    large_client_header_buffers  4 512k;
    client_max_body_size 200M;
     
    upstream FR.com {
        server 10.120.133.95:8075 max_fails=15 fail_timeout=300s;
        server 10.120.133.96:8075 max_fails=15 fail_timeout=300s;
        keepalive 300;

        #<---------------ngx_healthcheck_module--------------->#
        #check interval=2000 rise=5 fall=10 timeout=10000 type=http;
        #check_http_send "GET /webroot/decision/system/health HTTP/1.0\r\n\r\n";
        #check_http_expect_alive http_2xx http_3xx;
        #<---------------ngx_healthcheck_module--------------->#
    }

    upstream WBS.com { 
        server 10.120.133.95:38888 max_fails=15 fail_timeout=300s;
        server 10.120.133.96:38888 max_fails=15 fail_timeout=300s;
        # 这里必须使用 ip_hash
        ip_hash;
    }

    server {
        listen       80;
        server_name  localhost;
        underscores_in_headers on;
        #charset koi8-r;
        #access_log  logs/host.access.log  main;
    
        #↓====================主动健康检查模块监测页面====================↓#
        #location /status {
        #    healthcheck_status html;
        #}
        #↑====================主动健康检查模块监测页面====================↑#
    
        location / {
            proxy_http_version 1.1;
            proxy_pass http://FR.com;
            proxy_next_upstream http_500 http_502 http_503 http_504 http_403 http_404 http_429 error timeout invalid_header non_idempotent;
            proxy_redirect off;

            # 如果有外网映射，这里应改写 外网地址:外网端口 形式（不要带协议）
            proxy_set_header Host $host:$server_port;
            proxy_set_header X-Real-IP $remote_addr;
            proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;

            #proxy_buffering off;
            #proxy_buffer_size     64k;
            #proxy_buffers         32 64k;
            #proxy_busy_buffers_size 64k;

            proxy_connect_timeout    75;
            proxy_read_timeout       400;
            proxy_send_timeout       400;
            proxy_set_header Upgrade $http_upgrade;
	        proxy_set_header Connection $http_connection;
        }

        # 定义 404 页面
        error_page  404              /404.html;

        # 定义 50x 页面
        error_page   500 502 503 504  /50x.html;
        location = /50x.html {
            root   html;
        }
    }

	# 如果支持容器 websocket 方案，以下两段可以不用配置
    server {  
        listen 38889;              
        server_name localhost;

        location / { 
            proxy_http_version 1.1;
            proxy_pass http://WBS.com;
            proxy_connect_timeout 75;
            proxy_read_timeout 400;
            proxy_send_timeout 400;

            # 升级目标为 $http_upgrade 值实际为websocket
            proxy_set_header Upgrade $http_upgrade;
            proxy_set_header Connection "upgrade";
        }
    }
}
EOF