#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称，安装目录定义
PACKAGE=solr-${SOLR_VER}.tgz
INS_DIR=${PACKAGE%%-*}

# 检测安装状态
SOLR_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${SOLR_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 解压安装包
mkdir -p ${SOLR_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${SOLR_HOME}
_log "解压 >>> ${PACKAGE}"
rm -rf ${SOLR_HOME}/server/solr

# 准备工程文件
PACKAGE=solr-core.tgz
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

tar xf ${PACKAGE_DIR}/${PACKAGE} -C ${SOLR_HOME}/server
_log "解压工程文件 ${PACKAGE} ."

# 修改数据库配置
MYSQL_USER=root
MYSQL_SERVER="${MYSQL[0]}:3306"
MYSQL_PASSWORD="${MYSQL_PASS}"
sed -i "s#MYSQL_USER#$MYSQL_USER#" ${SOLR_HOME}/server/solr/new_core/conf/solr-data-config.xml
sed -i "s#MYSQL_SERVER#$MYSQL_SERVER#" ${SOLR_HOME}/server/solr/new_core/conf/solr-data-config.xml
sed -i "s#MYSQL_PASSWORD#$MYSQL_PASSWORD#" ${SOLR_HOME}/server/solr/new_core/conf/solr-data-config.xml
_log "配置 solr 数据库为 $MYSQL_SERVER"

# 配置服务
. /etc/profile &>/dev/null
cat <<EOF  > /usr/lib/systemd/system/solr.service
[Unit]
Description=solr
After=network.target

[Service]
LimitCORE=infinity
LimitNPROC=1000000
LimitNOFILE=1000000
Restart=always
RestartSec=3
TimeoutSec=30
Type=forking
Environment="FORMAT_MESSAGES_PATTERN_DISABLE_LOOKUPS=true"
Environment="JAVA_HOME=${JAVA_HOME}"
WorkingDirectory=${SOLR_HOME}
ExecStart=/bin/bash bin/solr start -p 10010 -force
SuccessExitStatus=143

[Install]
WantedBy=multi-user.target
EOF

systemctl daemon-reload
_log "配置系统服务 ."

add_service_to_rc_local solr
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl restart solr
_log "启动 solr 服务 ."