#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装 redis-server
if rpm -qa | grep -q redis; then
    _info "本机已安装 $(rpm -qa redis) ."
    exit 0
else
    true && _log "安装 redis ."
    dnf install -y redis &>/dev/null
fi

# 配置 redis 监听地址及密码
sed -i "/^requirepass/d" /etc/redis/redis.conf
sed -i "s#^bind .*#bind 0.0.0.0#" /etc/redis/redis.conf
sed -i "/^# requirepass foobared/a\requirepass ${REDIS_PASS}" /etc/redis/redis.conf

# 配置服务
cat <<EOF  > /usr/lib/systemd/system/redis.service
[Unit]
Description=Redis persistent key-value database
After=network.target
After=network-online.target
Wants=network-online.target

[Service]
ExecStart=/usr/bin/redis-server /etc/redis/redis.conf --daemonize no --supervised systemd
ExecStop=/usr/libexec/redis-shutdown
Type=notify
User=redis
Group=redis
RuntimeDirectory=redis
RuntimeDirectoryMode=0755

LimitCORE=infinity
LimitNPROC=1000000
LimitNOFILE=1000000
Restart=always
RestartSec=3
TimeoutSec=10
MemoryMax=2048M

[Install]
WantedBy=multi-user.target
EOF

systemctl daemon-reload
_log "配置系统服务 ."

add_service_to_rc_local redis
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl restart redis
_log "启动 redis 服务 ."