#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称
PACKAGE=emqx-${EMQX_VER}-el9-amd64.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
EMQX_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${EMQX_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 创建服务运行用户
if ! id emqx &>/dev/null; then
    groupadd -g 617 -o -r emqx
    useradd -N -g emqx -o -r -M -s /sbin/nologin -c "EMQX Server" -u 617 emqx
    _log "创建服务运行用户 ."
fi

# 安装依赖
dnf install -y libatomic &>/dev/null
_log "安装依赖 libatomic ."

# 配置 EMQX 环境变量
update_env_marked EMQX_ENV "$(cat <<EOF
EMQX_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$EMQX_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 解压服务端
mkdir -p ${EMQX_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} -C ${EMQX_HOME}
_log "解压 >>> ${PACKAGE}"

# 配置 数据/日志 目录权限
chown -Rf emqx:emqx ${EMQX_HOME}/{data,log}

# 修改节点名称
sed -i "s#emqx@127.0.0.1#emqx@${LAN_IP_ADDRESS}#" ${EMQX_HOME}/etc/emqx.conf

# 配置服务
cat <<EOF  > /usr/lib/systemd/system/emqx.service 
[Unit]
Description=emqx daemon
After=network.target

[Service]
User=emqx
Group=emqx
Type=forking
Restart=always
LimitNOFILE=1048576
Environment="HOME=${EMQX_HOME}"
Environment="EMQX_NODE__COOKIE=emqxsecretcookie-hc"
ExecStart=/bin/sh ${EMQX_HOME}/bin/emqx start
ExecStop=/bin/sh ${EMQX_HOME}/bin/emqx stop

[Install]
WantedBy=multi-user.target
EOF

_log "配置系统服务 ."
systemctl daemon-reload

add_service_to_rc_local emqx
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl restart emqx
_log "启动 EMQX 服务 ."