#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称
PACKAGE=TDengine-server-${TDENGINE_VER}-Linux-x64.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
if command -v taosd &>/dev/null; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 准备软件安装包
if [ ! -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
else
    true && _info "本机已存在软件包 ${PACKAGE} ."
fi

# 修改系统信息
. /etc/os-release
if [ "$NAME" != "CentOS Linux" ]; then
    sed -i '/### START MARKER ###/,/### END MARKER ###/d' /etc/os-release
    sed -i '1i### START MARKER ###\nVERSION_ID="9"\nNAME="CentOS Linux"\nPRETTY_NAME="CentOS Linux 9 (Core)"\n### END MARKER ###' /etc/os-release
fi

TDENGINE_INS_TMP=/tmp/${INS_DIR:-InstallDir}
mkdir -p ${TDENGINE_INS_TMP}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${TDENGINE_INS_TMP}
_log "解压 >>> ${PACKAGE} ."

cd ${TDENGINE_INS_TMP}
_log "安装 tdengine ."
./install.sh -e no &>/tmp/taos.log

if grep -q successfully /tmp/taos.log; then
    _log "安装成功 ."
    cd ~ && rm -rf ${TDENGINE_INS_TMP} /tmp/taos.log
else
    _log "安装失败，请检查安装日志 /tmp/taos.log ."
    exit 1
fi

# 配置 tasod
#firstEp="$(awk "/${TDENGINE[0]}/{print \$2}" /etc/hosts)"
firstEp=$(awk -v host=${TDENGINE[0]} 'index($0, host){print $2}' /etc/hosts)
sed -i '/^fqdn/d' /etc/taos/taos.cfg
sed -i '/^firstEp/d' /etc/taos/taos.cfg
sed -i '/^fqdn/d' /etc/taos/taos.cfg
sed -i "/# fqdn/a\fqdn                        $(hostname -f)" /etc/taos/taos.cfg
sed -i "/# firstEp/a\firstEp                     ${firstEp}:6030" /etc/taos/taos.cfg
_log "配置 tdengine，firstEp: ${firstEp}:6030   fqdn: $(hostname -f) ."

add_service_to_rc_local taosd
add_service_to_rc_local taosadapter
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl start taosd taosadapter
_log "启动 taosd taosadapter 服务 ."


### 初始化
# # 等待服务准备就绪  示例调用
# ## wait_for_tdengine "mnode" "192.168.1.100"  # 传入 mnode 和 IP 地址
# ## wait_for_tdengine "dnode" "192.168.1.101"  # 传入 dnode 和 IP 地址
# wait_for_tdengine() {
#     node_type=$1
#     ip_address=$2
#     tdengine_ip=${3:-127.0.0.1}

#     # 获取节点的主机名
#     node_hostname=$(lookup_ip_hostname $ip_address)

#     node_state=False
#     if [ "$node_type" == "mnode" ]; then
#         # 如果是 mnode，查询 mnodes 状态
#         until [ "$node_state" == "ready" ]; do
#             true && _info "等待 tdengine 服务准备就绪  $node_hostname (来自 $tdengine_ip mnode 查询) ."
#             node_state=$(taos -h $tdengine_ip -s "show mnodes;" | awk -v node="${node_hostname}:6030" '{if ($3 == node) print $7}')
#             sleep 2
#         done
#         _info "服务 tdengine 准备就绪 ."
#     elif [ "$node_type" == "dnode" ]; then
#         # 如果是 dnode，查询 dnodes 状态
#         until [ "$node_state" == "ready" ]; do
#             true && _info "等待 tdengine 服务准备就绪  $node_hostname (来自 $tdengine_ip dnode 查询) ."
#             node_state=$(taos -h $tdengine_ip -s "show dnodes;" | awk -v node="${node_hostname}:6030" '{if ($3 == node) print $9}')
#             sleep 2
#         done
#         _info "服务 tdengine 准备就绪 ."
#     else
#         _info "未知节点类型 $node_type, 请提供 mnode 或 dnode."
#         exit 1
#     fi
# }

# ## 删除 dnode   taos -s "drop dnode 3;"
# ## 删除 mnode   taos -s "drop mnode on dnode 3;"
# # 如果是集群，第一个节点
# if [ "$LAN_IP_ADDRESS" == "${TDENGINE[0]}" ]; then
#     # 检查服务是否准备就绪
#     wait_for_tdengine "mnode" "${TDENGINE[0]}"

#     # 添加 dnode 节点
#     IFS=,; for dnode in $(lookup_ip_hostname ${TDENGINE[@]}); do
#         # 检查主节点准备就绪
#         wait_for_tdengine "mnode" "${TDENGINE[0]}"

#         # 检查需要添加的 dnode 节点准备就绪
#         wait_for_tdengine "dnode" "${dnode}" "${dnode}"

#         taos -s "create dnode '${dnode}:6030';" &>/dev/null
#         _info "添加节点 ${dnode}:6030 角色 dnode ."

#         wait_for_tdengine "dnode" "${dnode}"
#         _log "节点 ${dnode} 集群角色 dnode, 状态为 ready ."
#     done

#     green_echo ">>> dnodes 节点信息 ."
#     taos -s "show dnodes;"

#     # 加入 mnode 角色
#     mnode_id=1
#     for mnode in $(lookup_ip_hostname ${TDENGINE[@]}); do
#         wait_for_tdengine "dnode" "${mnode}"
#         _log "节点 ${mnode} 集群状态检查通过,角色 dnode, 状态为 ready ."

#         taos -s "create mnode on dnode ${mnode_id};" &>/dev/null
#         _info "节点 ${mnode} 加入 mnode 角色 ."

#         wait_for_tdengine "mnode" "${mnode}"
#         _log "节点 ${mnode} 集群状态检查通过,角色 mnode, 状态为 ready ."

#         let mnode_id++
#     done
#     green_echo ">>> mnodes 节点信息 ."
#     taos -s "show mnodes;"

#     # 创建数据库
#     green_echo ">>> 创建数据库 ."
#     taos -s "CREATE DATABASE mango BUFFER 256 CACHESIZE 1 CACHEMODEL 'last_row' COMP 2 DURATION 14400m WAL_FSYNC_PERIOD 3000 MAXROWS 4096 MINROWS 100 STT_TRIGGER 1 KEEP 36500d PAGES 256 PAGESIZE 4 PRECISION 'ms' REPLICA 3 WAL_LEVEL 1 VGROUPS 8 SINGLE_STABLE 0 TABLE_PREFIX 0 TABLE_SUFFIX 0 TSDB_PAGESIZE 4 WAL_RETENTION_PERIOD 3600 WAL_RETENTION_SIZE 0 KEEP_TIME_OFFSET 0;"
#     #taos -s "show create database mango\G;"
#     green_echo ">>> show vnodes ."
#     taos -s "show vnodes;"
# fi


# taos -s "CREATE DATABASE mango BUFFER 256 CACHESIZE 1 CACHEMODEL 'last_row' COMP 2 DURATION 14400m WAL_FSYNC_PERIOD 3000 MAXROWS 4096 MINROWS 100 STT_TRIGGER 1 KEEP 36500d PAGES 256 PAGESIZE 4 PRECISION 'ms' REPLICA 3 WAL_LEVEL 1 VGROUPS 8 SINGLE_STABLE 0 TABLE_PREFIX 0 TABLE_SUFFIX 0 TSDB_PAGESIZE 4 WAL_RETENTION_PERIOD 3600 WAL_RETENTION_SIZE 0 KEEP_TIME_OFFSET 0;"
# taos -s "show create database mango\G;"
# taos -s "show vnodes;"