#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 安装包名称
PACKAGE=hadoop-${HADOOP_VER}.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
HADOOP_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${HADOOP_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 配置 HADOOP 环境变量
update_env_marked HADOOP_ENV "$(cat <<EOF
HADOOP_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
HADOOP_MAPRED_HOME=\$HADOOP_HOME
pathmunge \$HADOOP_HOME/bin
pathmunge \$HADOOP_HOME/sbin
export HADOOP_HOME HADOOP_MAPRED_HOME
export HADOOP_CLASSPATH=\`hadoop classpath\`
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${HADOOP_HOME}/logs
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${HADOOP_HOME}
_log "解压 >>> ${PACKAGE}"

# 创建 hadoop 配置文件
. /etc/profile &>/dev/null
listIniKeysValues hadoop environment

MASTER_NODE=$(lookup_ip_hostname ${HADOOP[0]})
cat > /tmp/hadoop.env <<EOF
CORE_CONF_fs_defaultFS=hdfs://${MASTER_NODE}:8020
CORE_CONF_fs_trash_interval=1440
CORE_CONF_io_file_buffer_size=13107200
CORE_CONF_hadoop_tmp_dir=${HADOOP_HOME}/namesecondary

HDFS_CONF_dfs_replication=3
HDFS_CONF_dfs_namenode_name_dir=${HADOOP_HOME}/namenode
HDFS_CONF_dfs_datanode_data_dir=${HADOOP_HOME}/datanode
HDFS_CONF_dfs_permissions_enalbed=false
HDFS_CONF_dfs_namenode_http___address=${MASTER_NODE}:9870
HDFS_CONF_dfs_namenode_secondary_http___address=${MASTER_NODE}:50090

YARN_CONF_yarn_nodemanager_aux___services=mapreduce_shuffle
YARN_CONF_yarn_yarn_resourcemanager_hostname=${MASTER_NODE}
YARN_CONF_yarn_resourcemanager_scheduler_address=${MASTER_NODE}:8030
YARN_CONF_yarn_resourcemanager_resource___tracker_address=${MASTER_NODE}:8031
YARN_CONF_yarn_resourcemanager_address=${MASTER_NODE}:8032
YARN_CONF_yarn_resourcemanager_admin_address=${MASTER_NODE}:8033
YARN_CONF_yarn_resourcemanager_webapp_address=${MASTER_NODE}:8088
YARN_CONF_yarn_nodemanager_vmem___check___enabled=false

MAPRED_CONF_mapreduce_framework_name=yarn
MAPRED_CONF_mapreduce_jobhistory_address=${MASTER_NODE}:10020
MAPRED_CONF_mapreduce_jobhistory_webapp_address=${MASTER_NODE}:19888
MAPRED_CONF_mapred_child_java_opts=-Xmx2048m
MAPRED_CONF_mapreduce_map_memory_mb=2048
MAPRED_CONF_mapreduce_reduce_memory_mb=4096
MAPRED_CONF_mapreduce_map_java_opts=-Xmx1500m
MAPRED_CONF_mapreduce_reduce_java_opts=-Xmx3100m
EOF
_log "创建 hadoop 配置文件"

for line in $(cat /tmp/hadoop.env);do
    eval export $line
done
_log "读取模板变量设置为全局变量"


# 拼接 hadoop 配置
function addProperty() {
    local path=$1
    local name=$2
    local value=$3
    local entry="  <property>\n    <name>${name}</name>\n    <value>${value}</value>\n  </property>"
    local escapedEntry=$(printf '%s' "$entry" | sed 's/\//\\\//g')
    sed -i "/<\/configuration>/ s/.*/${escapedEntry}\n&/" $path
}

# 读取环境变量解析参数&& 参数值
function configure() {
    local path=$1
    local module=$2
    local envPrefix=$3
    local var value
    echo Configuring ${module}-site.xml

    while IFS='=' read -r line; do
        if [[ $line =~ ^${envPrefix}_([^=]+) ]]; then
            c=${BASH_REMATCH[1]}
            name=${c//___/-}
            name=${name//__/@}
            name=${name//_/.}
            name=${name//@/_}
            value=${!line}
            echo " - Setting $name=$value"
            addProperty ${HADOOP_HOME}/etc/hadoop/${module}-site.xml $name $value
            eval unset "$line"
        fi
    done < <(sort -r < <(grep "^$envPrefix" < <(compgen -v)))

    # while IFS='=' read -r line; do
    #     #if [[ $line =~ ^${envPrefix}_(.+)= ]]; then
    #     if [[ $line =~ ^${envPrefix}_([^=]+)=(.*)$ ]]; then
    #         c=${BASH_REMATCH[1]}
    #         name=${c//___/-}
    #         name=${name//__/@}
    #         name=${name//_/.}
    #         name=${name//@/_}
    #         var="${envPrefix}_${c}"
    #         value=${!var}
    #         echo " - Setting $name=$value"
    #         addProperty ${HADOOP_HOME}/etc/hadoop/${module}-site.xml $name $value
    #         eval unset "${line%%=*}"
    #     fi
    # done < <(printenv)
}


# 备份默认配置
/bin/cp -r ${HADOOP_HOME}/etc ${HADOOP_HOME}/etc.bak
[ -f "${HADOOP_HOME}/etc/hadoop/mapred-site.xml" ] || cp ${HADOOP_HOME}/etc/hadoop/mapred-site.xml.template ${HADOOP_HOME}/etc/hadoop/mapred-site.xml

# 配置 hadoop 环境变量
cat >> ${HADOOP_HOME}/etc/hadoop/hadoop-env.sh <<EOF
# mango  $(date '+%F %T')
export HADOOP_IDENT_STRING=root
export HADOOP_SECURE_DN_USER=root
export HADOOP_PID_DIR=./pid
export HADOOP_SECURE_DN_PID_DIR=./pid
EOF

# 配置 yarn 环境变量
cat >> ${HADOOP_HOME}/etc/hadoop/yarn-env.sh <<EOF
# mango  $(date '+%F %T')
export HADOOP_YARN_USER=root
export YARN_IDENT_STRING=root
export YARN_PID_DIR=./pid
EOF

# 配置 mapred 环境变量
cat >> ${HADOOP_HOME}/etc/hadoop/mapred-env.sh <<EOF
# mango  $(date '+%F %T')
export HADOOP_MAPRED_IDENT_STRING=root
export HADOOP_MAPRED_PID_DIR=./pid
EOF

# 生成 hadoop 配置
if [ ! -f ${HADOOP_HOME}/etc/.init ]; then
    configure ${HADOOP_HOME}/etc/hadoop/core-site.xml core CORE_CONF
    configure ${HADOOP_HOME}/etc/hadoop/hdfs-site.xml hdfs HDFS_CONF
    configure ${HADOOP_HOME}/etc/hadoop/yarn-site.xml yarn YARN_CONF
    configure ${HADOOP_HOME}/etc/hadoop/mapred-site.xml mapred MAPRED_CONF
    _log "生成配置文件 core-site.xml hdfs-site.xml yarn-site.xml mapred-site.xm ."
    echo "# $(date +'%F %T') core-site.xml hdfs-site.xml yarn-site.xml mapred-site.xm" > ${HADOOP_HOME}/etc/.init
fi

# hadoop  
if [ "$LAN_IP_ADDRESS" == "${HADOOP[0]}" ]; then
    # 配置 systemctl 启动 namenode
    cat > /usr/lib/systemd/system/hdp-namenode.service <<-EOF
	[Unit]
	Description=hadoop namenode
	After=syslog.target network.target

	[Service]
	LimitCORE=infinity
	LimitNPROC=1000000
	LimitNOFILE=1000000
	Restart=always
	RestartSec=3
	TimeoutSec=10
	Type=forking
	User=root
	Group=root
	Environment="JAVA_HOME=${JAVA_HOME}" "JRE_HOME=${JRE_HOME}"
	WorkingDirectory=${HADOOP_HOME}
	ExecStart=/bin/bash sbin/hadoop-daemon.sh start namenode
	ExecStop=/bin/bash sbin/hadoop-daemon.sh stop namenode
	SuccessExitStatus=143

	[Install]
	WantedBy=multi-user.target
	EOF

    # 配置 systemctl 启动 resourcemanager
    cat > /usr/lib/systemd/system/hdp-resourcemanager.service <<-EOF
	[Unit]
	Description=hadoop resourcemanager
	After=syslog.target network.target
	Wants=hdp-namenode.service

	[Service]
	LimitCORE=infinity
	LimitNPROC=1000000
	LimitNOFILE=1000000
	Restart=always
	RestartSec=3
	TimeoutSec=10
	Type=forking
	User=root
	Group=root
	Environment="JAVA_HOME=${JAVA_HOME}"
	WorkingDirectory=${HADOOP_HOME}
	ExecStart=/bin/bash sbin/yarn-daemon.sh start resourcemanager
	ExecStop=/bin/bash sbin/yarn-daemon.sh stop resourcemanager
	SuccessExitStatus=143

	[Install]
	WantedBy=multi-user.target
	EOF

    _log "配置系统服务 ."
    systemctl daemon-reload

    add_service_to_rc_local hdp-namenode
    add_service_to_rc_local hdp-resourcemanager
    _log "添加服务启动命令到 /etc/rc.d/rc.local ."
fi

# hadoop 所有节点
if contains "$LAN_IP_ADDRESS" HADOOP[@]; then
    # 配置 systemctl 启动 datanode
    cat > /usr/lib/systemd/system/hdp-datanode.service <<-EOF
	[Unit]
	Description=hadoop datanode
	After=syslog.target network.target

	[Service]
	LimitCORE=infinity
	LimitNPROC=1000000
	LimitNOFILE=1000000
	Restart=always
	RestartSec=3
	TimeoutSec=10
	Type=forking
	User=root
	Group=root
	Environment="JAVA_HOME=${JAVA_HOME}"
	WorkingDirectory=${HADOOP_HOME}
	ExecStart=/bin/bash sbin/hadoop-daemon.sh start datanode
	ExecStop=/bin/bash sbin/hadoop-daemon.sh stop datanode
	SuccessExitStatus=143

	[Install]
	WantedBy=multi-user.target
	EOF

    # 配置 systemctl 启动 nodemanager
    cat > /usr/lib/systemd/system/hdp-nodemanager.service <<-EOF
	[Unit]
	Description=hadoop nodemanager
	After=syslog.target network.target
	Wants=hdp-datanode.service

	[Service]
	LimitCORE=infinity
	LimitNPROC=1000000
	LimitNOFILE=1000000
	Restart=always
	RestartSec=3
	TimeoutSec=10
	Type=forking
	User=root
	Group=root
	Environment="JAVA_HOME=${JAVA_HOME}"
	WorkingDirectory=${HADOOP_HOME}
	ExecStart=/bin/bash sbin/yarn-daemon.sh start nodemanager
	ExecStop=/bin/bash sbin/yarn-daemon.sh stop nodemanager
	SuccessExitStatus=143

	[Install]
	WantedBy=multi-user.target
	EOF

    _log "配置系统服务 ."
    systemctl daemon-reload

    add_service_to_rc_local hdp-datanode
    add_service_to_rc_local hdp-nodemanager
    _log "添加服务启动命令到 /etc/rc.d/rc.local ."
fi

# hadoop 最后一个节点
if [ "$LAN_IP_ADDRESS" == "${HADOOP[-1]}" ]; then
    # 配置 systemctl 启动 secondarynamenode
    cat > /usr/lib/systemd/system/hdp-secondarynamenode.service <<-EOF
	[Unit]
	Description=hadoop secondarynamenode
	After=syslog.target network.target
	Wants=hdp-namenode.service

	[Service]
	LimitCORE=infinity
	LimitNPROC=1000000
	LimitNOFILE=1000000
	Restart=always
	RestartSec=3
	TimeoutSec=10
	Type=forking
	User=root
	Group=root
	Environment="JAVA_HOME=${JAVA_HOME}"
	WorkingDirectory=${HADOOP_HOME}
	ExecStart=/bin/bash sbin/hadoop-daemon.sh start secondarynamenode
	ExecStop=/bin/bash sbin/hadoop-daemon.sh stop secondarynamenode
	SuccessExitStatus=143

	[Install]
	WantedBy=multi-user.target
	EOF

    _log "配置系统服务 ."
    systemctl daemon-reload

    add_service_to_rc_local hdp-secondarynamenode
    _log "添加服务启动命令到 /etc/rc.d/rc.local ."
fi

# # hadoop 初始化
# if [ "$LAN_IP_ADDRESS" == "${HADOOP[0]}" ]; then
#     if [ ! -d "${HADOOP_HOME}/namenode" ]; then 
#         hdfs namenode -format --force &>/tmp/namenode-format.out
#         _log "格式化 namenode ."
#     fi
# fi