#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 安装包名称
PACKAGE=TDengine-server-${TDENGINE_VER}-Linux-x64.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
if command -v taosd &>/dev/null; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 准备软件安装包
if [ ! -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
else
    true && _info "本机已存在软件包 ${PACKAGE} ."
fi

TDENGINE_INS_TMP=/tmp/${INS_DIR}
mkdir -p ${TDENGINE_INS_TMP}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${TDENGINE_INS_TMP}
_log "解压 >>> ${PACKAGE}"

cd ${TDENGINE_INS_TMP}
_log "安装 tdengine ."
./install.sh -e no &>/tmp/taos.log

if grep -q successfully /tmp/taos.log; then
    _log "安装成功 ."
    cd ~ && rm -rf ${TDENGINE_INS_TMP}
    rm -f /tmp/taos.log
else
    _log "安装失败，请检查安装日志 /tmp/taos.log ."
fi

listIniKeysValues tdengine environment
firstEp="$(awk "/${TDENGINE[0]}/{print \$2}" /etc/hosts)"
sed -i '/^fqdn/d' /etc/taos/taos.cfg
sed -i '/^firstEp/d' /etc/taos/taos.cfg
sed -i '/^fqdn/d' /etc/taos/taos.cfg
sed -i "/# fqdn/a\fqdn                        $(hostname -f)" /etc/taos/taos.cfg
sed -i "/# firstEp/a\firstEp                     ${firstEp}:6030" /etc/taos/taos.cfg
_log "配置 tdengine，firstEp: ${firstEp}:6030   fqdn: $(hostname -f) ."