#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 加载区域变量
setup_environment_from_config

# 检测 HOSTS,SERVERS 数组个数
if [ "${#HOSTS[@]}" != "${#SERVERS[@]}" ]; then
    _err "HOSTS与SERVERS 数组个数不匹配 ."
    exit 0
fi

# 1.初始化
if [ "$(grep -Fx "$LAN_IP_ADDRESS" < <(sed 's/ /\n/g' <<<${SERVERS[@]}))" ]; then
    ${basedir}/01init.sh
else
    _err "当前主机 [$LAN_IP_ADDRESS] 不存在于安装列表中 ."
    exit 2
fi

# 2.MySQL
if [ "$(host_ip ${MYSQL_NODE} | grep $LAN_IP_ADDRESS)" ]; then
    ${basedir}/02mysql.sh
    if [ "$(host_ip ${MYSQL_NODE} | awk -F ',' '{print $1}' | grep $LAN_IP_ADDRESS)" ]; then
        ${basedir}/021mysql-cluster.sh
    fi
fi

# 3.Nginx
if [ "$(host_ip ${NGINX_NODE} | grep $LAN_IP_ADDRESS)" ]; then
    ${basedir}/03nginx.sh
    ${basedir}/04keepalived.sh
fi

# 4.Doris FE
if [ "$(host_ip ${FE_NODE} | grep $LAN_IP_ADDRESS)" ]; then
    ${basedir}/05doris-fe.sh
fi

# 5.Doris BE
if [ "$(host_ip ${BE_NODE} | grep $LAN_IP_ADDRESS)" ]; then
    ${basedir}/05doris-be.sh
fi

# 6.Redis
if [ "$(host_ip ${REDIS_NODE} | grep $LAN_IP_ADDRESS)" ]; then
    ${basedir}/06redis.sh
fi

# 7.NFS
if [ "$(host_ip ${NFS_NODE} | grep $LAN_IP_ADDRESS)" ]; then
    ${basedir}/07nfs.sh
fi

# 8.Kettle
if [ "$(host_ip ${KETTLE_NODE} | grep $LAN_IP_ADDRESS)" ]; then
    ${basedir}/08kettle.sh
fi

# 9.DataEase
if [ "$(host_ip ${DE_NODE} | grep $LAN_IP_ADDRESS)" ]; then
    ${basedir}/09de.sh
fi
