#!/bin/bash

# 获取当前路径
basedir=$(cd "$(dirname $0)"; pwd)

# 读取配置文件
if [ -f "${basedir}/config" ]; then
    . ${basedir}/config
else
    echo "找不到配置文件 ${basedir}/config ."
    exit 1
fi

if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

function chk_mysql () {
    title "检测 MySQL 服务状态 ."
    for node in $MYSQL_NODE; do
        node=$(host_ip $node)
        _info "$node 查看主从运行状态 ."
        ssh -T ${node} <<< "mysql -e 'show slave status\G' | grep _Running: | sed ':a;N;s/\n/ /g;ta'"
    done
}

function chk_redis () {
    title "检测 Redis 服务状态 ."
    for node in $REDIS_NODE; do
        node=$(host_ip $node)
        _info "$node 查看 Redis 集群状态 ."
	ssh -T ${node} <<< "redis-cli -h $node -a admin123456 -c cluster nodes"
    done 
}

function chk_doris () {
    title "检测 Doris Frontends 服务状态 ."
    ssh -T ${FE_NODE} <<< "mysql -uroot -h127.0.0.1 -P9030 -e \"show proc '/frontends'\G;\" | egrep 'row|IP|HostName|Alive|Last|IsMaster'"
    title "检测 Doris Backends 服务状态 ."
    ssh -T ${FE_NODE} <<< "mysql -uroot -h127.0.0.1 -P9030 -e \"show proc '/backends'\G;\" | egrep 'row|IP|HostName|Alive|Last'"
}

name=$1

case $name in
    ($name)
        chk_${name} ;;
esac


grep -oP 'start_time=\K\[\K[^]]+(?=\])' < <(systemctl show categraf.service --property=ExecStart)