#!/bin/bash

cat <<EOF  > /dev/null
# 用法
mv /etc/yum.repos.d/epel.repo /tmp/
curl -s http://script.wh.hc-yun.com:3380/install/chrony.sh | bash -s -- client
#curl -s http://script.wh.hc-yun.com:3380/install/chrony.sh | bash -s -- server
systemctl status chronyd
chronyc sources -v
mv /tmp/epel.repo /etc/yum.repos.d/epel.repo

# windows 激活
# slmgr /skms kms.jihuowin.com && slmgr /ato

# windows
# w32tm /config /manualpeerlist:192.168.2.5 /syncfromflags:manual /update


# windows chrony client   https://ghfast.top/https://github.com/flashcatcloud/categraf/releases/download/v0.4.3/categraf-v0.4.3-windows-amd64.zip
EOF

#curl -s http://script.wh.hc-yun.com:3380/install/yum-centos-7.sh | bash -s

# 默认 NTP 服务器地址
DEFAULT_NTP_SERVER="ntp.aliyun.com ntp.tencent.com cn.pool.ntp.org time.ustc.edu.cn ntp.ntsc.ac.cn"

# 默认 Chronyd 服务器根据网络 IP 地址进行匹配
DEFAULT_CHRONYD_SERVER="ntp.aliyun.com ntp.tencent.com cn.pool.ntp.org time.ustc.edu.cn ntp.ntsc.ac.cn"

# 检查是否以 root 用户运行
if [ "$EUID" -ne 0 ]; then
    echo "请以 root 用户运行此脚本"
    exit 1
fi

# 检查参数
if [ $# -lt 1 ]; then
    echo "用法: $0 <server|client> [NTP 服务器地址或 Chronyd 服务器 IP]"
    exit 1
fi

# 获取默认上网网卡和 IP 地址
get_default_network() {
    # 获取默认网卡
    #ip route get 1.1.1.1 | awk '{print $5}'

    # 获取默认网卡 IP 地址
    ip route get 1.1.1.1 | awk '{for(i=1;i<=NF;i++) if($i=="src") print $(i+1); exit}'
}

# 根据 IP 地址获取默认 Chronyd 服务器
get_default_chronyd_server() {
    local host_ip=$(get_default_network)
    case ${host_ip%.*} in
        10.34.44)     echo "10.34.44.41"     ;;
        10.34.45)     echo "10.34.45.41"     ;;
        172.168.100)  echo "172.168.100.41"  ;;
        192.168.2)    echo "192.168.2.5"     ;;
        192.168.10)   echo "192.168.2.5"     ;;
        192.168.30)   echo "192.168.30.15"   ;;
        *)            echo "$DEFAULT_CHRONYD_SERVER" ;;
    esac
}

# 安装 chrony
install_chrony() {
    if ! command -v chronyd &> /dev/null; then
        echo "正在安装 chrony..."
        if command -v yum &> /dev/null; then
            yum install -y chrony
            [ $? -ne 0 ] && { echo "软件包安装失败, 退出安装 ."; exit 1;}
        elif command -v apt-get &> /dev/null; then
            apt-get update
            apt-get install -y chrony
            [ $? -ne 0 ] && { echo "软件包安装失败, 退出安装 ."; exit 1;}
        else
            echo "无法确定包管理器，请手动安装 chrony"
            exit 1
        fi
    else
        echo "chrony 已安装"
    fi
}

# 配置 chronyd 服务端
configure_server() {
    echo "正在配置 chronyd 服务端..."

    # 备份/恢复默认配置
    if [ -f /etc/chrony.conf.default ]; then
        cat /etc/chrony.conf.default > /etc/chrony.conf
    elif [ -f /etc/chrony.conf ]; then
        cat /etc/chrony.conf > /etc/chrony.conf.default
    fi

    # 配置 NTP 服务器
    sed -i '/^#\?pool /d; /^#\?server /d' /etc/chrony.conf
    i=2
    for server in $NTP_SERVER; do
        sed -i "${i}a server $server iburst" /etc/chrony.conf
        let i++
    done

    # 修改服务端配置
    sed -i "s@^#\?allow .*@allow 0.0.0.0/0@"  /etc/chrony.conf
    sed -i "s@^#\?local stratum .*@local stratum 10@" /etc/chrony.conf

    # 启动并配置 chrony 服务
    systemctl restart chronyd
    systemctl enable chronyd

    # 立即强制同步
    chronyc makestep > /dev/null

    # 手动触发同步
    chronyc tracking > /dev/null

    # 查看同步状态
    chronyc sources -v

    echo "chronyd 服务端已配置并启动 . 上游地址为 [$NTP_SERVER] ."
    echo "查看连接的客户端 >>> chronyc clients"
}

# 配置 chronyd 客户端
configure_client() {
    echo "正在配置 chronyd 客户端..."

    # 备份/恢复默认配置
    if [ -f /etc/chrony.conf.default ]; then
        cat /etc/chrony.conf.default > /etc/chrony.conf
    elif [ -f /etc/chrony.conf ]; then
        cat /etc/chrony.conf > /etc/chrony.conf.default
    fi

    # 使用 sed 插入 Chronyd 服务器
    sed -i '/^#\?pool /d; /^#\?server /d' /etc/chrony.conf
    i=2
    for server in $CHRONYD_SERVER; do
        sed -i "${i}a server $server iburst" /etc/chrony.conf
        let i++
    done

    # 启动并配置 chrony 服务
    systemctl restart chronyd
    systemctl enable chronyd

    # 立即强制同步
    chronyc makestep > /dev/null

    # 手动触发同步
    chronyc tracking > /dev/null

    # 查看同步状态
    chronyc sources -v

    echo "chronyd 客户端已配置并启动 . 上游地址为 [$CHRONYD_SERVER] ."
}

# 处理模式参数
MODE=$1
NTP_SERVER=${2:-$DEFAULT_NTP_SERVER}
CHRONYD_SERVER=${2:-$(get_default_chronyd_server)}

# 主逻辑
case $MODE in
    server)
        install_chrony
        configure_server
        ;;
    client)
        install_chrony
        configure_client
        ;;
    *)
        echo "无效模式: $MODE"
        echo "用法: $0 <server|client> [NTP 服务器地址或 Chronyd 服务器 IP]"
        exit 1
        ;;
esac