#!/bin/bash

# 获取当前路径
basedir=$(cd "$(dirname $0)"; pwd)

# 读取配置文件
if [ -f "${basedir}/config" ]; then
    . ${basedir}/config
else
    echo "找不到配置文件 ${basedir}/config ."
    exit 1
fi

if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

function mysql () {
    title "卸载 MySQL 服务 ."
    if [ "$1" == 'all' ]; then
        systemctl stop mysqld
        _info "停止服务 mysqld ."

        rm -rf ${SOFT_INSTALL_DIR}/mysql
        _info "删除数据目录 ${SOFT_INSTALL_DIR}/mysql ."

        rpm -e $(rpm -qa | grep mysql-community) &>/dev/null
        _info "卸载 mysql-community 软件包 ."

        sed -i '/start mysqld/d' /etc/rc.d/rc.local
        _log "清理 mysql 启动信息 /etc/rc.d/rc.local ."

	# 清理末尾空行
	sed -i ':n;/^\n*$/{$! N;$d;bn}' /etc/rc.d/rc.local
    else
        systemctl stop mysqld &>/dev/null
        systemctl disable mysqld &>/dev/null
        _info "停止服务 mysqld ."

        rm -rf ${SOFT_INSTALL_DIR}/mysql
        _info "删除数据目录 ${SOFT_INSTALL_DIR}/mysql ."
    fi
}

function nginx () {
    title "卸载 nginx 服务 ."

    systemctl stop nginx &>/dev/null
    pkill -9 nginx
    _info "关闭 nginx 服务 ."

    rpm -e nginx &>/dev/null
    _info "卸载 nginx 软件包 ."

    rm -rf /etc/nginx
    _info "清理配置文件 ."

    rm -rf /etc/nginx
    sed -i '/start nginx/d' /etc/rc.d/rc.local
    _log "清理 nginx 启动信息 /etc/rc.d/rc.local ."

    # 清理末尾空行
    sed -i ':n;/^\n*$/{$! N;$d;bn}' /etc/rc.d/rc.local
}

function keepalived () {
    title "卸载 keepalived 服务 ."

    systemctl stop keepalved &>/dev/null
    pkill -9 keepalived
    _info "关闭 keepalived 服务 ."

    rpm -e keepalived &>/dev/null
    _info "卸载 redis 软件包 ."

    rm -rf /etc/keepalived
    _info "清理配置文件 ."

    sed -i '/start keepalived/d' /etc/rc.d/rc.local
    _info "清理 keepalived 启动信息 /etc/rc.d/rc.local ."

    # 清理末尾空行
    sed -i ':n;/^\n*$/{$! N;$d;bn}' /etc/rc.d/rc.local
}

function redis () {
    title "卸载 redis 服务 ."

    systemctl stop redis &>/dev/null
    pkill -9 redis-server
    _info "关闭 redis 服务 ."

    rpm -e redis &>/dev/null
    _info "卸载 redis 软件包 ."

    rm -rf /var/lib/redis /var/log/redis /etc/redis
    sed -i '/start redis/d' /etc/rc.d/rc.local
    _info "清理 redis 启动信息 /etc/rc.d/rc.local ."

    # 清理末尾空行
    sed -i ':n;/^\n*$/{$! N;$d;bn}' /etc/rc.d/rc.local
}

function nfs () {
    title "卸载 nfs-server 服务 ."
    systemctl disable nfs-server.service &>/dev/null
    systemctl stop nfs-server.service &>/dev/null
    _info "禁用/关闭 nfs-server 服务 ."

    echo > /etc/exports
    _info "清空 nfs-server 配置文件 ."
}

function doris () {
    title "卸载 doris 服务 ."
    systemctl disable doris-fe doris-be &>/dev/null
    systemctl stop doris-fe doris-be &>/dev/null
    _info "关闭 redis 服务 ."

    rm -rf ${SOFT_INSTALL_DIR}/apache-doris-fe-${DORIS_VER}-bin
    rm -rf ${SOFT_INSTALL_DIR}/apache-doris-be-${DORIS_VER}-bin-x86_64 
    rm -f /usr/lib/systemd/system/doris-be.service /usr/lib/systemd/system/doris-fe.service &>/dev/null
    _info "删除软件/服务 ."

    sed -i '/start doris/d' /etc/rc.d/rc.local
    _info "清理 doris 启动信息 /etc/rc.d/rc.local ."

    # 清理末尾空行
    sed -i ':n;/^\n*$/{$! N;$d;bn}' /etc/rc.d/rc.local
}

function kettle () {
    title "卸载 kettle 服务 ."
    systemctl disable kettle &>/dev/null
    systemctl stop kettle &>/dev/null
    _info "关闭 kettle 服务 ."

    sed -i '/kettle\/data nfs/d' /etc/fstab
    _info "删除开机自动挂载 NFS ."

    umount ${SOFT_INSTALL_DIR}/kettle/data
    _info "取消 NFS 挂载 ."

    rm -f /usr/lib/systemd/system/kettle.service
    rm -rf ${SOFT_INSTALL_DIR}/kettle
    sed '/KETTLE_HOME=/d' /etc/profile
    _info "删除软件/服务 ."
    
    sed -i '/start kettle/d' /etc/rc.d/rc.local
    _info "清理 kettle 启动信息 /etc/rc.d/rc.local ."

    # 清理末尾空行
    sed -i ':n;/^\n*$/{$! N;$d;bn}' /etc/rc.d/rc.local
} 

function dataease () {
    /usr/local/bin/dectl uninstall
    _log "卸载 dataease 服务 ."
}

function uninstall () {
    # 1.MySQL
    if [ "$(host_ip ${MYSQL_NODE} | grep $LAN_IP_ADDRESS)" ]; then
        ${basedir}/02mysql.sh
        if [ "$(host_ip ${MYSQL_NODE} | awk -F ',' '{print $1}' | grep $LAN_IP_ADDRESS)" ]; then
            ${basedir}/clear.sh mysql all
        fi
    fi

    # 2.Nginx
    if [ "$(host_ip ${NGINX_NODE} | grep $LAN_IP_ADDRESS)" ]; then
        ${basedir}/clear.sh nginx
        ${basedir}/clear.sh keepalived
    fi

    # 3.Doris FE
    if [ "$(host_ip ${FE_NODE} | grep $LAN_IP_ADDRESS)" ]; then
        ${basedir}/clear.sh doris
    fi

    # 4.Doris BE
    if [ "$(host_ip ${BE_NODE} | grep $LAN_IP_ADDRESS)" ]; then
        ${basedir}/clear.sh doris
    fi

    # 5.Redis
    if [ "$(host_ip ${REDIS_NODE} | grep $LAN_IP_ADDRESS)" ]; then
        ${basedir}/clear.sh redis
    fi

    # 6.NFS
    if [ "$(host_ip ${NFS_NODE} | grep $LAN_IP_ADDRESS)" ]; then
        ${basedir}/clear.sh nfs
    fi

    # 7.Kettle
    if [ "$(host_ip ${KETTLE_NODE} | grep $LAN_IP_ADDRESS)" ]; then
        ${basedir}/clear.sh kettle
    fi

    # 8.DataEase
    if [ "$(host_ip ${DE_NODE} | grep $LAN_IP_ADDRESS)" ]; then
        ${basedir}/clear.sh dataease
    fi
}

name=$1

case $name in
    (all)
	uninstall
	;;
    ($name)
        $name $2
	;;
esac
