#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 加载函数
if [ -f "${basedir}/../function.sh" ]; then
    . ${basedir}/../function.sh
fi

# 加载环境变量
. /usr/local/.mango/environment.txt

title "📦️ 初始化、配置 Doris 集群 ."

# 执行前确认
echo -ne "\033[031;1m[$LAN_IP_ADDRESS] 警告：此操作将初始化并配置 Doris 集群，是否继续？(请输入Yes继续)：\033[0m"
read -r user_input
if [[ "${user_input}" != "Yes" ]]; then
	_info "用户取消操作，退出。"
	exit 1
else
    echo
fi

### 清理数据
# for node in $DORIS_NODE; do
#     _warn "$node 节点停止服务, 清理数据 ."
#     ssh -p ${SSH_PORT:-22} -T ${node} 'systemctl stop doris-fe doris-be; rm -rf ${DORIS_HOME}/doris-meta/* ${DORIS_HOME}/storage/*' &>/dev/null
#     ssh -p ${SSH_PORT:-22} -T ${node} 'kill $(ps -eo pid,args | awk '\''$0 ~ /org.apache.doris.PaloFe/ && !/awk/{print $1}'\'')' &>/dev/null
# done
### 清理结束

# 判断是否存在数据目录
ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} '[ -d ${DORIS_HOME}/doris-meta/image ]'; REV=$?

# 不存在数据目录
if [ $REV -ne 0 ]; then
    # 角色分配
    # 节点数量大于、等于5个, 则前3个为 FE, 其他全部为 BE
    if (( ${#DORIS[@]} >= 5 )); then
        fe_node=${DORIS[@]:0:3}
        be_node=${DORIS[@]:3}
        deploy_type=cluster
    else
    # 节点数量少于5个, 则第1个为 FE, 其他全部为 BE
        fe_node=${DORIS[0]}
        be_node=${DORIS[@]:1}
        deploy_type=standalone
    fi

    # 关闭所有 FE 角色
    for node in $fe_node; do
        nc -vzw 1 $node 9030 &>/dev/null && ssh -p ${SSH_PORT:-22} -T $node "systemctl stop doris-fe"
        true; _info "[$node] 节点关闭 doris-fe ."
    done

    # 启动第一个 FE 角色
    ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "systemctl start doris-fe"
    [ $? -eq 0 ] && _log "[$node] 节点启动 doris-fe ." || _err "[$node] 节点启动 doris-fe 失败 ."

    # 启动所有 BE 角色
    for node in $be_node; do
        ssh -p ${SSH_PORT:-22} -T $node "systemctl start doris-be"
        [ $? -eq 0 ] && _log "[$node] 节点启动 doris-be ." || _err "[$node] 节点启动 doris-be 失败 ."
    done

    # 等待第一个 FE 角色准备就绪
    sleep 5 && until nc -vzw 1 ${DORIS[0]} 9030 &>/dev/null; do
        true; _info "[${DORIS[0]}] 节点等待 doris-fe 服务启动就绪 ."
        sleep 3
    done
    _log "[${DORIS[0]}] 节点 doris-fe 服务启动就绪 ."

    # BE 角色加入集群
    for be in $be_node; do
        ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "mysql -h127.0.0.1 -P9030 -e \"ALTER SYSTEM ADD BACKEND '${be}:9050'\""
        [ $? -eq 0 ] && _log "[${be}:9050] 节点加入 backends ." || _err "[${be}:9050] 节点加入 backends 失败 ."
    done

    # 如果是高可用集群，注册其他 FE
    if [ "$deploy_type" == 'cluster' ]; then
        FE_NODE=($fe_node)

        # 除第一个 FE 外的其他 FE 节点
        for fe in ${FE_NODE[@]:1}; do
            ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "mysql -h127.0.0.1 -P9030 -e \"ALTER SYSTEM ADD FOLLOWER '${fe}:9010'\""
            [ $? -eq 0 ] && _log "[${fe}:9100] 节点加入 frontends ." || _err "[${fe}:9100] 节点加入 frontends 失败 ."
        done
    fi

    # 启动其他 FE
    for fe in ${FE_NODE[@]:1}; do
        green_echo ">>> $fe 节点配置集群(FE) ."
        ssh -p ${SSH_PORT:-22} -T ${fe} "\${DORIS_HOME}/bin/start_fe.sh --helper "${DORIS[0]}:9010" &>/dev/null &"
        [ $? -eq 0 ] && _log "[${fe}:9100] 节点以 --helper 模式启动 doris-fe ." || _err "[${fe}:9100] 节点以 --helper 模式启动 doris-fe 失败 ."

        while : ; do
            eval $(ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "mysql -h127.0.0.1 -P9030 -e \"SHOW PROC '/frontends'\G\" 2>/dev/null" | sed -n "/$fe/,/CurrentConnected/p" | egrep "Join|Alive" | sed 's@: @=@')
            if [[ "$Join" == "true" && "$Alive" == "true" ]]; then
                _info "[${fe}:9010] 节点同步 [${DORIS[0]}:9010] FE 数据同步完成 ."
                break
            else
                true; _info "[${fe}:9010] 节点同步 [${DORIS[0]}:9010] 待完成 ."
            fi
            sleep 5
        done

        # 同步完成 kill 进程，以 systemct 方式启动服务
        # ssh -p ${SSH_PORT:-22} -T ${DORIS[1]} 'ps -eo pid,args | awk '\''$0 ~ /start_fe.sh/ && !/awk/{print $1}'\'''
        ssh -p ${SSH_PORT:-22} -T ${fe} 'kill $(ps -eo pid,args | awk '\''$0 ~ /org.apache.doris.PaloFe/ && !/awk/{print $1}'\'')'

        sleep 3
        ssh -p ${SSH_PORT:-22} -T ${fe} 'systemctl start doris-fe &>/dev/null'
        true && _info "[${fe}] 节点启动 doris 服务 doris-fe ."

        sleep 5 && until nc -vzw 1 ${fe} 9030 &>/dev/null; do
            true; _log "[${fe}] 节点等待 doris-fe 服务启动就绪 ."
            sleep 3
        done
        _log "[${fe}] 节点 doris-fe 服务启动就绪 ."
    done

    # 创建库
    ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "mysql -h127.0.0.1 -P9030 -e 'CREATE DATABASE blueberry_data;'"
    _log "创建数据库 blueberry_data ."

    # 修改登录密码  https://www.cnblogs.com/whiteY/p/18375883
    ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "mysql -h127.0.0.1 -P9030 -e \"SET PASSWORD FOR 'root' = PASSWORD('doris123');\""
    _log "修改 Doris 密码 ."

    # 备份
    ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "mysqldump -h127.0.0.1 -P9030 --no-tablespaces --databases blueberry_data > \${DORIS_HOME}/blueberry_data.sql"
else
    _warn "数据目录已经存在, 请删除数据目录后再执行集群初始化 ."
fi

green_echo ">>> frontends: "
# ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "mysql -h127.0.0.1 -P9030 -e \"SHOW PROC '/frontends'\G\" | egrep 'IsMaster|IP|HostName|Alive'"
awk -F': +' '
    BEGIN {
        BLUE = "\033[34m"
        YELLOW = "\033[33m"
        CYAN = "\033[36m"
        GREEN = "\033[32m"
        RESET = "\033[0m"
    }
    NR%4==1 { ip=$2 }
    NR%4==2 { host=$2 }
    NR%4==3 { role=$2 }
    NR%4==0 {
        alive=$2
        printf "\t%sIP:%s%s  %sHostName:%s%s  %sIsMaster:%s%s  %sAlive:%s%s\n",
                    BLUE,   ip,    RESET,
                    YELLOW, host,  RESET,
                    CYAN,   role,  RESET,
                    GREEN,  alive, RESET
    }' <<<$(egrep 'IsMaster|IP|HostName|Alive' < <(ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "mysql -h127.0.0.1 -P9030 -e \"SHOW PROC '/frontends'\G\""))

green_echo ">>> backends: "
# ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "mysql -h127.0.0.1 -P9030 -e \"SHOW PROC '/backends'\G\" | egrep 'BackendId|IP|HostName|Alive'"
awk -F': +' '
    BEGIN {
        BLUE = "\033[34m"
        YELLOW = "\033[33m"
        CYAN = "\033[36m"
        GREEN = "\033[32m"
        RESET = "\033[0m"
    }
    NR%4==1 { bid=$2 }
    NR%4==2 { ip=$2 }
    NR%4==3 { host=$2 }
    NR%4==0 {
        alive=$2
        printf "\t%sIP:%s%s  %sHostName:%s%s  %sBackendId:%s%s  %sAlive:%s%s\n",
                    BLUE,   ip,    RESET,
                    YELLOW, host,  RESET,
                    CYAN,   bid,   RESET,
                    GREEN,  alive, RESET
    }' <<<$(egrep 'BackendId|IP|HostName|Alive' < <(ssh -p ${SSH_PORT:-22} -T ${DORIS[0]} "mysql -h127.0.0.1 -P9030 -e \"SHOW PROC '/backends'\G\""))


# # 备份
#  mysqldump -h127.0.0.1 -P9030 --no-tablespaces --databases blueberry_data > blueberry_data.sql

# # 还原
# mysql -h127.0.0.1 -P9030 < blueberry_data.sql

# # 查看
# mysql -h127.0.0.1 -P9030 -e 'use blueberry_data; show tables;'