#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 加载函数
if [ -f "${basedir}/../function.sh" ]; then
    . ${basedir}/../function.sh
fi

# 加载环境变量
. /usr/local/.mango/environment.txt

title "📦️ 初始化 Hadoop 集群 ."

# 执行前确认
echo -ne "\033[031;1m[$LAN_IP_ADDRESS] 警告：此操作将初始化 Hadoop 集群并格式化数据，是否继续？(请输入Yes继续)：\033[0m"
read -r user_input
if [[ "${user_input}" != "Yes" ]]; then
	_info "用户取消操作，退出。"
	exit 1
else
    echo
fi

### 清理数据

# set +H  # 关闭历史扩展
# ssh -T 192.168.10.61 "awk '!/#/ && /hdp-/{print \$0}' /etc/rc.d/rc.local"
# ssh -T 192.168.10.61 'awk '\''!/#/&&/hdp-/{printf "%s ", $3} END {print ""}'\'' /etc/rc.d/rc.local'
# set -H  # 恢复历史扩展

# for node in $DORIS_NODE; do
#     _warn "[$node] 节点停止服务, 清理数据 ."
#     ssh -p ${SSH_PORT:-22} -T ${node} 'systemctl stop $(awk '\''!/#/&&/hdp-/{printf "%s ", $3} END {print ""}'\'' /etc/rc.d/rc.local)'
#     ssh -p ${SSH_PORT:-22} -T ${node} 'rm -rf ${HADOOP_HOME}/{namenode,datanode,journalnode,tmpdir}/*' &>/dev/null
# done
### 清理结束


# 判断是否存在数据目录
ssh -p ${SSH_PORT:-22} -T ${HADOOP[0]} '[ -d ${HADOOP_HOME}/namenode/current ]'; REV=$?

# 不存在数据目录
if [ $REV -ne 0 ]; then
    for node in ${ZOOKEEPER[@]}; do
        nc -vzw 1 $node 2181 &>/dev/null || ssh -p ${SSH_PORT:-22} -T $node "systemctl start zookeeper"
        [ $? -eq 0 ] && _log "[$node] 节点启动 zookeeper ." || _err "[$node] 节点启动 zookeeper 失败 ."
    done
	sleep 5

    # 格式化 zkfc
    ssh -p ${SSH_PORT:-22} -T ${HADOOP[0]} "hdfs zkfc -formatZK -force" &>/tmp/zkfc-format.out
    [ $? -eq 0 ] && _log "[${HADOOP[0]}] 节点格式化 zkfc ." || _err "[${HADOOP[0]}] 节点格式化 zkfc 失败, 错误日志 /tmp/zkfc-format.out ."
    sleep 5

    # 启动 journalnode
    for node in ${HADOOP[@]:2}; do
        ssh -p ${SSH_PORT:-22} -T $node "systemctl start hdp-journalnode"
        [ $? -eq 0 ] && _log "[$node] 节点启动 hdp-journalnode ." || _err "[$node] 节点启动 hdp-journalnode 失败 ."
    done
    sleep 5

    # 初始化 NameNode1
    ssh -p ${SSH_PORT:-22} -T ${HADOOP[0]} "hdfs namenode -format -force" &>/tmp/namenode-format.out
    [ $? -eq 0 ] && _log "[${HADOOP[0]}] 节点格式化 namenode ." || _err "[${HADOOP[0]}] 节点格式化 namenode 失败, 错误日志 /tmp/namenode-format.out ."

    # 启动 NameNode1
    ssh -p ${SSH_PORT:-22} -T ${HADOOP[0]} "systemctl start hdp-namenode"
    [ $? -eq 0 ] && _log "[${HADOOP[0]}] 节点启动 namenode ." || _err "[${HADOOP[0]}] 节点启动 namenode 失败 ."
    sleep 5

    # 同步/启动 NameNode2
    ssh -p ${SSH_PORT:-22} -T ${HADOOP[1]} "hdfs namenode -bootstrapStandby -force" &>/tmp/namenode-bootstrapStandby.out
	[ $? -eq 0 ] && _log "[${HADOOP[1]}] 节点同步 namenode ." || _err "[${HADOOP[1]}] 节点同步 namenode 失败 ."
    ssh -p ${SSH_PORT:-22} -T ${HADOOP[1]} "systemctl start hdp-namenode"
    [ $? -eq 0 ] && _log "[${HADOOP[1]}] 节点启动 namenode ." || _err "[${HADOOP[1]}] 节点启动 namenode 失败 ."

    # 启动 Zkfc、ResourceManager
    for node in ${HADOOP[@]:0:2}; do
        ssh -p ${SSH_PORT:-22} -T $node "systemctl start hdp-zkfc hdp-resourcemanager"
        [ $? -eq 0 ] && _log "[$node] 节点启动 hdp-zkfc、hdp-resourcemanager ." || _err "[$node] 节点启动 hdp-zkfc、hdp-resourcemanager 失败 ."
    done

    # 启动 DataNode、hdp-nodemanager
    for node in ${HADOOP[@]:2}; do
        ssh -p ${SSH_PORT:-22} -T $node "systemctl start hdp-datanode hdp-nodemanager"
        [ $? -eq 0 ] && _log "[$node] 节点启动 hdp-datanode、hdp-nodemanager ." || _err "[$node] 节点启动 hdp-datanode、hdp-nodemanager 失败 ."
    done

    # 启动 Kafka
    for node in ${KAFKA[@]}; do
        ssh -p ${SSH_PORT:-22} -T $node "systemctl start kafka"
        [ $? -eq 0 ] && _log "[$node] 节点启动 kafka ." || _err "[$node] 节点启动 kafka 失败 ."
    done
else
    _warn "数据目录已经存在, 请删除数据目录后再执行集群初始化 ."
fi

green_echo "\n>>> NameNode: "
for node in ${HADOOP[@]:0:2}; do
    clusterID=$(ssh -p ${SSH_PORT:-22} -T $node "awk -F= '/clusterID=/{print \$2}' \$HADOOP_HOME/namenode/current/VERSION")
    green_echo "\t$node  $clusterID"
done

green_echo "\n>>> DataNode: "
for node in ${HADOOP[@]:2}; do
    clusterID=$(ssh -p ${SSH_PORT:-22} -T $node "awk -F= '/clusterID=/{print \$2}' \$HADOOP_HOME/datanode/current/VERSION")
    green_echo "\t$node  $clusterID"
done

green_echo "\n>>> JournalNode: "
for node in ${HADOOP[@]:2}; do
    clusterID=$(ssh -p ${SSH_PORT:-22} -T $node "awk -F= '/clusterID=/{print \$2}' \$HADOOP_HOME/journalnode/mghdpha/current/VERSION")
    green_echo "\t$node  $clusterID"
done


# /home/hadoop/hadoop-2.7.7/{namenode,datanode,journalnode,tmpdir}