#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 加载函数
if [ -f "${basedir}/../function.sh" ]; then
    . ${basedir}/../function.sh
fi

# 加载环境变量
. /usr/local/.mango/environment.txt

# 判断 tsdb 类型
if [ "$TSDB_TYPE" == "tdengine" ]; then
    _info "当前 TSDB 类型为 tdengine, 无需初始化 opentsdb ."
    exit 1
fi

title "📦️ 初始化 Opentsdb (tsdb, tsdb-uid)表 ."

# 执行前确认
echo -ne "\033[031;1m[$LAN_IP_ADDRESS] 警告：此操作将初始化 Opentsdb (tsdb, tsdb-uid)表，是否继续？(请输入Yes继续)：\033[0m"
read -r user_input
if [[ "${user_input}" != "Yes" ]]; then
	_info "用户取消操作，退出。"
	exit 1
else
    echo
fi

### 清理数据
# for node in $TDENGINE_NODE; do
#     _warn "$node 节点停止服务, 清理数据 ."
#     ssh -p ${SSH_PORT:-22} -T ${node} 'systemctl stop taosd taosadapter; rm -rf /var/lib/taos/* /var/log/taos/* /var/lib/taos/.running' &>/dev/null
#     ssh -p ${SSH_PORT:-22} -T ${node} 'kill $(ps -eo pid,args | awk '\''$0 ~ /taos/ && !/awk/{print $1}'\''); systemctl start taosd taosadapter' &>/dev/null
# done
### 清理结束


# 等待服务准备就绪  示例调用
## wait_for_tdengine "mnode" "192.168.1.100"  # 传入 mnode 和 IP 地址
## wait_for_tdengine "dnode" "192.168.1.101"  # 传入 dnode 和 IP 地址


# 启动 hbase 服务  ## ssh -p ${SSH_PORT:-22} -T ${HADOOP[0]} "mango start hbase"
for node in ${HBASE[@]}; do
    ssh -p ${SSH_PORT:-22} -T $node 'hbase_service=$(awk "/^systemctl/&&/hbase-/{print \$NF}" /etc/rc.d/rc.local); systemctl start $hbase_service'
    [ $? -eq 0 ] && _log "[$node] 节点启动 hbase ." || _err "[$node] 节点启动 hbase 失败 ."
done
sleep 5

# 检查 hbase 服务是否启动成功
until nc -vzw 1 ${HBASE[0]} 16010 &>/dev/null; do  { purple_echo "\t[${HBASE[0]}:16010] 节点等待 hbase 服务准备就绪 ."; sleep 3; } done
_log "[${HBASE[0]}:16010] 节点 hbase 服务准备就绪 ."

# copy 创建 opentsdb 表文件到当前节点
scp -qP ${SSH_PORT:-22} ${OPENTSDB[0]}:/usr/share/opentsdb/tools/create_table.sh $PACKAGE_DIR/script/cluster/create_table.sh
scp -qP ${SSH_PORT:-22} ${PACKAGE_DIR}/script/cluster/create_table.sh ${HBASE[0]}:/tmp/create_table.sh
_log "[${OPENTSDB[0]}] 节点复制 create_table.sh 脚本到 ${HBASE[0]}:/tmp/create_table.sh ."

## 其他压缩算法依赖
# yum install -y openssl openssl-devel svn ncurses-devel zlib-devel libtool gnuplot snappy snappy-devel bzip2 bzip2-devel lzo lzo-devel lzop autoconf automake lz4
# 创建 opentsdb 表
ssh -p ${SSH_PORT:-22} -T ${HBASE[0]} bash <<-'EOF'
    . /etc/profile &>/dev/null
    . /usr/local/.mango/function.sh

    # 检查 opentsdb 表是否存在
    if echo "exists 'tsdb'" | hbase shell 2>/dev/null | grep -q 'does exist'; then
        _info "表 'tsdb' 已存在，跳过创建。"
        exit 0
    fi

    # 创建 opentsdb 表
    env COMPRESSION=SNAPPY TSDB_TTL=2147483647 HBASE_HOME=$HBASE_HOME /tmp/create_table.sh 2>/dev/null
    _log "创建 opentsdb 表 tsdb 结束 ."
EOF

# 验证 opentsdb 表是否创建成功
green_echo ">>> 验证表是否创建成功 ."
ssh -p ${SSH_PORT:-22} -T ${HBASE[0]} bash <<-'EOF'
    . /etc/profile &>/dev/null
    . /usr/local/.mango/function.sh

    # 检查 opentsdb 表是否存在
    for table in tsdb tsdb-uid tsdb-tree tsdb-meta; do
        if echo "exists 'tsdb'" | hbase shell 2>/dev/null | grep -q 'does exist'; then
            _info "[$table] 表创建成功 ."
        fi
    done
EOF

# 启动 opentsdb 服务 ## ssh -p ${SSH_PORT:-22} -T ${HADOOP[0]} "mango start opentsdb"
for node in ${OPENTSDB[@]}; do
    nc -vzw 1 $node 14242 &>/dev/null || ssh -p ${SSH_PORT:-22} -T $node "systemctl start opentsdb"
    [ $? -eq 0 ] && _log "[$node] 节点启动 opentsdb ." || _err "[$node] 节点启动 opentsdb 失败 ."
done

# 等待 opentsdb 服务准备就绪
until nc -vzw 1 ${OPENTSDB[0]} 14242 &>/dev/null; do { purple_echo "\t[${OPENTSDB[0]}:14242] 节点等待 opentsdb 服务准备就绪."; sleep 3; } done

# 创建 opentsdb 测试数据 1745683200  ## date -d2025-04-27 +%s
CHECK_POINT='{"start": 1745683200, "queries": [{"metric": "check.tsdb", "aggregator": "avg", "tags": {"name": "opentsdb"}}]}'
until curl -s -X POST -H "Content-Type: application/json" http://${OPENTSDB[0]}:14242/api/query -d "$CHECK_POINT" | grep -q 1234567890; do
    CREATE_POINT='{"metric": "check.tsdb", "timestamp": 1745683200, "value": 1234567890, "tags": {"name": "opentsdb"}}'
    curl -s -X POST -H 'Content-Type: application/json' http://${OPENTSDB[0]}:14242/api/put -d "$CREATE_POINT"

    purple_echo "\t[${OPENTSDB[0]}:14242] 节点等待 opentsdb 服务准备就绪."
    sleep 3
done
_log "创建测试数据成功 ."

# install -D -m 744 -o opentsdb -g opentsdb /usr/share/opentsdb/bin/mygnuplot.sh /tmp/.tsdb/tsdb-gnuplot/mygnuplot.sh