#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 加载函数
if [ -f "${basedir}/../function.sh" ]; then
    . ${basedir}/../function.sh
fi

# 加载环境变量
. /usr/local/.mango/environment.txt

# 判断 tsdb 类型
if [ "$TSDB_TYPE" == "opentsdb" ]; then
    _info "当前 TSDB 为 opentsdb, 无需初始化 tdengine 集群 ."
    exit 1
fi

title "📦️ 初始化、配置 Tdengine 集群 ."

# 执行前确认
echo -ne "\033[031;1m[$LAN_IP_ADDRESS] 警告：此操作将初始化并配置 Tdengine 集群，是否继续？(请输入Yes继续)：\033[0m"
read -r user_input
if [[ "${user_input}" != "Yes" ]]; then
	_info "用户取消操作，退出。"
	exit 1
else
    echo
fi

### 清理数据
# for node in $TDENGINE_NODE; do
#     _warn "$node 节点停止服务, 清理数据 ."
#     ssh -p ${SSH_PORT:-22} -T ${node} 'systemctl stop taosd taosadapter; rm -rf /var/lib/taos/* /var/log/taos/* /var/lib/taos/.running' &>/dev/null
#     ssh -p ${SSH_PORT:-22} -T ${node} 'kill $(ps -eo pid,args | awk '\''$0 ~ /taos/ && !/awk/{print $1}'\''); systemctl start taosd taosadapter' &>/dev/null
# done
### 清理结束


# 等待服务准备就绪  示例调用
## wait_for_tdengine "mnode" "192.168.1.100"  # 传入 mnode 和 IP 地址
## wait_for_tdengine "dnode" "192.168.1.101"  # 传入 dnode 和 IP 地址
wait_for_tdengine() {
    node_type=$1
    ip_address=$2
    tdengine_ip=${3:-127.0.0.1}

    . /usr/local/.mango/function.sh
    . /usr/local/.mango/environment.txt
    
    # 获取节点的主机名
    node_hostname=$(lookup_ip_hostname $ip_address)

    node_state=False
    if [ "$node_type" == "mnode" ]; then
        # 如果是 mnode，查询 mnodes 状态
        until [ "$node_state" == "ready" ]; do
            true && _info "[${node_hostname}:6030] 节点等待 mnode 角色准备就绪 (来自 $tdengine_ip mnode 查询 $node_hostname) ."
            node_state=$(taos -h $tdengine_ip -s "show mnodes;" | awk -v node="${node_hostname}:6030" '{if ($3 == node) print $7}')
            sleep 2
        done
        _info "[${node_hostname}:6030] 节点 mnode 角色准备就绪 ."
    elif [ "$node_type" == "dnode" ]; then
        # 如果是 dnode，查询 dnodes 状态
        until [ "$node_state" == "ready" ]; do
            true && _info "[${node_hostname}:6030] 节点等待 dnond 角色准备就绪 (来自 $tdengine_ip dnode 查询 $node_hostname) ."
            node_state=$(taos -h $tdengine_ip -s "show dnodes;" | awk -v node="${node_hostname}:6030" '{if ($3 == node) print $9}')
            sleep 2
        done
        _info "[${node_hostname}:6030] 节点 dnond 角色准备就绪 ."
    else
        _info "未知节点类型 $node_type, 请提供 mnode 或 dnode."
        exit 1
    fi
}


# 检查是否已经初始化
max_id=$(awk '/^\s+id: /{count++} END {print count}' < <(ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "taos -s 'show dnodes\G'"))

# max_id 等于 1，集群未初始化
if [ "$max_id" == 1 ]; then
    # 添加 dnode 节点
    green_echo ">>> Tdengine 集群添加 dnode 节点 ."
    IFS=,; for dnode in $(lookup_ip_hostname ${TDENGINE[@]}); do
        # 检查主节点准备就绪(第一个主节点自检)
        ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "$(declare -f wait_for_tdengine); wait_for_tdengine 'mnode' '${TDENGINE[0]}' '${TDENGINE[0]}'"

        # 检查 dnode 节点准备就绪
        until nc -vzw 1 ${dnode} 6030 &>/dev/null; do  { purple_echo "\t[${dnode}:6030] 节点等待 taosd 服务准备就绪 ."; sleep 3; } done

        # 添加 dnode 节点
        ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "taos -s \"create dnode '${dnode}:6030';\"  &>/dev/null"
        _info "[${dnode}:6030] 节点配置角色 dnode ."

        # 等待 dnode 节点加入集群
        ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "$(declare -f wait_for_tdengine); wait_for_tdengine 'dnode' '${dnode}' '${TDENGINE[0]}'"
        _info "[${dnode}:6030] 节点 dnode 角色就绪 ."
    done

    green_echo ">>> dnodes 节点信息 ."
    ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "taos -s 'show dnodes\G'"


    # 加入 mnode 角色
    green_echo ">>> Tdengine 集群添加 mnode 节点 ."
    mnode_id=1
    for mnode in $(lookup_ip_hostname ${TDENGINE[@]}); do
        ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "$(declare -f wait_for_tdengine); wait_for_tdengine 'dnode' '${mnode}' '${TDENGINE[0]}'"
        _log "[${mnode}] 节点集群状态检查通过,角色 dnode, 状态为 ready ."

        ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "taos -s 'create mnode on dnode ${mnode_id};' &>/dev/null"
        _info "[${mnode}] 节点配置角色 mnode ."

        ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "$(declare -f wait_for_tdengine); wait_for_tdengine 'mnode' '${mnode}' '${TDENGINE[0]}'"
        _log "[${mnode}] 节点 mnode 角色就绪."

        let mnode_id++
    done

    green_echo ">>> mnodes 节点信息 ."
    ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "taos -s 'show mnodes\G'"


    # 创建数据库
    ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "taos -s \"CREATE DATABASE mango BUFFER 256 CACHESIZE 1 CACHEMODEL 'last_row' COMP 2 DURATION 14400m WAL_FSYNC_PERIOD 3000 MAXROWS 4096 MINROWS 100 STT_TRIGGER 1 KEEP 36500d PAGES 256 PAGESIZE 4 PRECISION 'ms' REPLICA 3 WAL_LEVEL 1 VGROUPS 8 SINGLE_STABLE 0 TABLE_PREFIX 0 TABLE_SUFFIX 0 TSDB_PAGESIZE 4 WAL_RETENTION_PERIOD 3600 WAL_RETENTION_SIZE 0 KEEP_TIME_OFFSET 0;\"" &>/dev/null
    _info "创建业务数据库 mango ."
    #ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "taos -s 'show create database mango\G';"

    green_echo ">>> show vnodes ."
    ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "taos -s 'show vnodes'"
else
    _warn "dnode 节点数量大于 1, 请删除数据目录，重启服务后再执行集群初始化 ."

    green_echo ">>> mnodes 节点信息 ."
    ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "taos -s 'show mnodes\G'"

    green_echo ">>> dnodes 节点信息 ."
    ssh -p ${SSH_PORT:-22} -T ${TDENGINE[0]} "taos -s 'show dnodes\G'"
fi