#!/bin/bash

# 获取脚本所在目录绝对路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 调试开关，0 表示启用
DEBUG=0
DEBUG_FILE=/var/log/debug.log
MANGO_ENV=/usr/local/.mango/environment.txt
# /home/.environment && chattr +i /home/.environment

# 操作日志记录
LOG_FILE=/var/log/install.log


# # 定义斜体控制序列
# ITALIC_ON="\033[3m"
# ITALIC_OFF="\033[23m"

# # 重定向所有输出到处理流（自动添加斜体）
# exec > >(
#   while IFS= read -r line; do
#     echo -e "${ITALIC_ON}${line}${ITALIC_OFF}"
#   done
# )

# color
red_echo ()      { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[031;1m$@\033[0m"; }
green_echo ()    { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[032;1m$@\033[0m"; }
yellow_echo ()   { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[033;1m$@\033[0m"; }
blue_echo ()     { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[034;1m$@\033[0m"; }
purple_echo ()   { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[035;1m$@\033[0m"; }
bred_echo ()     { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[041;1m$@\033[0m"; }
bgreen_echo ()   { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[042;1m$@\033[0m"; }
byellow_echo ()  { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[043;1m$@\033[0m"; }
bblue_echo ()    { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[044;1m$@\033[0m"; }
bpurple_echo ()  { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[045;1m$@\033[0m"; }
bgreen_echo ()   { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[042;34;1m$@\033[0m"; }

# 输出标题样式
function title() {
    local retval=$?
    local left="$2"
    local timestamp=$(date '+%Y-%m-%d %H:%M:%S')
    local text="$1"
    local total_length=90
    local str_length=$(wc -m <<<$text)
    local chinese_length=$(wc -l < <(grep -oP "\p{Han}" <<<$text))
    local text_length=$(( str_length + chinese_length - 1 ))
    local left_padding_length=$(( (total_length - text_length) / 2 ))
    local left_padding_length=${left:-32}
    local right_padding_length=$(( total_length - text_length - left_padding_length ))
    text="$(printf "%${left_padding_length}s$text%${right_padding_length}s" ' ' ' ')"
    echo
    bblue_echo "$text"
    # bblue_echo "$text  ${left_padding_length}|$text_length|${right_padding_length}"
    echo "[$(green_echo $LAN_IP_ADDRESS)] $retval | $timestamp | $BASH_LINENO | ${BASH_SOURCE[1]} ${BASH_LINENO} | ${FUNCNAME} | $@" >> $LOG_FILE
}

# function _log_common() {
#     local retval=$?
#     local state=$1
#     local color_func=$2
#     local ip_color_func=$3
#     local message=$4
#     local status=$($color_func $retval)
#     local state_colored=$($color_func $state)
#     local timestamp=$(date '+%Y-%m-%d %H:%M:%S')
#     local ip_colored=$($ip_color_func $LAN_IP_ADDRESS)
#     local lineno=$(printf "%03d\n" ${BASH_LINENO[1]})
#     echo "[$ip_colored] $status $timestamp $lineno $message $state_colored"
#     echo "[$ip_colored] $status | $timestamp | ${BASH_SOURCE[2]} ${BASH_LINENO[1]} | ${FUNCNAME[2]} | $message $state_colored" >> $LOG_FILE

#     if [[ "${FUNCNAME[1]}" == "_err" ]]; then
#         exit $retval
#     else
#         return $retval
#     fi
# }
function _log_common() {
    local retval=$?
    local color_func=$2
    local ip_color_func=$3
    local message=$4
    local status=$($color_func $retval)
    local timestamp=$(date '+%Y-%m-%d %H:%M:%S')
    local ip_colored=$($ip_color_func $LAN_IP_ADDRESS)
    local lineno=$(printf "%03d\n" ${BASH_LINENO[1]})
    echo "[$ip_colored] $status $timestamp $lineno $message"
    echo "[$ip_colored] $status | $timestamp | ${BASH_SOURCE[2]} ${BASH_LINENO[1]} | ${FUNCNAME[2]} | $message $state_colored" >> $LOG_FILE

    if [[ "${FUNCNAME[1]}" == "_err" ]]; then
        exit $retval
    else
        return $retval
    fi
}

# 根据前一个命令的返回值 0/非0，调用 _log_common，打印/记录 OK 或 ERROR 日志，执行后状态码为前一个命令的返回值
# 区分执行结果
function _log() {
    if [ $? -eq 0 ]; then
        _log_common "OK" blue_echo blue_echo "$@"
    else
        _log_common "ERROR" red_echo blue_echo "$@"
    fi
}

# 调用 _log_common，记录 INFO 日志，执行后状态码为前一个命令的返回值
# 不区分执行结果仅提示
function _info() {
    _log_common "INFO" green_echo green_echo "$@"
}

# 调用 _log_common，记录 WARN 日志，执行后状态码为前一个命令的返回值
# 不区分执行结果仅提示
function _warn() {
    _log_common "WARN" yellow_echo yellow_echo "$@"
}

# 调用 _log_common，记录/输出 ERROR 日志并退出程序。
# 不区分执行结果仅退出
function _err() { 
    _log_common "ERROR" red_echo red_echo "$@"
    exit $?
}

# 将服务启动命令添加到 /etc/rc.d/rc.local
function add_service_to_rc_local() {
    local name=$1
    grep -qr "# start ${name}" /etc/rc.d/rc.local || echo -e "\n# start ${name}\nsystemctl start ${name}" >> /etc/rc.d/rc.local
}


# 获取局域网 IP 地址
function get_lan_ip_address() {
    ip addr show ${INTERFACE:-$(ip route | awk '/default/ {print $5}')} | awk '/inet / {sub(/\/.*/, "", $2); print $2; exit}'
}

# 循环执行直到成功
#  input  -->  retry_until_success curl http://127.0.0.1:8080
function retry_until_success() {
    while true; do $@ && return; done
}

# 检查变量是否存在于数组内
function contains() {
    local search_value="$1"
    local array=("${!2}")
    for item in "${array[@]}"; do
        [[ "$item" == "$search_value" ]] && return 0
    done
    return 1
}

# 用于在 /etc/profile.d/hc-env.sh 中添加或更新环境变量
function update_env_marked() {
    local suffix="$1"
    local content="$2"
    local envfile="/etc/profile.d/hc-env.sh"

    local action="增加"
    local end_marker="# END ${suffix}"
    local start_marker="# BEGIN ${suffix}"

    # 检查是否已经存在标记段落
    if [ -f "$envfile" ]; then
        if grep -q "$start_marker" "$envfile"; then
            # 删除已存在的标记段落
            sed -i "/$start_marker/,/$end_marker/d" "$envfile"
            action="更新"

            # 清理文件空行
            sed -i '/./,/^$/!d' "$envfile"
        fi
    fi

    # 在文件末尾添加新的配置
    {
        echo "$start_marker"
        echo "# ${action}  $(date +'%F %T')"
        echo "$content"
        echo "$end_marker"
        echo
    } >> "$envfile"

    _log "环境变量配置 $envfile 已${action} ."
}


# 安装rpm包
# input  -->  fetch_and_install_rpm /tmp/fastdfs-6.08/libfastcommon/libfastcommon-1.0.64-1.el7.x86_64.rpm
function fetch_and_install_rpm() {
    local url="$1"
    local pkg="${url##*/}"
    local name="${pkg%.*}"
    if rpm -q $name --quiet; then
        _info "$name 已经安装 ." 
    else
        rpm -ivh $url &>/dev/null
        _log "安装 $pkg ."
    fi
}


# 传入端口主机名获取 IP:PORT,IP:PORT
# input  -->  lookup_ips_with_port 2181 node41 node42 node50 node44
# out    -->  192.168.10.41:2181,192.168.10.42:2181,192.168.10.50:2181,192.168.10.44:2181
function lookup_ips_with_port() {
    local port="$1"
    local hosts="${@:2}"

    awk -v hosts="$hosts" -v port="$port" '
        BEGIN {
            split(hosts, hostArray, " ");
            for (i in hostArray) hostSet[hostArray[i]] = 1;
        }
        !/^#/ {
            if ($1 in hostSet) ip[$1] = $1;
            if ($2 in hostSet) ip[$2] = $1;
        }
        END {
            output = "";
            for (i in hostArray) {
                if (length(output) > 0) output = output ",";
                output = output (ip[hostArray[i]] ? ip[hostArray[i]] ":" port : hostArray[i] ":" port);
            }
            print output;
        }
    ' /etc/hosts
}

# 传入端口 IP 获取 HOSTNAME:PORT,HOSTNAME:PORT
# input  -->  lookup_ips_with_port 2181 192.168.10.41 192.168.10.42
# out    -->  node41:2181,node42:2181
function lookup_hostname_with_port() {
    local port="$1"
    local hosts="${@:2}"

    awk -v hosts="$hosts" -v port="$port" '
        BEGIN {
            split(hosts, hostArray, " ");
            for (i in hostArray) hostSet[hostArray[i]] = 1;
        }
        !/^#/ {
            if ($1 in hostSet) ip[$1] = $2;
            if ($2 in hostSet) ip[$2] = $2;
        }
        END {
            output = "";
            for (i in hostArray) {
                if (length(output) > 0) output = output ",";
                output = output (ip[hostArray[i]] ? ip[hostArray[i]] ":" port : hostArray[i] ":" port);
            }
            print output;
        }
    ' /etc/hosts
}


# 传入主机名获取 IP
# input  -->  lookup_ip_addresses node41 node42 node50 node44
# out    -->  192.168.10.41,192.168.10.42,192.168.10.50,192.168.10.44
function lookup_ip_addresses() {
    local hosts="$@"
    awk -v hosts="$hosts" '
        BEGIN {
            split(hosts, hostArray, " ");
            for (i in hostArray) hostSet[hostArray[i]] = 1;
        }
        !/^#/ {
            if ($1 in hostSet) ip[$1] = $1;
            if ($2 in hostSet) ip[$2] = $1;
        }
        END {
            output = "";
            for (i in hostArray) {
                if (length(output) > 0) output = output ",";
                output = output (ip[hostArray[i]] ? ip[hostArray[i]] : hostArray[i]);
            }
            print output;
        }
    ' /etc/hosts
}

# 传入IP 获取主机名
# input  -->  lookup_ip_hostname 192.168.10.41 192.168.10.42
# out    -->  node41,node42
function lookup_ip_hostname() {
    local hosts="$@"
    awk -v hosts="$hosts" '
        BEGIN {
            split(hosts, hostArray, " ");
            for (i in hostArray) hostSet[hostArray[i]] = 1;
        }
        !/^#/ {
            if ($1 in hostSet) ip[$1] = $2;
            if ($2 in hostSet) ip[$2] = $2;
        }
        END {
            output = "";
            for (i in hostArray) {
                if (length(output) > 0) output = output ",";
                output = output (ip[hostArray[i]] ? ip[hostArray[i]] : hostArray[i]);
            }
            print output;
        }
    ' /etc/hosts
}


# 传入主机获取指纹
# input  -->  get_ssh_known_hosts node41 node42 node50 node44
# out    -->  ssh-rsa xxxxx
function get_ssh_known_hosts() {
    local hosts="$@"
    hostaddr=${hosts/,/ }
    for addr in $hostaddr; do
        ssh-keyscan -p ${SSH_PORT:-22} -t rsa ${addr} 2>/dev/null
    done
}


# 传入主机名获取 SSH 公钥
# input  -->  get_ssh_pubkey node41 node42 node50 node44
# out    -->  ssh-rsa xxxxx
function get_ssh_pubkey() (
    local hosts="$@"
    hostaddr=${hosts/,/ }
    for addr in $hostaddr; do
        ssh -p ${SSH_PORT:-22} -T ${addr} "[ -f ~/.ssh/id_rsa ] || ssh-keygen -q -t rsa -P '' -f ~/.ssh/id_rsa; cat ~/.ssh/id_rsa.pub"
    done
)


# ssh 秘钥登录
# input  -->  deploy_ssh_keys "192.168.10.42 192.168.10.43 192.168.10.44 node45 node46" user passwd 22
function deploy_ssh_keys() {
    local hosts="$1"
    local username="$2"
    local password="$3"
    local port="${4:-22}"

    #echo "======================================================================="
    printf "%*s\n" "90" | tr ' ' "="
    echo "hosts:"
    #awk '{print "    "$0}' < <(uniq < <(sort <<< "${hosts// /$'\n'}"))
    awk '{print "    "$0}' < <(sort -u <<<"${hosts// /$'\n'}")
    #echo "======================================================================="
    printf "%*s\n" "90" | tr ' ' "="
    green_echo ">>> 配置从当前节点 [$LAN_IP_ADDRESS] 到所有节点秘钥登录 ."

    # 生成 SSH 密钥
    if [ ! -f ~/.ssh/id_rsa ]; then
        ssh-keygen -q -t rsa -P '' -f ~/.ssh/id_rsa
        _log "生成当前主机 RSA 秘钥 ."
    fi

    # 安装 sshpass
    if ! rpm -q sshpass --quiet; then
        dnf install -y sshpass &>/dev/null
        _log "安装 sshpass ."
    fi

    # 去重和排序主机列表
    local sorted_hosts

    sorted_hosts=$(sort -u <<<"${hosts// /$'\n'}")

    local count=1
    local node_count=$(wc -l <<<"$sorted_hosts")

    for host in $sorted_hosts; do
        # 检查主机是否可达
        if ! ping -i 0.2 -c 3 -W 1 "$host" &> /dev/null; then
            _err "ping 检测异常 $host ."
            continue
        fi

        # 从 /etc/hosts 获取主机地址和主机名
        local hostaddr hostname
        result=$(awk -v h="$host" '!/^#/ && ($1 == h || $2 == h) {print $1, $2; exit}' /etc/hosts)

        if [ -n "$result" ]; then
            # 赋值变量
            read -r hostaddr hostname <<<"$result"

            # 配置 SSH 密钥
            for addr in "$hostname" "$hostaddr"; do
                sshpass -p "$password" ssh-copy-id -o StrictHostKeyChecking=no -p "$port" "${username}@${addr}" &>/dev/null
                _log "[$count/$node_count] 配置当前主机到 [$host] 秘钥登录，目的主机：${username}@${addr}"
            done

            # 验证 SSH 密钥登录
            if ssh -o StrictHostKeyChecking=no -p "$port" "$username@$hostname" hostname &>/dev/null; then
                _log "验证秘钥登录 ."
            else
                _err "验证秘钥登录失败 ."
            fi
        else
            _info "注意 /etc/hosts 文件内没有找到 $host 跳过此主机秘钥登录 ."
        fi

        let count++
    done
}

# IP 地址扩展
process_ip_addresses() {

    local ip_string="$1"
    local section="$2"
    local result=()
    local valid_ip=()

    # 将传入的参数格式化为数组
    IFS=',' read -ra parts <<< "$ip_string"

    # 循环数组生成 IP 扩展列表
    for part in "${parts[@]}"; do
        # 匹配地址段 192.168.10.41-45
        if [[ "$part" =~ ^([0-9]+\.[0-9]+\.[0-9]+\.)([0-9]+)-([0-9]+)$ ]]; then
            local prefix="${BASH_REMATCH[1]}"
            if [ "${BASH_REMATCH[2]}" -lt "${BASH_REMATCH[3]}" ]; then
                for ((i=BASH_REMATCH[2]; i<=BASH_REMATCH[3]; i++)); do
                    result+=("${prefix}${i}")
                    continue
                done
            else
                MSG="$(red_echo ">>> IP 地址异常，IP 地址段开始值必须小于结束值 .\n[$section]\n$part") code: 2"
                return 2
            fi
        # 匹配正常 IP 192.168.10.46
        elif [[ "$part" =~ ^([0-9]+\.[0-9]+\.[0-9]+\.)[0-9]+$ ]]; then
            prefix="${BASH_REMATCH[1]}"
            result+=("$part")
        # 匹配 192.168.10.47,48,49
        elif [[ "$part" =~ ^[0-9]+$ ]]; then
            result+=("${prefix}${part}")
        else
            MSG="$(red_echo ">>> IP 地址异常，配置无法识别 .\n[$section]\n$part")"
            return 2
        fi
    done

    # IP 扩展列表去除非法 IP
    for ip in ${result[@]}; do
        # 使用正则表达式检查 IP 地址格式和每段的值
        if [[ "$ip" =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]; then
            IFS='.' read -r -a octets <<< "$ip"
            local valid=0
            if [ "${octets[0]}" == "0" ]; then
                MSG="$(red_echo ">>> IP 地址异常，IP 地址第一段不能为 ${octets[0]} 请调整配置文件后重试 .\n[$section]\n$ip") code: 2"
                return 2
            fi

            if [[ "${octets[3]}" =~ ^(0|255)$ ]]; then
                MSG="$(red_echo ">>> IP 地址异常，IP 地址第四段不能为 ${octets[3]} 请调整配置文件后重试 .\n[$section]\n$ip") code: 3"
                return 3
            fi

            for octet in "${octets[@]}"; do
                if ((octet < 0 || octet > 255)); then
                    MSG="$(red_echo ">>> IP 地址异常，IP 地址非法请调整配置文件后重试 .\n[$section]\n$ip") code: 4"
                    return 4
                fi
            done

            valid_ip+=("$ip")
        else
            MSG="$(red_echo ">>> IP 地址非法不能为 ${ip} 请调整配置文件后重试 .\n[$section]\n$ip")"
            return 4
        fi
    done

    echo ${valid_ip[@]}
}

# 解析配置文件
# input  -->  listIniKeysValues zookeeper print
# out    -->  192.168.10.41 192.168.10.42 192.168.10.43

# input  -->  listIniKeysValues zookeeper environment
# out    -->  ZOOKEEPER[0]=192.168.10.41  ZOOKEEPER[1]=192.168.10.42  ZOOKEEPER[2]=192.168.10.43
listIniKeysValues() {
    local section="$1"
    local action="${2:-print}"
    local inifile="${listIniKeys:-$basedir/config.ini}"

    if [ $# -lt 1 ] || [ ! -f "$inifile" ]; then
        echo "Usage: listIniKeysValues <section> [action]"
        exit 1
    fi

    # 获取指定区域内容
    content=$(awk -v section="[$section]" '$0 == section {flag=1; next}flag && /^\[/ {flag=0}flag {print}' "$inifile")

    if [ ! -n "$content" ]; then
        _warn ">>> 配置解析失败 ---> listIniKeysValues $inifile $section $action"
        return 1
    else
        case "$action" in
            print)
                # 打印区域内容
                if [[ $section =~ ^[a-z] ]]; then
                    # 过滤注释行,空行
                    filtered_content="$(awk '!/^#|^$|^::/{printf "%s%s", separator, $0; separator=","}' <<<$content)"

                    # 解析IP列表(对IP地址扩展)
                    if ! process_ip_addresses "$filtered_content" "$section" &>/dev/null; then
                        _warn "$(yellow_echo '>>>' [$action] → [$section]) $MSG code: 1"
                        return 1
                    fi

                    original_ips="$(process_ip_addresses "$filtered_content" "$section")"

                    # 排序去重，格式化输出
                    #unique_ips="$(awk '{printf "%s%s", separator, $0; separator=" "}' < <(sort -ut . -k1,1n -k2,2n -k3,3n -k4,4n < <(tr ' ' '\n' <<<"$original_ips")))"
                    unique_ips="$(awk '{printf "%s%s", separator, $0; separator=" "}' < <(sort -ut . -k1,1n -k2,2n -k3,3n -k4,4n <<<${original_ips// /$'\n'}))"

                    # 计算原始IP列表、去重后IP列表中的值的数量
                    unique_count=$(wc -w <<<$unique_ips)
                    original_count=$(wc -w <<<$original_ips)

                    # 判断两个值是否相同，如果不同则打印差集
                    if [ "$original_count" -ne "$unique_count" ]; then
                        # 将IP列表转换为每行一个IP的格式
                        original_ips_list=$(sort <<<"${original_ips// /$'\n'}")
                        unique_ips_list=$(sort <<<"${unique_ips// /$'\n'}")
                        
                        # 打印差集（仅存在于 original_ips 的 IP）
                        diff=$(comm -23 <(echo "$original_ips_list") <(echo "$unique_ips_list"))
                        
                        MSG="$(red_echo ">>> 注意以下 IP 地址重复，请调整配置文件后重试 code: 6 .\n[$section]\n$diff")"
                        return 6
                    fi

                    #echo $unique_ips       # 排序后的IP
                    echo $original_ips     # 默认顺序
                else
                    # 如果是全局配置区块默认分行显示
                    awk '!/^#|^$|^::/{print $0}' <<< "$content"
                fi
                ;;
            environment)
                if [[ $section =~ ^[a-z] ]]; then
                    # 调用 print 功能验证区域信息，非正常打印错误信息
                    if ! listIniKeysValues $section print &>/dev/null; then
                        _warn "$(yellow_echo '>>>' [$action] → [$section]) $MSG"
                        return 1
                    else
                        # # 调用 print 功能，获取区域内 IP
                        unique_ips=$(listIniKeysValues $section print)

                        # 创建域名称变量，例如 zookeeper --> ${ZOOKEEPER[@]}
                        local index=0
                        for node in $unique_ips; do
                            eval "${section^^}[${index}]='$node'"
                            [[ $DEBUG == 0 ]] && echo "${section^^}[${index}]='$node'" >> $DEBUG_FILE
                            index=$((index + 1))
                        done
                    fi
                else
                    # 如果是全局配置区块默认分行显示
                    local index=0 IFS='='
                    while read -r key value; do
                        eval "$key=$value"
                        [[ $DEBUG == 0 ]] && echo "$key=$value" >> $DEBUG_FILE
                    done < <(awk '!/^#|^$|^::/{print $0}' <<< $content)
                fi
                ;;
            *)
                echo "Unknown action: $action"
                exit 1
                ;;
        esac
    fi
}

# 解析配置文件
# input  -->  listIniKeysValues zookeeper,kafka print
# out    -->  192.168.10.41 192.168.10.42 192.168.10.43
#             192.168.10.41 192.168.10.42 192.168.10.43
# input  -->  listIniKeysValues zookeeper,kafka environment
# out    -->  ZOOKEEPER[0]=192.168.10.41  ZOOKEEPER[1]=192.168.10.42  ZOOKEEPER[2]=192.168.10.43
#             KAFKA[0]=192.168.10.41      KAFKA[1]=192.168.10.42      KAFKA[2]=192.168.10.43
listIniKeysValuess() {
    local sections="$1"
    local action="${2:-print}"
    local inifile="${listIniKeys:-$basedir/config.ini}"

    if [ $# -lt 1 ] || [ ! -f "$inifile" ]; then
        echo "Usage: listIniKeysValues <section> [action]"
        exit 1
    fi

    IFS=,; for section in $sections; do
      # 获取指定区域内容
      content=$(awk -v section="[$section]" '$0 == section {flag=1; next}flag && /^\[/ {flag=0}flag {print}' "$inifile")

      if [ ! -n "$content" ]; then
          _warn ">>> 配置解析失败 ---> listIniKeysValues $inifile $section $action"
          return 1
      else
          case "$action" in
              print)
                  # 打印区域内容
                  if [[ $section =~ ^[a-z] ]]; then
                      # 过滤注释行,空行
                      filtered_content="$(awk '!/^#|^$|^::/{printf "%s%s", separator, $0; separator=","}' <<<$content)"

                      # 解析IP列表(对IP地址扩展)
                      if ! process_ip_addresses "$filtered_content" "$section" &>/dev/null; then
                          _warn "$(yellow_echo '>>>' [$action] → [$section]) $MSG code: 1"
                          return 1
                      fi

                      original_ips="$(process_ip_addresses "$filtered_content" "$section")"

                      # 排序去重，格式化输出
                      #unique_ips="$(awk '{printf "%s%s", separator, $0; separator=" "}' < <(sort -ut . -k1,1n -k2,2n -k3,3n -k4,4n < <(tr ' ' '\n' <<<"$original_ips")))"
                      unique_ips="$(awk '{printf "%s%s", separator, $0; separator=" "}' < <(sort -ut . -k1,1n -k2,2n -k3,3n -k4,4n <<<${original_ips// /$'\n'}))"

                      # 计算原始IP列表、去重后IP列表中的值的数量
                      unique_count=$(wc -w <<<$unique_ips)
                      original_count=$(wc -w <<<$original_ips)

                      # 判断两个值是否相同，如果不同则打印差集
                      if [ "$original_count" -ne "$unique_count" ]; then
                          # 将IP列表转换为每行一个IP的格式
                          original_ips_list=$(sort <<<"${original_ips// /$'\n'}")
                          unique_ips_list=$(sort <<<"${unique_ips// /$'\n'}")
                          
                          # 打印差集（仅存在于 original_ips 的 IP）
                          diff=$(comm -23 <(echo "$original_ips_list") <(echo "$unique_ips_list"))
                          
                          MSG="$(red_echo ">>> 注意以下 IP 地址重复，请调整配置文件后重试 code: 6 .\n[$section]\n$diff")"
                          return 6
                      fi

                      #echo $unique_ips       # 排序后的IP
                      echo $original_ips     # 默认顺序
                  else
                      # 如果是全局配置区块默认分行显示
                      awk '!/^#|^$|^::/{print $0}' <<< "$content"
                  fi
                  ;;
              environment)
                  if [[ $section =~ ^[a-z] ]]; then
                      # 调用 print 功能验证区域信息，非正常打印错误信息
                      if ! listIniKeysValues $section print &>/dev/null; then
                          _warn "$(yellow_echo '>>>' [$action] → [$section]) $MSG"
                          return 1
                      else
                          # # 调用 print 功能，获取区域内 IP
                          unique_ips=$(listIniKeysValues $section print)

                          # 创建域名称变量，例如 zookeeper --> ${ZOOKEEPER[@]}
                          local index=0
                          for node in $unique_ips; do
                              eval "${section^^}[${index}]='$node'"
                              [[ $DEBUG == 0 ]] && echo "${section^^}[${index}]='$node'" >> $DEBUG_FILE
                              index=$((index + 1))
                          done
                      fi
                  else
                      # 如果是全局配置区块默认分行显示
                      local index=0 IFS='='
                      while read -r key value; do
                          eval "$key=$value"
                          [[ $DEBUG == 0 ]] && echo "$key=$value" >> $DEBUG_FILE
                      done < <(awk '!/^#|^$|^::/{print $0}' <<< $content)
                  fi
                  ;;
              *)
                  echo "Unknown action: $action"
                  exit 1
                  ;;
          esac
      fi
    done
}

# 根据区域名称创建变量
function setup_environment_from_config () {
    local section_list= list=
    [ -f $basedir/config.ini ] || _err "没有读取到配置文件，请确认配置文件是否存在 $basedir/config.ini ."
    sections=$(awk -F'[][]' '/^\[.*\]$/ && $2 ~ /^[a-z]+$/{print $2}' $basedir/config.ini)
    for section in $sections; do
        if ! listIniKeysValues $section print &>/dev/null; then
            _warn "$(yellow_echo '>>>' [print] → [$section]) $MSG"
            return 1
        else
            # 解析区域名称变量，例如 ${ZOOKEEPER_NODE}
            eval "${section^^}_NODE='$(listIniKeysValues $section print)'"
            [[ $DEBUG == 0 ]] && echo "${section^^}_NODE='$(eval echo '$'"${section^^}_NODE")'" >> $DEBUG_FILE

            # 检测区域 IP 是否存在于 SERVERS
            for node in $(eval echo '$'"${section^^}_NODE"); do
                if ! contains "$node" SERVERS[@]; then
                    [ "$section" != 'vip' ] && _err "$(red_echo ">>> 以下IP 在环境变量 SERVERS[@] 内没有匹配到 .\n[${section}]\n$node")"
                fi
            done

            # 解析区域名称数组，例如 ${ZOOKEEPER[@]}
            listIniKeysValues $section environment
        fi
    done

    #sleep 1

    # # 解析所有区域名称变量
    # for section in $section_list; do
    #     listIniKeysValues $section environment
    # done
}

function environment() {
    local full=$1

    if [ -n "$full" ]; then
        truncate -s 0 $basedir/environment.txt

        # 配置 ENV_CONFIG 变量
        listIniKeysValues ENV_CONFIG environment

        setup_environment_from_config
    else
        # 配置 ENV_CONFIG 变量
        listIniKeysValues ENV_CONFIG environment
    fi
}

function main() {
    # 读取 ENV_CONFIG 变量
    environment

    # 获取本机 IP 地址
    LAN_IP_ADDRESS=$(get_lan_ip_address)
}

main

# https://lengyuewusheng.com/%E5%B8%B8%E7%94%A8shell%E5%91%BD%E4%BB%A4%E6%80%BB%E7%BB%93.html

# https://rare-gallery.com/xfsearch/alt/hanfu/
# https://spring.io/blog/2015/12/10/spring-boot-memory-performance

# https://www.zhihu.com/question/315793102/answer/1639340828

# ps aux | grep zookeeper| grep -v grep | awk '{print $11 "\t" $2 "\t" $6/1024"MB" }'


# https://spring.io/guides/gs/spring-boot-docker
# FROM openjdk:8-jdk-alpine
# RUN addgroup -S spring && adduser -S spring -G spring
# USER spring:spring
# ARG DEPENDENCY=target/dependency
# COPY ${DEPENDENCY}/BOOT-INF/lib /app/lib
# COPY ${DEPENDENCY}/META-INF /app/META-INF
# COPY ${DEPENDENCY}/BOOT-INF/classes /app
# ENTRYPOINT ["java","-cp","app:app/lib/*","hello.Application"]

# docker run -e "JAVA_TOOL_OPTIONS=-agentlib:jdwp=transport=dt_socket,address=5005,server=y,suspend=n" -p 8080:8080 -p 5005:5005 -t springio/gs-spring-boot-docker



# 函数多重调用用法 , 内层将错误信息包装给变量，然后 return (正常return 0 异常return 指定数字，将数字也加入到错误变量)；外层判断变量执行结果(需要的数据也可用变量传递,只判断命令执行结果)，如果异常 打印内层包装的变量


# while read line; do  $line; done < <(cat /etc/rc.d/rc.local | sed 's|start|stop|' |egrep -v '^$|^#')

# stop     bash < <(tac < <(sed "s|start|stop|g" < <(grep systemctl /etc/rc.d/rc.local)))
# start    bash < <(grep systemctl /etc/rc.d/rc.local)
# status   bash < <(tac < <(sed "s|start|status|g" < <(grep systemctl /etc/rc.d/rc.local)))

# stop     grep systemctl /etc/rc.d/rc.local | sed "s|start|stop|g" | tac

# mapred --daemon start historyserver


cat > /dev/null <<'Example'

# 变量匹配
var=192.168.1.100

# 从右边开始匹配，删除匹配部分返回剩下的 删除最短匹配
echo ${var%.*}    # 192.168.1
# 从右边开始匹配，删除匹配部分返回剩下的 删除最长匹配
echo ${var%%.*}   # 192

# 从左边开始匹配，删除匹配部分返回剩下的 删除最短匹配
echo ${var#*.}    # 168.1.100

# 从左边开始匹配，删除匹配部分返回剩下的 删除最长匹配
echo ${var##*.}   # 100


# 变量替换
url=https://www.baidu.com/www.baidu.com/1234.https

# 从左往右替换第一个 www 为 test
echo ${url/www/test}    # https://test.baidu.com/www.baidu.com/1234.https

# 从左往右替换所有 www 为 test
echo ${url//www/test}    # https://test.baidu.com/test.baidu.com/1234.https

# 删除变量中所有 www.
echo ${url//www.}    # https://baidu.com/baidu.com/1234.https

# 后缀匹配
echo ${url/%https/http}    # https://www.baidu.com/www.baidu.com/1234.http

# 前缀匹配
echo ${url/#https/http}    # http://www.baidu.com/www.baidu.com/1234.https



# 如果 var 没有定义则或空返回 1234
var= key=${var:-1234}; echo $key

# 如果 var 不为空则返回 1234, 如果为空或未定义返回空
var=abc key=${var:+1234}; echo $key

# 如果 var 没有定义则或空返回 1234，并且将 1234 赋值给 var, 否则返回原值
var= key=${var:=1234}; echo "key: $key var: $var"

# 如果 var 没有定义则或空返回 error var is null且返回状态值1,如果不为空返回原值
echo ${var:? error var is null}



# 获取以 H 开头的所有变量(普通/全局)，打印命令名称及值
for envs in  ${!H*}; do echo "$envs='${!envs}'";done

for envs in  ${!H*}; do echo "$envs='$(eval echo '$'${envs})'";done

vars=(${!H*})
for ((i=0;i<${#vars};i++)); do
    echo "${vars[i]}='${!vars[i]}'"
done

envs=H
eval echo '$'{\!$envs\*}
for key in  $(eval echo '$'{\!$envs\*}); do echo "$key='${!key}'";done

# 列出以 H 开头的全局变量
grep ^H < <(compgen -e)

compgen -e | grep ^H


for ((i=0; i<100; i++));do
    array[$i]=$i
done

for ((i=0; i<100; i++));do
    echo ${array[$i]}
done

Example