#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 加载区域变量
if ! environment full; then
    _err "环境变量加载异常，请检查配置文件 ."
fi

# 检测 HOSTS,SERVERS 数组个数
if [ "${#HOSTS[@]}" != "${#SERVERS[@]}" ]; then
    _err "HOSTS与SERVERS 数组个数不匹配 "HOSTS：${#HOSTS[@]}" SERVERS："${#SERVERS[@]}"."
fi

## 0. 配置服务管理
script_dir=/usr/local/.mango
[ -d "$script_dir" ] && rm -rf "$script_dir"
mkdir -p "$script_dir"

# 复制配置文件,脚本文件
cp -r ${basedir}/{config.ini,function.sh,status.sh,mango,info.sh} ${script_dir}
ln -sf ${script_dir}/mango /usr/local/bin/mango
chmod +x /usr/local/bin/mango

## 1. 生成环境变量文件
# 获取所有区域
#sections=$(awk -F'[][]' '/^\[.*\]$/ && $2 ~ /^[a-z]+$/{print $2}' $basedir/config.ini)
sections=$(awk -F'[][]' '/^\[.*\]$/{print $2}' $basedir/config.ini)

# 解析所有区域名称变量
[ -f "$MANGO_ENV" ] && rm -f "$MANGO_ENV"
for section in $sections; do
    if [[ $section =~ ^[a-z] ]]; then
        eval "${section^^}_NODE='$(listIniKeysValues $section print)'"
        [[ $DEBUG == 0 ]] && echo "${section^^}_NODE='$(eval echo '$'"${section^^}_NODE")'" >> ${MANGO_ENV:-/dev/null}

        # 调用 print 功能验证区域信息，非正常打印错误信息
        if ! listIniKeysValues $section print &>/dev/null; then
            _warn "$(yellow_echo '>>>' [$action] → [$section]) $MSG"
            return 1
        else
            # # 调用 print 功能，获取区域内 IP
            unique_ips=$(listIniKeysValues $section print)

            # 创建域名称变量，例如 zookeeper --> ${ZOOKEEPER[@]}
            index=0
            for node in $unique_ips; do
                eval "${section^^}[${index}]='$node'"
                [[ $DEBUG == 0 ]] && echo "${section^^}[${index}]='$node'" >> ${MANGO_ENV:-/dev/null}
                index=$((index + 1))
            done
        fi
    else
        content=$(awk -v section="[$section]" '$0 == section {flag=1; next}flag && /^\[/ {flag=0}flag {print}' $basedir/config.ini)
        index=0 IFS='='
        while read -r key value; do
            eval "$key=$value"
            [[ $DEBUG == 0 ]] && echo "$key=$value" >> ${MANGO_ENV:-/dev/null}
        done < <(awk '!/^#|^$|^::/{print $0}' <<< $content)
    fi
done


## 2.初始化
if contains "$LAN_IP_ADDRESS" SERVERS[@]; then
    title "📦 系统参数优化 ."
    if [ -f /etc/profile.d/hc-env.sh ]; then
        _info "本机已初始化，跳过此步骤 ."
    else
        ${basedir}/01init.sh
    fi
else
    _err "当前主机 [$LAN_IP_ADDRESS] 不存在于安装列表中 ."
    exit 2
fi

## 3.安装 beszel 监控
if contains "$LAN_IP_ADDRESS" SERVERS[@]; then
    title "📦 安装 beszel ."
    ${basedir}/00beszel.sh
fi

## 4. zookeeper
if contains "$LAN_IP_ADDRESS" ZOOKEEPER[@]; then
    title "📦 安装 zookeeper ."
    ${basedir}/02zookeeper.sh
fi

## 5. kafka
if contains "$LAN_IP_ADDRESS" KAFKA[@]; then
    title "📦 安装 kafka ."
    ${basedir}/03kafka.sh
fi

## 6. hadoop
if contains "$LAN_IP_ADDRESS" HADOOP[@]; then
    title "📦 安装 hadoop ."
    ${basedir}/04hadoop.sh

    if [ "$TSDB_TYPE" == "tdengine" ]; then
        title "📦 安装 tdengine-client."
        ${basedir}/19tdengine-client.sh
    fi
fi

## 7. hbase、opentsdb
if [ "$TSDB_TYPE" == "opentsdb" ]; then
    # 5. hbase
    if contains "$LAN_IP_ADDRESS" HBASE[@]; then
        title "📦 安装 hbase."
        ${basedir}/05hbase.sh
    fi

    # 5.1 opentsdb
    if contains "$LAN_IP_ADDRESS" OPENTSDB[@]; then
        title "📦 安装 opentsdb."
        ${basedir}/05opentsdb.sh
    fi
fi

## 8. flink
if contains "$LAN_IP_ADDRESS" FLINK[@]; then
    title "📦 安装 flink ."
    ${basedir}/05flink.sh

    if [ "$TSDB_TYPE" == "tdengine" ]; then
        title "📦 安装 tdengine-client."
        ${basedir}/19tdengine-client.sh
    fi
fi

## 9 . seatunnel
if contains "$LAN_IP_ADDRESS" SEATUNNEL[@]; then
    title "📦 安装 seatunnel ."
    ${basedir}/06seatunnel.sh
fi

## 10. doris
if contains "$LAN_IP_ADDRESS" DORIS[@]; then
    title "📦 安装 doris ."
    ${basedir}/07doris.sh
fi

# 8. mysql
if contains "$LAN_IP_ADDRESS" MYSQL[@]; then
    title "📦 安装 mysql ."
    if [ "${#MYSQL[@]}" -ge 2 ]; then
        ${basedir}/08mysql-ha.sh
    else
        ${basedir}/08mysql.sh
    fi
fi

# 9. nacos
if contains "$LAN_IP_ADDRESS" NACOS[@]; then
    if [ "${#NACOS[@]}" -ge 3 ]; then
        title "📦 安装 nacos ha ."
        ${basedir}/09nacos-ha.sh

        # 第二个节点安装 MySQL
        if [ "$LAN_IP_ADDRESS" == "${NACOS[1]}" ]; then
            green_echo ">>> 安装 mysql ."
            ${basedir}/08mysql.sh
        fi
    else
        title "📦 安装 nacos ."
        ${basedir}/09nacos.sh
    fi
fi

# 10. dolphinscheduler
if contains "$LAN_IP_ADDRESS" DOLPHINSCHEDULER[@]; then
    title "📦 安装 DolphinScheduler ."
    ${basedir}/10dolphinscheduler.sh
fi

# 11. fastdfs
if contains "$LAN_IP_ADDRESS" FASTDFS[@]; then
    title "📦 安装 FASTDFS ."
    ${basedir}/11fastdfs.sh
fi

# 12. mango
if contains "$LAN_IP_ADDRESS" MONGODB[@]; then
    title "📦 安装 mangodb."
    ${basedir}/12mongodb.sh
fi

# 13. minio
if contains "$LAN_IP_ADDRESS" MINIO[@]; then
    title "📦 安装 minio."
    ${basedir}/13minio.sh
fi

# 14. 帆软
if contains "$LAN_IP_ADDRESS" FANRUAN[@]; then
    title "📦 安装 帆软."
    ${basedir}/14fanruan.sh
fi

# 15. solr
if contains "$LAN_IP_ADDRESS" SOLR[@]; then
    title "📦 安装 solr."
    ${basedir}/15solr.sh
fi

# 16. redis
if contains "$LAN_IP_ADDRESS" REDIS[@]; then
    title "📦 安装 redis."
    ${basedir}/16redis.sh
fi

# 17. emqx
if contains "$LAN_IP_ADDRESS" EMQX[@]; then
    title "📦 安装 emqx."
    ${basedir}/17emqx.sh
fi

# 18 tdengine
if [ "$TSDB_TYPE" == "tdengine" ]; then
    if contains "$LAN_IP_ADDRESS" TDENGINE[@]; then
        title "📦 安装 tdengine ."
        ${basedir}/18tdengine.sh
    fi
fi

# 19 采集
if contains "$LAN_IP_ADDRESS" CAIJI[@]; then
    if [ "$TSDB_TYPE" == "tdengine" ]; then
        title "📦 安装 tdengine-client."
        ${basedir}/19tdengine-client.sh
    fi

    title "📦 安装 nginx ."
    ${basedir}/21caiji.sh

    # 配置数据采集、开机自启动脚本
    mkdir -p /home/mango
    install -m 755 ${basedir}/start.sh /home/mango/start.sh

    list='dc-api dc-mqtt calc-root'
    sed -i "s@^list=.*@list='$list'@" /home/mango/start.sh

    mkdir -p /script
    install -m 755 ${basedir}/start_mango.sh /script/start_mango.sh
fi

# 20. 微服务
if contains "$LAN_IP_ADDRESS" WFW[@]; then
    if [ "$TSDB_TYPE" == "tdengine" ]; then
        title "📦 安装 tdengine-client."
        ${basedir}/19tdengine-client.sh
    fi

    title "📦 安装 nginx libreoffice ."
    ${basedir}/20wfw.sh

    if [ -n "$VIP" ]; then
        green_echo ">>> 安装 keepalived ."
        ${basedir}/keepalived.sh nginx "$WFW_NODE" "$VIP"
    fi

    # 配置微服务、开机自启动脚本
    mkdir -p /home/mango
    install -m 755 ${basedir}/start.sh /home/mango/start.sh

    mkdir -p /script
    install -m 755 ${basedir}/start_mango.sh /script/start_mango.sh
fi


## 21. 配置服务开机自启动
# 创建服务管理脚本
install -m 755 /dev/null /etc/system.rc.local
cat <<EOF   > /etc/system.rc.local
#!/bin/bash

# 配置当前节点所有服务 启动/关闭 及开机自启动
active=\$1

systemctl daemon-reload

case \$active in
    start)
$(awk '/^systemctl/{printf "      %s $active %s\n", $1, $3}' /etc/rc.d/rc.local)$([[ "$(hostname)" =~ (web|wfw|caiji) ]] && echo -e '\n      (/script/start_mango.sh &)')
      ;;
    stop)
$(awk '/^systemctl/{printf "      %s $active %s\n", $1, $3}' /etc/rc.d/rc.local | tac)$([[ "$(hostname)" =~ (web|wfw|caiji) ]] && echo -e '\n      (/home/mango/start.sh $active all &)')
      ;;
    status)
      service_list="\$(awk '/^systemctl/{printf "%s ", \$3}' /etc/rc.d/rc.local)"

      # 定义输出格式（第一列固定40字符宽度）
      format="%-40s %-16s %-16s\n"
    
      echo '----------------------------------------------------------------------------------------------'
      printf "\033[33m\${format}\033[0m" "\${HOSTNAME}" "\$(hostname -i)" "\$(date +'%F %T')"
      # 格式化标题中文占用宽度*2，英文占用宽度*1
      printf "%-44s %-18s %-20s\n" "服务名称" "状态" "启动时间"
      echo '----------------------------------------------------------------------------------------------'

      for service in \$service_list; do
          status=\$(systemctl is-active \$service 2>/dev/null)
          source_start_time=\$(cut -d= -f2 < <(systemctl show \$service --property ExecMainStartTimestamp 2>/dev/null))
          [ -z "\$source_start_time" ] && start_time="N/A" || start_time=\$(date -d "\$source_start_time" +"%Y-%m-%d %H:%M:%S")
          # 带颜色格式化输出（注意ANSI颜色码长度不影响对齐）
          if [ "\$status" == "active" ]; then
              printf "\033[32m\${format}\033[0m" "\$service" "\$status" "\$start_time"
          else
              printf "\033[31m\${format}\033[0m" "\$service" "\$status" "\$start_time"
          fi
      done
      ;;
    restart)
$(awk '/^systemctl/{printf "      %s $active %s\n      sleep 10\n", $1, $3}' /etc/rc.d/rc.local)
      ;;
    *)
      echo "\$0 {stop|start|status|restart}"
      ;;
esac
EOF

# 创建开机自启动服务
cat <<'EOF'   > /etc/systemd/system/start_mango.service
[Unit]
Description=Start Mango Script at Boot
After=network.target

[Service]
Type=simple
Environment=SERVICE_MODE=systemctl
ExecStart=/etc/system.rc.local start

TimeoutSec=infinity
RemainAfterExit=yes
GuessMainPID=no

[Install]
WantedBy=multi-user.target
EOF

systemctl daemon-reload
systemctl enable start_mango.service &>/dev/null


## 22. 初始化集群
if [ "$LAN_IP_ADDRESS" == "${TDENGINE[0]}" ]; then
    INIT_FILE="$PACKAGE_DIR/script/cluster/init.txt"

    title "📦 初始化集群 ."
    printf "\n\033[034;1m%-32s %s %-32s\033[0m\n" "" "[${HADOOP[0]}] 节点执行" | tr ' ' "-"
    printf "%-4s %s\n" "" "# 初始化 Hadoop 集群 ."
    printf "\033[032;1m%-4s %s\033[0m\n\n" "" "bash $PACKAGE_DIR/script/cluster/init-hadoop.sh"
    printf "%-4s %s\n" "" "# 初始化 Doris 集群 ."
    printf "\033[032;1m%-4s %s\033[0m\n\n" "" "bash $PACKAGE_DIR/script/cluster/init-doris.sh"

    echo -e "# 初始化集群 $(date '+%F %T') ." > $INIT_FILE
    echo -e "\n# 初始化 Hadoop 集群 ." >> $INIT_FILE
    echo "bash $PACKAGE_DIR/script/cluster/init-hadoop.sh" >> $INIT_FILE
    echo -e "\n# 初始化 Doris 集群 ." >> $INIT_FILE
    echo "bash $PACKAGE_DIR/script/cluster/init-doris.sh" >> $INIT_FILE

    if [ "$TSDB_TYPE" == "tdengine" ]; then
        printf "%-4s %s\n" "" "# 初始化 TDengine 集群 ."
        printf "\033[032;1m%-4s %s\033[0m\n\n" "" "bash $PACKAGE_DIR/script/cluster/init-tdengine.sh"
        echo -e "\n# 初始化 TDengine 集群 ." >> $INIT_FILE
        echo "bash $PACKAGE_DIR/script/cluster/init-tdengine.sh" >> $INIT_FILE
    elif [ "$TSDB_TYPE" == "opentsdb" ]; then
        printf "%-4s %s\n" "" "# 初始化 OpenTSDB 集群."
        printf "\033[032;1m%-4s %s\033[0m\n\n" "" "bash $PACKAGE_DIR/script/cluster/init-opentsdb.sh"
        echo -e "\n# 初始化 OpenTSDB 集群 ." >> $INIT_FILE
        echo "bash $PACKAGE_DIR/script/cluster/init-opentsdb.sh" >> $INIT_FILE
    fi

    if [ "${#MYSQL[@]}" == 2 ]; then
        printf "%-4s %s\n" "" "# 初始化 MySQL 集群 ."
        printf "\033[032;1m%-4s %s\033[0m\n" "" "bash $PACKAGE_DIR/script/cluster/init-mysql.sh"
        echo -e "\n# 初始化 MySQL 集群 ." >> $INIT_FILE
        echo "bash $PACKAGE_DIR/script/cluster/init-mysql.sh" >> $INIT_FILE
    fi
    printf "\033[034;1m%-90s\033[0m\n" | tr ' ' "-"
fi