#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 安装 keepalived
if ! rpm -q keepalived --quiet; then
    _log "安装 keepalived ."
    dnf install -y keepalived &>/dev/null
else
    _info "本机已安装 keepalived ."
    exit 0
fi

# 接收参数，转换为数组
app_name=$1
ip_array=($2)
keep_vip=$3

# 获取当前节点下标
position=0
for i in "${!ip_array[@]}"; do
    if [[ "${ip_array[$i]}" == "$LAN_IP_ADDRESS" ]]; then
        break
    fi
    position=$((i+1))
done

# 如果是数组第一个为主节点
if [ "$position" == 0 ]; then
    state=MASTER
    weight=-15
else
    state=BACKUP
    weight=-6
fi

priority=$((100-${position}*10))
router_id=$LAN_IP_ADDRESS
interface=$(ip route | awk "/$LAN_IP_ADDRESS/{print \$3}")
virtual_router_id=${keep_vip##*.}
_info "state: $state  priority: $priority  router_id: $router_id  interface: $interface  virtual_router_id: $virtual_router_id"


# 创建 keepalived 配置
cat > /etc/keepalived/keepalived.conf <<EOF
! Configuration File for keepalived

global_defs {
   router_id ${router_id}
   script_user root
   enable_script_security
}

vrrp_script ${app_name} {
    script "/etc/keepalived/check_${app_name}.sh"
    interval 2
    weight ${weight}
}

vrrp_instance VI_1 {
    state ${state}
    interface ${interface}
    virtual_router_id ${virtual_router_id}
    priority ${priority}
    advert_int 1
    authentication {
        auth_type PASS
        auth_pass ${app_name}_pass_${virtual_router_id}
    }

    virtual_ipaddress {
        $keep_vip
    }

    track_script {
        ${app_name}
    }
}
EOF

# nacos 状态检测
if [ "$app_name" == 'nacos' ]; then
    cat > /etc/keepalived/check_${app_name}.sh <<-'EOF'
	#!/bin/sh

	status=$(curl -s http://127.0.0.1:8848/nacos/actuator/health)

	if [[ "$status" != '{"status":"UP"}' ]]; then
	    exit 1
	fi
	EOF
    chmod +x /etc/keepalived/check_${app_name}.sh
fi

# nginx 状态检测
if [ "$app_name" == 'nginx' ]; then
    cat > /etc/keepalived/check_${app_name}.sh <<-'EOF'
	#!/bin/sh

	PROCN=`wc -l < <(ps -C nginx --no-header)`
	if [ $PROCN -eq 0 ]; then
	    exit 1
	fi
	EOF
    chmod +x /etc/keepalived/check_${app_name}.sh
fi

# 配置守护进程
grep -q 'Restart=always' /usr/lib/systemd/system/keepalived.service || \
    sed -i '/Type=notify/a\Restart=always\nRestartSec=3' /usr/lib/systemd/system/keepalived.service

_log "配置系统服务 ."
systemctl daemon-reload

add_service_to_rc_local keepalived
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl restart keepalived
_log "启动 keepalived 服务."