#!/bin/bash

if [ -d /usr/local/.mango ]; then
    . /usr/local/.mango/function.sh
    . /usr/local/.mango/environment.txt
else
    echo "读取环境变量失败 ."
    exit 1
fi

# 函数：使用 yq 修改 YAML 文件的指定参数
# 用法：yq_update <文件路径> <参数名> <值>
yq_update() {
    local file_path="$1"
    local param="$2"
    local value="$3"

    # 检查文件是否存在
    if [[ ! -f "$file_path" ]]; then
        echo "错误：文件 '$file_path' 不存在！" >&2
        return 1
    fi

    # 检查 yq 是否安装
    if ! command -v yq &> /dev/null; then
        [ -f ./yq ] && { mv yq /usr/local/bin/yq; chmod +x /usr/local/bin/yq; yq shell-completion bash > /etc/bash_completion.d/yq; rm -f ./yq; }
    fi

    # 执行 yq 修改
    #yq -i ".${param} = \"${value}\"" "$file_path"
    yq -i "(. | select(.${param})).${param} = \"${value}\"" "$file_path"

    # 检查是否成功
    if [[ $? -eq 0 ]]; then
        echo "已更新：${param} = ${value} (文件: ${file_path})"
    else
        echo "错误：修改失败！" >&2
        return 1
    fi
}


# ai.yml
yq_update "DEFAULT_GROUP/ai.yml"           "application.intent-url"      "http://${XH[0]}:8000"
yq_update "DEFAULT_GROUP/ai.yml"           "application.solr-url"        "http://${SOLR}:10010/solr/new_core"

# application.yml
yq_update "DEFAULT_GROUP/application.yml" "spring.data.redis.host" "${REDIS}"
yq_update "DEFAULT_GROUP/application.yml" "spring.cloud.nacos.discovery.server-addr" "127.0.0.1:8848"
yq_update "DEFAULT_GROUP/application.yml" "spring.kafka.consumer.bootstrap-servers" "$(lookup_ips_with_port 9092 ${KAFKA[@]})"
yq_update "DEFAULT_GROUP/application.yml" "spring.cloud.stream.kafka.binder.brokers" "$(lookup_ip_addresses ${KAFKA[@]})"
yq_update "DEFAULT_GROUP/application.yml" "spring.cloud.stream.kafka.binder.zk-nodes" "$(lookup_ip_addresses ${ZOOKEEPER[@]})"
yq_update "DEFAULT_GROUP/application.yml" "mango.database.host" "${MYSQL[0]}"
yq_update "DEFAULT_GROUP/application.yml" "mango.tsdb.tdengine.datasource.url" "jdbc:TAOS://${TSDB[0]}:6030/mango"

# auth-server-prod.yaml
yq_update  "DEFAULT_GROUP/auth-server-prod.yaml" "spring.datasource.url" "jdbc:mysql://mysql:3306/blueberry_auth"
yq_update  "DEFAULT_GROUP/auth-server-prod.yaml" "spring.data.redis.host" "redis"
yq_update  "DEFAULT_GROUP/auth-server-prod.yaml" "spring.kafka.bootstrap-servers" "$(lookup_ips_with_port 9092 ${KAFKA[@]})"

# backstage.yml
yq_update  "DEFAULT_GROUP/backstage.yml" "fdfs.tracker-list.0" "${FASTDFS[0]}:22122"
yq_update  "DEFAULT_GROUP/backstage.yml" "fdfs.tracker-list.1" "${FASTDFS[1]}:22122"
yq_update  "DEFAULT_GROUP/backstage.yml" "fdfs.port" 80
yq_update  "DEFAULT_GROUP/backstage.yml" "fdfs.web-server-url" "http://$VIP"
yq_update  "DEFAULT_GROUP/backstage.yml" "gateway.address" "http://localhost:8080"

# base.yml
yq_update  "DEFAULT_GROUP/base.yml" "application.minio.endpoint" "http://${MINIO}:9000"
yq_update  "DEFAULT_GROUP/base.yml" "fdfs.tracker-list.0" "${FASTDFS[0]}:22122"
yq_update  "DEFAULT_GROUP/base.yml" "fdfs.tracker-list.1" "${FASTDFS[1]}:22122"
yq_update  "DEFAULT_GROUP/base.yml" "fdfs.port" 80
yq_update  "DEFAULT_GROUP/base.yml" "fdfs.web-server-url" "http://$VIP"

# calc-root
yq_update  "DEFAULT_GROUP/calc-root.yml" "kafka.url"  "$(lookup_ips_with_port 9092 ${KAFKA[@]})"
yq_update  "DEFAULT_GROUP/calc-root.yml" "zookeeper.url"     "$(lookup_ips_with_port 2181 ${ZOOKEEPER[@]})"

# dataservice-server-prod.yaml
yq_update  "DEFAULT_GROUP/dataservice-server-prod.yaml"  "spring.datasource.url" "jdbc:mysql://mysql:3306/blueberry_dataservice"

# datasource-server-prod.yaml
yq_update  "DEFAULT_GROUP/datasource-server-prod.yaml"  "spring.datasource.url" "jdbc:mysql://mysql:3306/blueberry_datasource"
yq_update  "DEFAULT_GROUP/datasource-server-prod.yaml"  "blue.datasource.store.doris.host" "${DORIS[0]}"

# 
# datasync-server-prod.yaml
yq_update  "DEFAULT_GROUP/datasync-server-prod.yaml"  "spring.datasource.url" "jdbc:mysql://mysql:3306/blueberry_datasource_cdc?rewriteBatchedStatements=true&useSSL=true"
yq_update  "DEFAULT_GROUP/datasync-server-prod.yaml"  "x-doris.host"          "${DORIS[0]}"
yq_update  "DEFAULT_GROUP/datasync-server-prod.yaml"  "dolphin.post"          "http://${DOLPHINSCHEDULER[0]}:12345/dolphinscheduler"

# dc-api.yml
yq_update  "DEFAULT_GROUP/dc-api.yml"   "spring.data.mongodb.uri"  "mongodb://admin:zaq1%40WSX@${MONGODB}:27017/datacollect"

# dc-mqtt.yml
yq_update  "DEFAULT_GROUP/dc-mqtt.yml"  "spring.data.mongodb.uri"  "mongodb://admin:zaq1%40WSX@${MONGODB}:27017/datacollect"

# dc-record.yml
yq_update  "DEFAULT_GROUP/dc-record.yml"  "spring.data.mongodb.uri"  "mongodb://admin:zaq1%40WSX@${MONGODB}:27017/datacollect"

# diagnosis.yml
yq_update  "DEFAULT_GROUP/diagnosis.yml"  "spring.data.solr.host"  "http://${SOLR}:10010/solr"

# loong.yml
yq_update  "DEFAULT_GROUP/loong.yml"  "spring.mqtt.url"              "tcp://${EMQX}:1883"
yq_update  "DEFAULT_GROUP/loong.yml"  "application.airflow-http-api" "http://${LM}:8083/api/experimental/dags/data_quality_recalc/dag_runs"
yq_update  "DEFAULT_GROUP/loong.yml"  "application.fastdfs-url"      "http://${VIP}"
yq_update  "DEFAULT_GROUP/loong.yml"  "application.voice-solr-url"   "http://${SOLR}:10010/solr/new_core"
yq_update  "DEFAULT_GROUP/loong.yml"  "application.report.url"       "http://127.0.0.1/WebReport/ReportServer?op=fs_load&cmd=sso&fr_username=%s&fr_password=%s"
yq_update  "DEFAULT_GROUP/loong.yml"  "ffmpeg.path"                  "/usr/local/bin/ffmpeg"


# panel-server-prod.yaml
yq_update  "DEFAULT_GROUP/panel-server-prod.yaml"  "spring.datasource.url"            "jdbc:mysql://mysql:3306/blueberry_panel"
yq_update  "DEFAULT_GROUP/panel-server-prod.yaml"  "spring.kafka.bootstrap-servers"   "$(lookup_ips_with_port 9092 ${KAFKA[@]})"


# synctask-server-prod.yaml
yq_update  "DEFAULT_GROUP/synctask-server-prod.yaml"  "spring.datasource.url"     "jdbc:mysql://mysql:3306/blueberry_datasource_cdc"
yq_update "DEFAULT_GROUP/synctask-server-prod.yaml"   "spring.data.redis.host"    "redis"
yq_update "DEFAULT_GROUP/synctask-server-prod.yaml"   "dolphin.post"              "http://${DOLPHINSCHEDULER[0]}:12345/dolphinscheduler"
yq_update "DEFAULT_GROUP/synctask-server-prod.yaml"   "x-doris.host"              "${DORIS[0]}"

# system-server-prod.yaml
yq_update  "DEFAULT_GROUP/system-server-prod.yaml"  "spring.datasource.url"     "jdbc:mysql://mysql:3306/blueberry_system"
yq_update "DEFAULT_GROUP/system-server-prod.yaml"   "spring.data.redis.host"    "redis"
yq_update "DEFAULT_GROUP/system-server-prod.yaml"   "spring.kafka.bootstrap-servers"   "$(lookup_ips_with_port 9092 ${KAFKA[@]})"

# uaa.yml
yq_update "DEFAULT_GROUP/uaa.yml"   "application.ht-sid-full-url"              "http://${MINIO}:5889/auth/generate/sid/private"
yq_update "DEFAULT_GROUP/uaa.yml"   "application.social-login.wechat-web.relay-authorize-url"   "http://${VIP}/static/weixin/authorize.html"

# voice.yml
yq_update "DEFAULT_GROUP/voice.yml"   "spring.data.solr.host"                  "http://${SOLR}:10010/solr"
yq_update "DEFAULT_GROUP/voice.yml"   "application.voice-solr-url"             "http://${SOLR}:10010/solr/new_core"