#!/bin/bash

set -x

BOOTSTRAP_SERVER="$(hostname -f):9092"  # 替换为你的 Kafka Broker 地址
TOPIC_NAME="__consumer_offsets"    # 使用一个内置的 Topic 进行检查

# 使用 kafka-broker-api-versions.sh 检查 Broker 是否在线
kafka-broker-api-versions.sh --bootstrap-server $BOOTSTRAP_SERVER > /dev/null 2>&1
if [ $? -ne 0 ]; then
    echo "Kafka Broker is not responding. Restarting Kafka service..."
    systemctl restart kafka
    exit 1
fi

# 使用 kafka-topics.sh 检查 Topic 分区状态
TOPIC_STATUS=$(kafka-topics.sh --bootstrap-server $BOOTSTRAP_SERVER --describe --topic $TOPIC_NAME 2>&1)
if [[ $TOPIC_STATUS == *"ERROR"* ]]; then
    echo "Kafka Topic is not healthy. Restarting Kafka service..."
    systemctl restart kafka
    exit 1
fi

echo "Kafka Broker and Topic are healthy."
#exit 0

kafka-broker-api-versions.sh --bootstrap-server $BOOTSTRAP_SERVER | grep -oP '\b([0-9]+\.){3}[0-9]+:9092\b'
