#!/bin/bash

trap '' PIPE  # 忽略 SIGPIPE 信号

if [ -d "/usr/local/.mango" ]; then
    . /usr/local/.mango/function.sh
    . /usr/local/.mango/environment.txt
fi

# listIniKeysValues zookeeper environment
# listIniKeysValues kafka environment
# listIniKeysValues nacos environment
# listIniKeysValues tdengine environment

list=(
    ${ZOOKEEPER[0]}:2181
    ${KAFKA[0]}:9092
    ${NACOS[0]}:9848
    ${TDENGINE[0]}:6030
)

repeat() {
    while true; do sleep 3; $@ && return; done
}

log_message() {
    echo "[$(date +"%Y-%m-%d %H:%M:%S")] $1"
}

# 等待依赖启动
for target in ${list[@]}; do
    IP=${target%%:*}
    PORT=${target##*:}
    log_message "准备检测依赖 ${target} ."
    repeat nc -vw 1 ${IP} -z ${PORT} &>/dev/null
    log_message "依赖检测通过 ${target} ."
done

# 根据 tsdb 类型检查依赖
if [ "$TSDB_TYPE" == "tdengine" ]; then
    # 检查 tdengine 是否准备就绪
    TDENGINE_HOST=$(awk -v "node=${TDENGINE[0]}" 'index($1, node) {print $2}' /etc/hosts)
    until [ "$TEDENGIN_STATE" == 'ready' ]; do
        log_message "等待 tdengine 服务启动就绪 ${TDENGINE[0]} ."
        TEDENGIN_STATE=$(taos -h ${TDENGINE[0]} -s "show mnodes;" | awk -v node="${TDENGINE_HOST}:6030" '{if ($3 == node) print $7}')
        sleep 2
    done
    log_message "服务 tdengine 启动就绪 ."
elif [ "$TSDB_TYPE" == "opentsdb" ]; then
    # 检查 opentsdb 是否准备就绪
    CHECK_POINT='{"start": 1745683200, "queries": [{"metric": "check.tsdb", "aggregator": "avg", "tags": {"name": "opentsdb"}}]}'
    until curl -s -X POST -H "Content-Type: application/json" http://${OPENTSDB[0]}:14242/api/query -d "$CHECK_POINT" | grep -q 1234567890; do
        log_message "等待 opentsdb 服务启动就绪 ${OPENTSDB[0]}."
        sleep 2
    done
    log_message "服务 opentsdb 启动就绪 ."
fi

log_message "开始启动微服务 ."
/home/mango/start.sh start all
log_message "微服务启动完成 ."
log_message "脚本执行完毕 ."